@@ -431,7 +431,6 @@ void mgls_prepare1d(mglData *y, mglData *y1, mglData *y2, mglData *x1, mglData *
 	if(y)	y->Create(n,3);
 	if(x1)	x1->Create(n);		if(x2)	x2->Create(n);
 	if(y1)	y1->Create(n);		if(y2)	y2->Create(n);
-#pragma omp parallel for
 	for(long i=0;i<n;i++)
 	{
 		double xx = i/(n-1.);
@@ -453,7 +452,6 @@ void mgls_prepare2d(mglData *a, mglData *b, mglData *v)
 	long n=50,m=40;
 	if(a)	a->Create(n,m);		if(b)	b->Create(n,m);
 	if(v)	{	v->Create(9);	v->Fill(-1,1);	}
-#pragma omp parallel for collapse(2)
 	for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 	{
 		double x = i/(n-1.), y = j/(m-1.);
@@ -467,7 +465,6 @@ void mgls_prepare3d(mglData *a, mglData *b)
 {
 	long n=61,m=50,l=40;
 	if(a)	a->Create(n,m,l);		if(b)	b->Create(n,m,l);
-#pragma omp parallel for collapse(3)
 	for(long k=0;k<l;k++)	for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 	{
 		double x=2*i/(n-1.)-1, y=2*j/(m-1.)-1, z=2*k/(l-1.)-1;
@@ -481,7 +478,6 @@ void mgls_prepare2v(mglData *a, mglData *b)
 {
 	long n=20,m=30;
 	if(a)	a->Create(n,m);		if(b)	b->Create(n,m);
-#pragma omp parallel for collapse(2)
 	for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 	{
 		double x=i/(n-1.), y=j/(m-1.);
@@ -494,9 +490,9 @@ void mgls_prepare2v(mglData *a, mglData *b)
 void mgls_prepare3v(mglData *ex, mglData *ey, mglData *ez)
 {
 	long n=10;
+	double z0=0.3;
 	if(!ex || !ey || !ez)	return;
 	ex->Create(n,n,n);	ey->Create(n,n,n);	ez->Create(n,n,n);
-#pragma omp parallel for collapse(3)
 	for(long k=0;k<n;k++)	for(long j=0;j<n;j++)	for(long i=0;i<n;i++)
 	{
 		double x=2*i/(n-1.)-1, y=2*j/(n-1.)-1, z=2*k/(n-1.)-1;
@@ -508,11 +504,11 @@ void mgls_prepare3v(mglData *ex, mglData *ey, mglData *ez)
 /*		ex->a[i0]=3*z;
 		ey->a[i0]=1;
 		ez->a[i0]=-3*x;*/
-		double r1 = pow(x*x+y*y+(z-0.3)*(z-0.3)+0.03,1.5);
-		double r2 = pow(x*x+y*y+(z+0.3)*(z+0.3)+0.03,1.5);
+		double r1 = pow(x*x+y*y+(z-z0)*(z-z0)+0.03,1.5);
+		double r2 = pow(x*x+y*y+(z+z0)*(z+z0)+0.03,1.5);
 		ex->a[i0]=0.2*x/r1 - 0.2*x/r2;
 		ey->a[i0]=0.2*y/r1 - 0.2*y/r2;
-		ez->a[i0]=0.2*(z-0.3)/r1 - 0.2*(z+0.3)/r2;
+		ez->a[i0]=0.2*(z-z0)/r1 - 0.2*(z+z0)/r2;
 	}
 }
 //-----------------------------------------------------------------------------
