@@ -28,12 +28,10 @@
 
 #if MGL_HAVE_PTHREAD
 #include <pthread.h>
-#define MGL_PUSH(a,v,m)		{pthread_mutex_lock(&m);	a.push_back(v);	pthread_mutex_unlock(&m);}
+#define MGL_PUSH(a,v,m)	{pthread_mutex_lock(&m);	a.push_back(v);	pthread_mutex_unlock(&m);}
 #else
-#define MGL_PUSH(a,v,m)		a.push_back(v);
+#define MGL_PUSH(a,v,m)	a.push_back(v);
 #endif
-
-#define MGL_TO_WCS(str,code)	if(str){size_t s=mbstowcs(0,str,0); wchar_t *wcs=new wchar_t[s+1]; mbstowcs(wcs,str,s); wcs[s]=0; code; delete []wcs;}
 //-----------------------------------------------------------------------------
 inline mreal mgl_d(mreal v,mreal v1,mreal v2) { return v2!=v1?(v-v1)/(v2-v1):NAN; }
 //-----------------------------------------------------------------------------
@@ -41,6 +39,79 @@ mglPoint GetX(HCDT x, int i, int j, int k=0);
 mglPoint GetY(HCDT y, int i, int j, int k=0);
 mglPoint GetZ(HCDT z, int i, int j, int k=0);
 //-----------------------------------------------------------------------------
+/// Class for replacement of std::vector
+// NOTE memcpy is used --> no memory allocation in T
+template <class T> class mglStack
+{
+	T** dat;
+	size_t pb;	///< size of buffer (real size is 2^pb == 1L<<pb)
+	size_t np;	///< allocated pointers
+	size_t m;	///< used pointers (allocated size is m*nb)
+	size_t n;	///< used cells
+	void *mutex;
+public:
+	mglStack(const mglStack<T> &st)
+	{
+		np=st.np;	dat = (T**)malloc(np*sizeof(T*));
+		pb=st.pb;	m=n=0;	reserve(st.n);
+		for(size_t i=0;i<m;i++)	memcpy(dat[i],st.dat[i],(1L<<pb)*sizeof(T));
+		n=st.n;		mutex = 0;
+	}
+	mglStack(size_t Pbuf=10)
+	{	np=16;	pb=Pbuf;	dat = (T**)malloc(np*sizeof(T*));
+		dat[0] = new T[1L<<pb];	n=0;	m=1;	mutex = 0;	}
+	~mglStack()	{	clear();	delete [](dat[0]);	free(dat);	}
+	inline void set_mutex(void *m)	{	mutex = m;	}
+	void reserve(size_t num)
+	{
+		num+=n;
+		if(num>(m<<pb))
+		{
+			num = 1+ (num>>pb);
+			if(num>np)
+			{	dat = (T**)realloc(dat, num*sizeof(T*));	np=num;	}
+			for(size_t i=m;i<num;i++)	dat[i] = new T[1L<<pb];
+			m = num;
+		}
+	}
+	void clear()
+	{
+		if(mutex)
+		{
+#if MGL_HAVE_PTHREAD
+			pthread_mutex_lock((pthread_mutex_t *)mutex);
+#elif MGL_HAVE_OMP
+			omp_set_lock((omp_lock_t *)mutex);
+#endif
+		}
+		for(size_t i=0;i<m;i++)	delete [](dat[i]);
+		dat[0] = new T[1L<<pb];	n=0;	m=1;
+		if(mutex)
+		{
+#if MGL_HAVE_PTHREAD
+			pthread_mutex_unlock((pthread_mutex_t *)mutex);
+#elif MGL_HAVE_OMP
+			omp_unset_lock((omp_lock_t *)mutex);
+#endif
+		}
+	}
+	T &operator[](size_t i)	{	register size_t d=i>>pb;	return dat[d][i-(d<<pb)];	}
+	const T &operator[](size_t i)	const	{	register size_t d=i>>pb;	return dat[d][i-(d<<pb)];	}
+	void push_back(const T &t)
+	{
+		if(n>=(m<<pb))	reserve(1);
+		register size_t d=n>>pb;
+		dat[d][n-(d<<pb)] = t;	n++;
+	}
+	size_t size()	const	{	return n;	}
+	const mglStack<T> &operator=(const mglStack<T> &st)
+	{
+		pb=st.pb;	clear();	reserve(st.n);
+		for(size_t i=0;i<st.m && i<m;i++)	memcpy(dat[i],st.dat[i],(1L<<pb)*sizeof(T));
+		n = st.n;	return st;
+	}
+};
+//-----------------------------------------------------------------------------
 /// Structure for transformation matrix
 struct MGL_EXPORT mglMatrix
 {
@@ -48,16 +119,20 @@ struct MGL_EXPORT mglMatrix
 	mreal x,y,z,pf;
 	bool norot;	// flag to disable pnts rotation
 	mglMatrix()	{	memset(this,0,sizeof(mglMatrix));	clear();	}
+	mglMatrix(const mglMatrix &aa) : x(aa.x),y(aa.y),z(aa.z),pf(aa.pf),norot(aa.norot) 	{	memcpy(b,aa.b,9*sizeof(mreal));	}
+#if MGL_HAVE_RVAL
+	mglMatrix(mglMatrix &&aa) : x(aa.x),y(aa.y),z(aa.z),pf(aa.pf),norot(aa.norot) 	{	memcpy(b,aa.b,9*sizeof(mreal));	}
+#endif
 	void Rotate(mreal tetz,mreal tetx,mreal tety);
 	void RotateN(mreal Tet,mreal x,mreal y,mreal z);
 	inline void clear()	{	x=y=z=pf=0;	memset(b,0,9*sizeof(mreal));	b[0]=b[4]=b[8]=1;	norot=false;	}
-	inline mglMatrix &operator=(const mglMatrix &a)
-	{	x=a.x;	y=a.y;	z=a.z;	pf=a.pf;	memcpy(b,a.b,9*sizeof(mreal));	norot=false;	return *this;	}
+	inline const mglMatrix &operator=(const mglMatrix &a)
+	{	x=a.x;	y=a.y;	z=a.z;	pf=a.pf;	memcpy(b,a.b,9*sizeof(mreal));	norot=false;	return a;	}
 };
 inline bool operator==(const mglMatrix &a, const mglMatrix &b)
-{	return b.x==a.x&&b.y==a.y&&b.z==a.z&&b.pf==a.pf&&!memcmp(b.b,a.b,9*sizeof(mreal));}
+{	return ((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z)+(a.pf-b.pf)*(a.pf-b.pf)==0)&&!memcmp(b.b,a.b,9*sizeof(mreal));}
 inline bool operator!=(const mglMatrix &a, const mglMatrix &b)
-{	return b.x!=a.x||b.y!=a.y||b.z!=a.z||b.pf!=a.pf||memcmp(b.b,a.b,9*sizeof(mreal));	}
+{	return ((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z)+(a.pf-b.pf)*(a.pf-b.pf)!=0)||memcmp(b.b,a.b,9*sizeof(mreal));	}
 //-----------------------------------------------------------------------------
 /// Structure for simplest primitives
 struct MGL_EXPORT mglPrim	// NOTE: use float for reducing memory size
@@ -79,7 +154,12 @@ struct MGL_EXPORT mglPrim	// NOTE: use float for reducing memory size
 		};
 		uint64_t m;
 	};
-	mglPrim(int t=0)	{	n1=n2=n3=n4=id=0;	z=s=w=p=0;	type=t;	}
+	mglPrim(int t=0):n1(0),n2(0),n3(0),n4(0),type(t),angl(0),id(0),z(0),w(0),m(0)	{}
+	mglPrim(const mglPrim &aa) : n1(aa.n1),n2(aa.n2),n3(aa.n3),n4(aa.n4),type(aa.type),angl(aa.angl),id(aa.id),z(aa.z),w(aa.w),m(aa.m) 	{}
+#if MGL_HAVE_RVAL
+	mglPrim(mglPrim &&aa) : n1(aa.n1),n2(aa.n2),n3(aa.n3),n4(aa.n4),type(aa.type),angl(aa.angl),id(aa.id),z(aa.z),w(aa.w),m(aa.m) 	{}
+#endif
+	const mglPrim &operator=(const mglPrim &aa)	{	memcpy(this, &aa, sizeof(mglPrim));	return aa;	}
 };
 bool operator<(const mglPrim &a,const mglPrim &b);
 bool operator>(const mglPrim &a,const mglPrim &b);
@@ -90,7 +170,12 @@ struct MGL_EXPORT mglGroup
 	std::vector<long> p;	///< list of primitives (not filled!!!)
 	int Id;					///< Current list of primitives
 	std::string Lbl;		///< Group label
-	mglGroup(const char *lbl="", int id=0)	{	Lbl=lbl;	Id=id;	}
+	mglGroup(const char *lbl="", int id=0) : Id(id), Lbl(lbl)	{}
+	mglGroup(const mglGroup &aa) : p(aa.p),Id(aa.Id),Lbl(aa.Lbl)	{}
+#if MGL_HAVE_RVAL
+	mglGroup(mglGroup &&aa) : p(aa.p),Id(aa.Id),Lbl(aa.Lbl)	{}
+#endif
+	inline const mglGroup &operator=(const mglGroup &aa)	{	Lbl = aa.Lbl;	Id = aa.Id;	p = aa.p;	return aa;	}
 };
 //-----------------------------------------------------------------------------
 /// Structure for text label
@@ -99,8 +184,13 @@ struct MGL_EXPORT mglText
 	std::wstring text;
 	std::string stl;
 	mreal val;
-	mglText(const wchar_t *txt=L"", const char *fnt="", mreal v=0)	{	text=txt;	stl=fnt;	val=v;	}
-	mglText(const std::wstring &txt, mreal v=0)	{	text=txt;	val=v;	}
+	mglText(const wchar_t *txt=L"", const char *fnt="", mreal v=0) : text(txt), stl(fnt), val(v) {}
+	mglText(const std::wstring &txt, mreal v=0): text(txt), val(v)	{}
+	mglText(const mglText &aa) : text(aa.text),stl(aa.stl),val(aa.val)	{}
+#if MGL_HAVE_RVAL
+	mglText(mglText &&aa) : text(aa.text),stl(aa.stl),val(aa.val)	{}
+#endif
+	const mglText&operator=(const mglText &aa)	{ text=aa.text;	stl=aa.stl;	val=aa.val;	return aa;	}
 };
 //-----------------------------------------------------------------------------
 /// Structure for internal point representation
@@ -112,24 +202,21 @@ struct MGL_EXPORT mglPnt	// NOTE: use float for reducing memory size
 	float u,v,w;	// normales
 	float r,g,b,a;	// RGBA color
 	short sub;		// subplot id and rotation information (later will be in subplot)
-	mglPnt()	{	xx=yy=zz=x=y=z=c=t=ta=u=v=w=r=g=b=a=sub=0;	}
+	mglPnt(float X=0, float Y=0, float Z=0, float U=0, float V=0, float W=0, float R=0, float G=0, float B=0, float A=0):xx(X),yy(Y),zz(Z),x(X),y(Y),z(Z),c(0),t(0),ta(0),u(U),v(V),w(W),r(R),g(G),b(B),a(A),sub(0)	{}
+	mglPnt(const mglPnt &aa) : xx(aa.xx),yy(aa.yy),zz(aa.zz), x(aa.x),y(aa.y),z(aa.z), c(aa.c),t(aa.t),ta(aa.ta), u(aa.u),v(aa.v),w(aa.w), r(aa.r),g(aa.g),b(aa.b),a(aa.a), sub(aa.sub)	{}
+#if MGL_HAVE_RVAL
+	mglPnt(mglPnt &&aa) : xx(aa.xx),yy(aa.yy),zz(aa.zz), x(aa.x),y(aa.y),z(aa.z), c(aa.c),t(aa.t),ta(aa.ta), u(aa.u),v(aa.v),w(aa.w), r(aa.r),g(aa.g),b(aa.b),a(aa.a), sub(aa.sub)	{}
+#endif
+	inline const mglPnt&operator=(const mglPnt &aa)	{	memcpy(this,&aa,sizeof(mglPnt));	return aa;	}
 };
 inline mglPnt operator+(const mglPnt &a, const mglPnt &b)
-{	mglPnt c=a;
-	c.x+=b.x;	c.y+=b.y;	c.z+=b.z;	c.u+=b.u;	c.v+=b.v;	c.w+=b.w;
-	c.r+=b.r;	c.g+=b.g;	c.b+=b.b;	c.a+=b.a;	return c;	}
+{	return mglPnt(a.x+b.x,a.y+b.y,a.z+b.z, a.u+b.u,a.v+b.v,a.w+b.w, a.r+b.r,a.g+b.g,a.b+b.b,a.a+b.a);	}
 inline mglPnt operator-(const mglPnt &a, const mglPnt &b)
-{	mglPnt c=a;
-	c.x-=b.x;	c.y-=b.y;	c.z-=b.z;	c.u-=b.u;	c.v-=b.v;	c.w-=b.w;
-	c.r-=b.r;	c.g-=b.g;	c.b-=b.b;	c.a-=b.a;	return c;	}
+{	return mglPnt(a.x-b.x,a.y-b.y,a.z-b.z, a.u-b.u,a.v-b.v,a.w-b.w, a.r-b.r,a.g-b.g,a.b-b.b,a.a-b.a);	}
 inline mglPnt operator*(const mglPnt &a, float b)
-{	mglPnt c=a;
-	c.x*=b;	c.y*=b;	c.z*=b;	c.u*=b;	c.v*=b;	c.w*=b;
-	c.r*=b;	c.g*=b;	c.b*=b;	c.a*=b;	return c;	}
+{	return mglPnt(a.x*b,a.y*b,a.z*b, a.u*b,a.v*b,a.w*b, a.r*b,a.g*b,a.b*b,a.a*b);	}
 inline mglPnt operator*(float b, const mglPnt &a)
-{	mglPnt c=a;
-	c.x*=b;	c.y*=b;	c.z*=b;	c.u*=b;	c.v*=b;	c.w*=b;
-	c.r*=b;	c.g*=b;	c.b*=b;	c.a*=b;	return c;	}
+{	return mglPnt(a.x*b,a.y*b,a.z*b, a.u*b,a.v*b,a.w*b, a.r*b,a.g*b,a.b*b,a.a*b);	}
 //-----------------------------------------------------------------------------
 /// Structure for glyph representation
 struct MGL_EXPORT mglGlyph
@@ -137,16 +224,20 @@ struct MGL_EXPORT mglGlyph
 	long nt, nl;			///< number of triangles and lines
 	short *trig, *line;	///< vertexes of triangles and lines
 
-	mglGlyph()	{	nl=nt=0;	trig=line=0;	}
-	mglGlyph(const mglGlyph &a)	{	nl=nt=0;	trig=line=0;	*this=a;	}
-	mglGlyph(long Nt, long Nl)	{	nl=nt=0;	trig=line=0;	Create(Nt,Nl);	}
+	mglGlyph():nt(0),nl(0),trig(0),line(0)	{}
+	mglGlyph(const mglGlyph &a):nt(0),nl(0),trig(0),line(0)	{	*this=a;	}
+	mglGlyph(long Nt, long Nl):nt(0),nl(0),trig(0),line(0)	{	Create(Nt,Nl);	}
+#if MGL_HAVE_RVAL
+	mglGlyph(mglGlyph &&aa) : nt(aa.nt),nl(aa.nl),trig(aa.trig), line(aa.line)	{	aa.trig=aa.line=0;	}
+#endif
 	~mglGlyph()	{	if(trig)	delete []trig;	if(line)	delete []line;	}
 
 	void Create(long Nt, long Nl);
-	bool operator==(const mglGlyph &g);
-	inline mglGlyph &operator=(const mglGlyph &a)
+	bool operator==(const mglGlyph &g) MGL_FUNC_PURE;
+	inline bool operator!=(const mglGlyph &g)	{	return !(*this==g);	}
+	inline const mglGlyph &operator=(const mglGlyph &a)
 	{	Create(a.nt, a.nl);	memcpy(trig, a.trig, 6*nt*sizeof(short));
-		memcpy(line, a.line, 2*nl*sizeof(short));	return *this;	}
+		memcpy(line, a.line, 2*nl*sizeof(short));	return a;	}
 };
 //-----------------------------------------------------------------------------
 #define MGL_TEXTURE_COLOURS 512
@@ -160,19 +251,29 @@ struct MGL_EXPORT mglTexture
 	int Smooth;			///< Type of texture (smoothing and so on)
 	mreal Alpha;			///< Transparency
 
-	mglTexture()	{	n=Smooth=0;	Alpha=1;	}
-	mglTexture(const char *cols, int smooth=0,mreal alpha=1)
-	{	n=0;	Set(cols,smooth,alpha);	}
+	mglTexture():n(0),Smooth(0),Alpha(1)	{}
+	mglTexture(const char *cols, int smooth=0,mreal alpha=1):n(0)
+	{	Set(cols,smooth,alpha);	}
+	mglTexture(const mglTexture &aa) : n(aa.n),Smooth(aa.Smooth),Alpha(aa.Alpha)
+	{	memcpy(col,aa.col,MGL_TEXTURE_COLOURS*sizeof(mglColor));	memcpy(Sch,aa.Sch,260);	}
+#if MGL_HAVE_RVAL
+	mglTexture(mglTexture &&aa) : n(aa.n),Smooth(aa.Smooth),Alpha(aa.Alpha)
+	{	memcpy(col,aa.col,MGL_TEXTURE_COLOURS*sizeof(mglColor));	memcpy(Sch,aa.Sch,260);	}
+#endif
 	void Clear()	{	n=0;	}
 	void Set(const char *cols, int smooth=0,mreal alpha=1);
 	void Set(HCDT val, const char *cols);
 	void GetC(mreal u,mreal v,mglPnt &p) const;
-	mglColor GetC(mreal u,mreal v=0) const;
+	mglColor GetC(mreal u,mreal v=0) const MGL_FUNC_PURE;
 	inline bool IsSame(const mglTexture &t) const
 	{	return n==t.n && !memcmp(col,t.col,MGL_TEXTURE_COLOURS*sizeof(mglColor));	}
 	void GetRGBA(unsigned char *f) const;
 	void GetRGBAPRC(unsigned char *f) const;
 	void GetRGBAOBJ(unsigned char *f) const;	// Export repeating border colors, since OBJ by default wraps textures and we need an extra boundary to work around implementation quirks
+	inline const mglTexture &operator=(const mglTexture &aa)
+	{	n=aa.n;	Smooth=aa.Smooth;	Alpha=aa.Alpha;
+		memcpy(col,aa.col,MGL_TEXTURE_COLOURS*sizeof(mglColor));
+		memcpy(Sch,aa.Sch,260);	return aa;	}
 };
 //-----------------------------------------------------------------------------
 const mglColor NC(-1,-1,-1);
@@ -195,15 +296,14 @@ public:
 	mglBase();
 	virtual ~mglBase();
 
-	bool Stop;			///< Flag that execution should be terminated.
 	mglPoint Min;		///< Lower edge of bounding box for graphics.
 	mglPoint Max;		///< Upper edge of bounding box for graphics.
 	mreal ZMin;			///< Adjusted minimal z-value 1D plots
 	std::string Mess;	///< Buffer for receiving messages
 	int ObjId;			///< object id for mglPrim
 	int HighId;			///< object id to be highlited
-	std::vector<mglGroup> Grp;		///< List of groups with names -- need for export
-	std::vector<mglActivePos> Act;	///< Position of active points
+	mglStack<mglGroup> Grp;		///< List of groups with names -- need for export
+	mglStack<mglActivePos> Act;	///< Position of active points
 	std::string PlotId;	///< Id of plot for saving filename (in GLUT window for example)
 
 	mreal CDef;			///< Default (current) color in texture
@@ -329,13 +429,13 @@ public:
 	/// Set default font size
 	inline void SetFontSize(mreal val)	{	FontSize=val>0 ? val:-FontSize*val;	}
 	inline mreal GetFontSize() const	{	return FontSize;	}
-	mreal TextWidth(const char *text, const char *font, mreal size) const;
-	mreal TextWidth(const wchar_t *text, const char *font, mreal size) const;
-	mreal TextHeight(const char *font, mreal size) const;
+	mreal TextWidth(const char *text, const char *font, mreal size) const MGL_FUNC_PURE;
+	mreal TextWidth(const wchar_t *text, const char *font, mreal size) const MGL_FUNC_PURE;
+	mreal TextHeight(const char *font, mreal size) const MGL_FUNC_PURE;
 	inline mreal FontFactor() const		{	return font_factor;	}
-	virtual mreal GetRatio() const;
-	virtual int GetWidth() const;
-	virtual int GetHeight() const;
+	virtual mreal GetRatio() const MGL_FUNC_CONST;
+	virtual int GetWidth() const MGL_FUNC_CONST;
+	virtual int GetHeight() const MGL_FUNC_CONST;
 
 	/// Set to use or not text rotation
 	inline void SetRotatedText(bool val)	{	set(val,MGL_ENABLE_RTEXT);	}
@@ -363,7 +463,9 @@ public:
 	{	return AddPnt(&B,p,c,n,a,scl);	}
 	long AddPnt(const mglMatrix *M, mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1);
 	long CopyNtoC(long k, mreal c);
-	long CopyProj(long from, mglPoint p, mglPoint n);
+	long CopyProj(long from, mglPoint p, mglPoint n, short sub=0);
+	void SetRGBA(long k, const mglColor &c)
+	{	mglPnt &p=Pnt[k];	p.r = c.r;	p.g = c.g;	p.b = c.b;	p.a = c.a;	}
 	virtual void Reserve(long n);		///< Allocate n-cells for Pnt and return current position
 	/// Set to reduce accuracy of points (to reduce size of output files)
 	inline void SetReduceAcc(bool val)	{	set(val, MGL_REDUCEACC);	}
@@ -388,7 +490,11 @@ public:
 	inline long GetGlfNum() const		{	return Glf.size();	}
 	inline const mglPnt &GetPnt(long i) const	{	return Pnt[i];		}
 	inline long GetPntNum() const		{	return Pnt.size();	}
-	inline mglPrim &GetPrm(long i)		{	return Prm[i];		}
+//	inline mglPrim &GetPrm(long i)		{	return Prm[i];		}
+	inline mglPrim &GetPrm(long i, bool sort=true)
+	{	return (sort && PrmInd) ? Prm[PrmInd[i]]:Prm[i];	}
+	inline const mglPrim &GetPrm(long i, bool sort=true) const
+	{	return (sort && PrmInd) ? Prm[PrmInd[i]]:Prm[i];	}
 	inline long GetPrmNum() const		{	return Prm.size();	}
 	inline const mglText &GetPtx(long i) const	{	return Ptx[i];		}
 	inline long GetPtxNum() const		{	return Ptx.size();	}
@@ -405,7 +511,7 @@ public:
 	inline mreal GetC(long s,mreal z,bool scale = true) const
 	{	return s+(scale?GetA(z):(z>0?z/MGL_FEPSILON:0));	}
 	/// Get alpha value depending on single variable a
-	mreal GetA(mreal a) const;
+	mreal GetA(mreal a) const MGL_FUNC_PURE;
 	/// Set pen/palette
 	char SetPenPal(const char *stl, long *id=0, bool pal=true);
 	/// Add texture (like color scheme) and return the position of first color
@@ -431,9 +537,19 @@ public:
 	inline mreal mark_size()	{	return MarkSize*font_factor;	}
 //	inline char last_color()	{	return last_style[1];	}
 	inline const char *last_line()	{	return last_style;	}
-	void resort();	///< Resort primitives in creation order (need for export in 3D formats)
+	int PrmCmp(long i, long j) const MGL_FUNC_PURE;	// compare 2 primitives with indexes i,j
+	/// Check if plot termination is asked
+	bool NeedStop()	{	if(event_cb)	event_cb(event_par);	return Stop;	}
+	/// Ask to stop drawing
+	void AskStop(bool stop=true)	{	Stop = stop;	}
+	/// Set callback function for event processing
+	void SetEventFunc(void (*func)(void *), void *par)	{	event_cb=func;	event_par=par;	}
 
 protected:
+	bool Stop;			///< Flag that execution should be terminated.
+	void (*event_cb)(void *);	///< Function to be called for event processing
+	void *event_par;	///< Parameter for event processing function
+
 	mglPoint OMin;		///< Lower edge for original axis (before scaling)
 	mglPoint OMax;		///< Upper edge for original axis (before scaling)
 	mglPoint AMin;		///< Lower edge for axis scaling
@@ -442,16 +558,20 @@ protected:
 	mglPoint FMax;		///< Actual upper edge after transformation formulas.
 	mglPoint Org;		///< Center of axis cross section.
 	int WarnCode;		///< Warning code
-	std::vector<mglPnt> Pnt; 	///< Internal points
-	std::vector<mglPrim> Prm;	///< Primitives (lines, triangles and so on) -- need for export
-	std::vector<mglPrim> Sub;	///< InPlot regions {n1=x1,n2=x2,n3=y1,n4=y2,id}
+	long *PrmInd;		///< Indexes of sorted primitives
+	mglStack<mglPnt> Pnt; 	///< Internal points
+	mglStack<mglPrim> Prm;	///< Primitives (lines, triangles and so on) -- need for export
+	mglStack<mglPrim> Sub;	///< InPlot regions {n1=x1,n2=x2,n3=y1,n4=y2,id}
 	std::vector<mglText> Ptx;	///< Text labels for mglPrim
 	std::vector<mglText> Leg;	///< Text labels for legend
 	std::vector<mglGlyph> Glf;	///< Glyphs data
-	std::vector<mglTexture> Txt;	///< Pointer to textures
+	mglStack<mglTexture> Txt;	///< Pointer to textures
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_t mutexPnt, mutexTxt, mutexLeg, mutexGlf, mutexAct, mutexDrw;
-	pthread_mutex_t mutexSub, mutexPrm, mutexPtx, mutexStk, mutexGrp;
+	pthread_mutex_t mutexSub, mutexPrm, mutexPtx, mutexStk, mutexGrp, mutexClf;
+#endif
+#if MGL_HAVE_OMP
+	omp_lock_t lockClf;
 #endif
 
 	int TernAxis;		///< Flag that Ternary axis is used
@@ -462,6 +582,7 @@ protected:
 	mreal AmbBr;		///< Default ambient light brightness
 	mreal DifBr;		///< Default diffusive light brightness
 
+	mreal persp;		///< Original value for perspective
 	mglMatrix Bp;		///< Transformation matrix for View() and Zoom()
 	mglMatrix B;		///< Transformation matrix
 	mglMatrix B1;		///< Transformation matrix for colorbar
@@ -482,10 +603,11 @@ protected:
 	mreal ArrowSize;	///< The size of arrows.
 	char last_style[64];///< Last pen style
 	mreal font_factor;	///< Font scaling factor
-	
+
 	long dr_x, dr_y, dr_p;	///< default drawing region for quality&4 mode
 
 	virtual void LightScale(const mglMatrix *M)=0;			///< Scale positions of light sources
+	void ClearPrmInd();
 
 	// block for SaveState()
 	mglPoint MinS;		///< Saved lower edge of bounding box for graphics.
@@ -505,6 +627,8 @@ protected:
 	int DefMaskAn;	///< Default mask rotation angle in degrees
 
 private:
+	mglBase(const mglBase &){}	// copying is not allowed
+	const mglBase &operator=(const mglBase &t){return t;}	// copying is not allowed
 
 	mglPoint CutMin;	///< Lower edge of bounding box for cut off.
 	mglPoint CutMax;	///< Upper edge of bounding box for cut off.
@@ -521,7 +645,9 @@ bool MGL_EXPORT mgl_check_dim2(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT a, const ch
 bool MGL_EXPORT mgl_check_dim3(HMGL gr, bool both, HCDT x, HCDT y, HCDT z, HCDT a, HCDT b, const char *name);
 bool MGL_EXPORT mgl_check_vec3(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay, HCDT az, const char *name);
 bool MGL_EXPORT mgl_check_trig(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HCDT a, const char *name, int d=3);
+bool MGL_EXPORT mgl_isnboth(HCDT x, HCDT y, HCDT z, HCDT a);
 bool MGL_EXPORT mgl_isboth(HCDT x, HCDT y, HCDT z, HCDT a);
+inline bool mgl_islog(mreal a,mreal b)	{	return (a>0 && b>10*a) || (b<0 && a<10*b);	}
 //-----------------------------------------------------------------------------
 #endif
 #endif
