@@ -32,13 +32,13 @@ class QScrollArea;
 class QSpinBox;
 class QTimer;
 class mglCanvas;
+class mglTask;
 //-----------------------------------------------------------------------------
 /// Class is Qt widget which display MathGL graphics
 class MGL_EXPORT QMathGL : public QWidget
 {
 	Q_OBJECT
 public:
-	friend void *mgl_qt_thr(void *);
 	QString appName; 	///< Application name for message boxes
 	bool autoResize; 	///< Allow auto resizing (default is false)
 	bool enableMouse;	///< Enable mouse handlers
@@ -62,18 +62,19 @@ public:
 	inline void zoomRegion(mreal xx1,mreal xx2,mreal yy1, mreal yy2)
 	{	x1=xx1;	y1=yy1;	x2=xx2;	y2=yy2;	}
 
-	int getPer()	{return int(per);}	///< Get perspective value
-	int getPhi()	{return int(phi);}	///< Get Phi-angle value
-	int getTet()	{return int(tet);}	///< Get Theta-angle value
-	bool getAlpha()	{return alpha;}	///< Get transparency state
-	bool getLight()	{return light;}	///< Get lightning state
-	bool getZoom()	{return zoom;}	///< Get mouse zooming state
-	bool getRotate(){return rotate;}	///< Get mouse rotation state
-	bool getViewYZ(){return viewYZ;}	///< Get mouse rotation axis
+	int getPer() const	{return int(per);}	///< Get perspective value
+	int getPhi() const	{return int(phi);}	///< Get Phi-angle value
+	int getTet() const	{return int(tet);}	///< Get Theta-angle value
+	bool getAlpha() const	{return alpha;}	///< Get transparency state
+	bool getLight() const	{return light;}	///< Get lightning state
+	bool getZoom() const	{return zoom;}	///< Get mouse zooming state
+	bool getRotate() const	{return rotate;}///< Get mouse rotation state
+	bool getViewYZ() const	{return viewYZ;}///< Get mouse rotation axis
 	bool isActive(int xs,int ys);	///< Check if active point is pressed
 
 public slots:
-	void refresh();
+	void refresh();			///< Redraw image with new zoom and view parameters
+	void refreshHQ();		///< Redraw image with HQ (can be slower than refresh())
 	void update();			///< Update picture
 	void copy();			///< copy graphics to clipboard
 	void copyClickCoor();	///< copy click coordinates to clipboard
@@ -87,6 +88,7 @@ public slots:
 	void setGrid(bool r);	///< Switch on/off grid drawing
 	void imgSize(int w, int h);	///< Set image size
 	void setViewYZ(bool v);	///< Switch on/off rotation around Y and Z axis
+	void setDotsPreview(bool d=true);	///< Set to use dots for image preview/rotation
 
 	void setCustZoom(bool a);	///< Switch on/off using custom zoom
 	void setCustDraw(bool a);	///< Switch on/off using custom draw
@@ -127,6 +129,8 @@ public slots:
 	void addCurve();					///< add curve into primitives
 	void addRhomb();					///< add rhombus into primitives
 	void addEllipse();					///< add ellipse into primitives
+	void addArc();					///< add arc into primitives
+	void addPolygon(int n=-1);			///< add polygon into primitives
 	void addText(QString txt="");		///< add text into primitives
 	void setStyle(int id, QString stl);///< set style for primitive with id
 
@@ -152,7 +156,7 @@ signals:
 	void frameChanged(int);		///< Need another frame to show
 	void showWarn(QString);		///< Show warnings
 	void posChanged(QString message);	///< user click to show mouse position
-	void objChanged(int objId);	///< User double-click to select object/line
+	void objChanged(int objId);	///< User click to select object/line
 	void refreshData();
 	void doubleClick(int id);	///< Double mouse click by object with id
 	void askStyle(int id);		///< Update style
@@ -176,8 +180,9 @@ protected:
 	int (*draw_func)(mglBase *gr, void *par);
 	mglDraw *draw;		///< Class for drawing -- need to call directly due to inheritance mechanism
 	QString mousePos;	///< Last mouse position
-	QPixmap pic;			///< Pixmap for drawing (changed by update)
-	double tet, phi;		///< Rotation angles
+	QPixmap pic;		///< Pixmap for drawing (changed by update)
+	QImage img;			///< Last used HQ image
+	double tet, phi;	///< Rotation angles
 	double per;			///< Value of perspective ( must be in [0,1) )
 	bool alpha;			///< Transparency state
 	bool light;			///< Lightning state
@@ -185,17 +190,24 @@ protected:
 	bool custDraw;		///< Use custom draw before main drawing
 	bool zoom;			///< Mouse zoom state
 	bool grid;			///< Grid drawing state
-	bool rotate;			///< Mouse rotation state
-	bool viewYZ;			///< Set mouse rotation around Y and Z axis (instead of X and Z)
+	bool rotate;		///< Mouse rotation state
+	bool viewYZ;		///< Set mouse rotation around Y and Z axis (instead of X and Z)
+	bool dotsRefr;		///< Set dots for image preview/rotation
 	mreal x1,x2,y1,y2;	///< Zoom in region
 	mreal ax1,ax2,ay1,ay2;	///< Axis range zoom
 	bool showMessage;	///< Flag for showing messages (enabled by each execute())
 	QMenu *popup;		///< Pointer to pop-up menu
 	QTimer *timer;		///< Timer for animation
+	QTimer *timerRefr;	///< Timer for redrawing
+private slots:
+	void afterPlot();	///< minor tuning after plot was done
 private:
 	int x0, y0, xe, ye;		///< Temporary variables for mouse
 	uchar *grBuf;
-	void draw_thr();
+	void drawPrim();
+	int prevQuality;
+//	QThread *thread;
+//	mglTask *task;
 };
 //-----------------------------------------------------------------------------
 /// Class for drawing the MGL script
@@ -212,8 +224,8 @@ public:
 };
 //-----------------------------------------------------------------------------
 /// Convert bitmap from mglCanvasWnd to QPixmap
-void mglConvertFromGraph(QPixmap &pic, mglCanvas *gr, uchar **buf);
+void mglConvertFromGraph(QPixmap &pic, mglCanvas *gr, uchar **buf, QImage *out=NULL);
 /// Make menu, toolbars and return popup menu for MainWindow
-QMenu *mglMakeMenu(QMainWindow* Wnd, QMathGL* QMGL, QSpinBox*& tet, QSpinBox*& phi);
+MGL_EXPORT QMenu *mglMakeMenu(QMainWindow* Wnd, QMathGL* QMGL, QSpinBox*& tet, QSpinBox*& phi);
 //-----------------------------------------------------------------------------
 #endif
