@@ -38,6 +38,8 @@ int MGL_EXPORT mgl_qt_run_();
 /// Wrapper class for windows displaying graphics
 class MGL_EXPORT mglQT : public mglWnd
 {
+	mglQT(const mglQT &t) {}	// copying is not allowed
+	const mglQT &operator=(const mglQT &t)	{	return t;	}
 public:
 	mglQT(const char *title="MathGL") : mglWnd()
 	{	gr = mgl_create_graph_qt(0,title,0,0);	}
@@ -48,6 +50,7 @@ public:
 	mglQT(mglDraw *draw, const char *title="MathGL") : mglWnd()
 	{	gr = mgl_create_graph_qt(draw?mgl_draw_class:0,title,draw,mgl_reload_class);
 		mgl_set_click_func(gr, mgl_click_class);	}
+	virtual ~mglQT() {}
 	int Run()	{	return mgl_qt_run();	}	///< Run main loop for event handling
 };
 //-----------------------------------------------------------------------------
