@@ -22,8 +22,9 @@
 
 #include "mgl2/define.h"
 //-----------------------------------------------------------------------------
-const mreal Pi = M_PI;
-const mreal NaN = NAN;
+const mreal mglPi = M_PI;
+const mreal mglNaN = NAN;
+const mreal mglInf = INFINITY;
 const mreal mgl_min_a = 1./256;
 //-----------------------------------------------------------------------------
 #define MGL_SET_XYZ(p,xx,yy,zz)		{p.x=(xx);p.y=(yy);p.z=(zz);}
@@ -35,12 +36,18 @@ const mreal mgl_min_a = 1./256;
 struct MGL_EXPORT mglPoint
 {
 	mreal x,y,z,c;
-	mglPoint(mreal X=0,mreal Y=0,mreal Z=0,mreal C=0){x=X;y=Y;z=Z;c=C;}
+	mglPoint(mreal X=0,mreal Y=0,mreal Z=0,mreal C=0):x(X),y(Y),z(Z),c(C) {}
+	mglPoint(const mglPoint &d):x(d.x),y(d.y),z(d.z),c(d.c) {}
+#if MGL_HAVE_RVAL
+	mglPoint(mglPoint &&d):x(d.x),y(d.y),z(d.z),c(d.c)	{}
+#endif
 	inline bool IsNAN()		{	return (x!=x || y!=y || z!=z || c!=c);	}
 	inline mreal val(int i)	{	return (i<2 ? (i==0 ? x:y) : (i==2 ? z:c));	}
 	inline mreal norm()		{	return sqrt(x*x+y*y+z*z);	}
 	inline void Normalize()	{	mreal v=norm();	x/=v;	y/=v;	z/=v;	}
 
+	inline const mglPoint &operator=(const mglPoint &p)
+	{	x=p.x;	y=p.y;	z=p.z;	c=p.c;	return p;	}
 	inline void operator+=(const mglPoint &a)	{	x+=a.x;	y+=a.y;	z+=a.z;	c+=a.c;	}
 	inline void operator-=(const mglPoint &a)	{	x-=a.x;	y-=a.y;	z-=a.z;	c-=a.c;	}
 	inline void operator+=(mreal a)	{	x+=a;	y+=a;	z+=a;	}
@@ -74,9 +81,11 @@ inline mglPoint operator^(const mglPoint &a, const mglPoint &b)
 inline mglPoint operator!(const mglPoint &a)
 {	mreal f=mgl_hypot(a.x,a.y);	return f==0?mglPoint(0.,1.,0.):mglPoint(-a.y/f, a.x/f, 0);	}
 inline bool operator==(const mglPoint &a, const mglPoint &b)
-{	return !memcmp(&a, &b, sizeof(mglPoint));	}
+{	return (a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z)+(a.c-b.c)*(a.c-b.c)==0;	}
+//{	return !memcmp(&a, &b, sizeof(mglPoint));	}
 inline bool operator!=(const mglPoint &a, const mglPoint &b)
-{	return memcmp(&a, &b, sizeof(mglPoint));	}
+{	return (a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z)+(a.c-b.c)*(a.c-b.c)!=0;	}
+//{	return memcmp(&a, &b, sizeof(mglPoint));	}
 inline bool operator<(const mglPoint &a, const mglPoint &b)
 {	return a.x<=b.x && a.y<=b.y && a.z<=b.z;	}
 inline bool operator>(const mglPoint &a, const mglPoint &b)
@@ -94,11 +103,16 @@ struct MGL_EXPORT mglColor
 	float a;	///< Alpha component of color
 
 	/// Constructor for RGB components manualy
-	mglColor(float R,float G,float B, float A=1){	r=R;	g=G;	b=B;	a=A;	}
+	mglColor(float R,float G,float B, float A=1):r(R),g(G),b(B),a(A)	{}
 	/// Constructor set default color
-	mglColor()		{	r=g=b=0;	a=1;	}
+	mglColor():r(0),g(0),b(0),a(1)	{}
 	/// Constructor set color from character id
 	mglColor(char c, float bright=1)		{	Set(c,bright);	}
+	/// Copy constructor
+	mglColor(const mglColor &d):r(d.r),g(d.g),b(d.b),a(d.a)	{}
+#if MGL_HAVE_RVAL
+	mglColor(mglColor &&d):r(d.r),g(d.g),b(d.b),a(d.a)	{}
+#endif
 	/// Set color as Red, Green, Blue values
 	void Set(float R,float G,float B,float A=1)	{	r=R;	g=G;	b=B;	a=A;	}
 	/// Set color as Red, Green, Blue values
@@ -123,11 +137,15 @@ struct MGL_EXPORT mglColor
 		float rgb[3];	mgl_chrrgb(p,rgb);
 		Set(mglColor(rgb[0],rgb[1],rgb[2]),bright);
 	}
+	inline const mglColor &operator=(const mglColor &p)
+	{	r=p.r;	g=p.g;	b=p.b;	a=p.a;	return p;	}
 	/// Copy color from other one
 	inline bool operator==(const mglColor &c) const
-	{	return !memcmp(this, &c, sizeof(mglColor));	}
+	{	return (r-c.r)*(r-c.r)+(g-c.g)*(g-c.g)+(b-c.b)*(b-c.b)+(a-c.a)*(a-c.a)==0;	}
+//	{	return !memcmp(this, &c, sizeof(mglColor));	}
 	inline bool operator!=(const mglColor &c) const
-	{	return memcmp(this, &c, sizeof(mglColor));		}
+	{	return (r-c.r)*(r-c.r)+(g-c.g)*(g-c.g)+(b-c.b)*(b-c.b)+(a-c.a)*(a-c.a)!=0;	}
+//	{	return memcmp(this, &c, sizeof(mglColor));		}
 	inline bool operator<(const mglColor &c) const
 	{	return memcmp(this, &c, sizeof(mglColor))<0;	}
 	// transparency still the same
