@@ -25,17 +25,20 @@
 //		C interfaces
 //
 //-----------------------------------------------------------------------------
+bool mglPrintWarn = true;
+void MGL_EXPORT mgl_suppress_warn(int on)	{	mglPrintWarn = !on;	}
+void MGL_EXPORT mgl_suppress_warn_(int *on)	{	mgl_suppress_warn(*on);	}
 void MGL_EXPORT mgl_set_quality(HMGL gr, int qual)	{	gr->SetQuality(qual);	}
 void MGL_EXPORT mgl_set_quality_(uintptr_t *gr, int *qual)	{	_GR_->SetQuality(*qual);	}
-int MGL_EXPORT mgl_get_quality(HMGL gr)	{	return gr->GetQuality();	}
-int MGL_EXPORT mgl_get_quality_(uintptr_t *gr)	{	return _GR_->GetQuality();	}
-int MGL_EXPORT mgl_is_frames(HMGL gr)
+int MGL_EXPORT_PURE mgl_get_quality(HMGL gr)	{	return gr->GetQuality();	}
+int MGL_EXPORT_PURE mgl_get_quality_(uintptr_t *gr)	{	return _GR_->GetQuality();	}
+int MGL_EXPORT_PURE mgl_is_frames(HMGL gr)
 {	return gr->get(MGL_VECT_FRAME) && !(gr->GetQuality()&MGL_DRAW_LMEM);	}
 void MGL_EXPORT mgl_set_draw_reg(HMGL gr, long nx, long ny, long m)	{	gr->SetDrawReg(nx,ny,m);	}
 void MGL_EXPORT mgl_set_draw_reg_(uintptr_t *gr, int *nx, int *ny, int *m)	{	_GR_->SetDrawReg(*nx,*ny,*m);	}
 //-----------------------------------------------------------------------------
-int MGL_EXPORT mgl_get_flag(HMGL gr, uint32_t flag)	{	return gr->get(flag);	}
-int MGL_EXPORT mgl_get_flag_(uintptr_t *gr, unsigned long *flag)	{	return _GR_->get(*flag);	}
+int MGL_EXPORT_PURE mgl_get_flag(HMGL gr, uint32_t flag)	{	return gr->get(flag);	}
+int MGL_EXPORT_PURE mgl_get_flag_(uintptr_t *gr, unsigned long *flag)	{	return _GR_->get(*flag);	}
 void MGL_EXPORT mgl_set_flag(HMGL gr, int val, uint32_t flag)		{	gr->set(val,flag);	}
 void MGL_EXPORT mgl_set_flag_(uintptr_t *gr, int *val, unsigned long *flag)	{	_GR_->set(*val,*flag);	}
 //-----------------------------------------------------------------------------
@@ -55,12 +58,43 @@ void MGL_EXPORT mgl_set_plotid(HMGL gr, const char *id)	{	gr->PlotId = id;	}
 void MGL_EXPORT mgl_set_plotid_(uintptr_t *gr, const char *id,int l)
 {	char *s=new char[l+1];	memcpy(s,id,l);	s[l]=0;
 	_GR_->PlotId = s;	delete []s;	}
-MGL_EXPORT const char *mgl_get_plotid(HMGL gr)	{	return gr->PlotId.c_str();	}
+MGL_EXPORT_PURE const char *mgl_get_plotid(HMGL gr)	{	return gr->PlotId.c_str();	}
+int MGL_EXPORT mgl_get_plotid_(uintptr_t *gr, char *out, int len)
+{
+	const char *res = mgl_get_plotid(_GR_);
+	if(out)	strncpy(out,res,len);
+	return strlen(res);
+}
 //-----------------------------------------------------------------------------
-MGL_EXPORT const char *mgl_get_mess(HMGL gr)	{	return gr->Mess.c_str();	}
-int MGL_EXPORT mgl_get_warn(HMGL gr)	{	return gr->GetWarn();	}
+MGL_EXPORT_PURE const char *mgl_get_mess(HMGL gr)	{	return gr->Mess.c_str();	}
+int MGL_EXPORT mgl_get_mess_(uintptr_t *gr, char *out, int len)
+{
+	const char *res = mgl_get_mess(_GR_);
+	if(out)	strncpy(out,res,len);
+	return strlen(res);
+}
+int MGL_EXPORT_PURE mgl_get_warn(HMGL gr)	{	return gr->GetWarn();	}
 void MGL_EXPORT mgl_set_warn(HMGL gr, int code, const char *txt)
 {	gr->SetWarn(code,txt);	}
+extern bool mglPrintWarn;
+void MGL_EXPORT mgl_set_global_warn(const char *txt)
+{
+	if(txt && *txt)
+	{
+		mglGlobalMess += txt;	mglGlobalMess += '\n';
+		if(mglPrintWarn)	fprintf(stderr,"Global message - %s\n",txt);
+	}
+}
+void MGL_EXPORT mgl_set_global_warn_(const char *txt, int l)
+{	char *s=new char[l+1];	memcpy(s,txt,l);	s[l]=0;	mgl_set_global_warn(s);	delete []s;	}
+MGL_EXPORT_PURE const char *mgl_get_global_warn()	{	return mglGlobalMess.c_str();	}
+int MGL_EXPORT mgl_get_global_warn_(char *out, int len)
+{
+	const char *res = mgl_get_global_warn();
+	if(out)	strncpy(out,res,len);
+	return strlen(res);
+}
+//-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_set_origin(HMGL gr, double x0, double y0, double z0)
 {	gr->SetOrigin(x0,y0,z0);	}
 void MGL_EXPORT mgl_set_palette(HMGL gr, const char *colors)
@@ -119,7 +153,7 @@ void MGL_EXPORT mgl_set_rdc_acc_(uintptr_t *gr, int *reduce)
 void MGL_EXPORT mgl_highlight_(uintptr_t *gr, int *id)	{	_GR_->Highlight(*id);	}
 void MGL_EXPORT mgl_set_origin_(uintptr_t *gr, mreal *x0, mreal *y0, mreal *z0)
 {	_GR_->SetOrigin(*x0,*y0,*z0);	}
-int MGL_EXPORT mgl_get_warn_(uintptr_t *gr)	{	return _GR_->GetWarn();	}
+int MGL_EXPORT_PURE mgl_get_warn_(uintptr_t *gr)	{	return _GR_->GetWarn();	}
 void MGL_EXPORT mgl_set_warn_(uintptr_t *gr, int *code, const char *txt, int l)
 {	char *s=new char[l+1];	memcpy(s,txt,l);	s[l]=0;
 	_GR_->SetWarn(*code, s);	delete []s;	}
@@ -201,6 +235,13 @@ void MGL_EXPORT mgl_def_font_(const char *name, const char *path,int l,int n)
 	char *d=new char[n+1];		memcpy(d,path,n);	d[n]=0;
 	mglDefFont.Load(name,path);	delete []s;		delete []d;	}
 //-----------------------------------------------------------------------------
+int MGL_EXPORT mgl_check_version(const char *ver)
+{	double v=0;	int r = sscanf(ver,"2.%lg",&v);
+	return r<1 || v>MGL_VER2;	}
+int MGL_EXPORT mgl_check_version_(const char *ver, int l)
+{	char *s=new char[l+1];		memcpy(s,ver,l);	s[l]=0;
+	int r=mgl_check_version(s);	delete []s;	return r;	}
+//-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_start_group(HMGL gr, const char *s)	{	gr->StartAutoGroup(s);	}
 void MGL_EXPORT mgl_end_group(HMGL gr)	{	gr->EndGroup();	}
 void MGL_EXPORT mgl_start_group_(uintptr_t *gr, const char *name,int l)
@@ -217,7 +258,7 @@ void MGL_EXPORT mgl_test_txt(const char *str, ...)
 		char buf[256];
 		va_list lst;
 		va_start(lst,str);
-		vsnprintf(buf,256,str,lst);
+		vsnprintf(buf,256,str,lst);	buf[255]=0;
 		va_end(lst);
 		printf("TEST: %s\n",buf);
 		fflush(stdout);
@@ -265,3 +306,10 @@ void MGL_EXPORT mgl_set_mask_val_(const char *id, uint64_t *mask,int)
 void MGL_EXPORT mgl_set_mask_angle(HMGL gr, int angle)	{	gr->SetMaskAngle(angle);	}
 void MGL_EXPORT mgl_set_mask_angle_(uintptr_t *gr, int *angle)	{	_GR_->SetMaskAngle(*angle);	}
 //---------------------------------------------------------------------------
+void MGL_EXPORT mgl_ask_stop(HMGL gr, int stop)		{	gr->AskStop(stop);	}
+void MGL_EXPORT mgl_ask_stop_(uintptr_t *gr, int *stop){	_GR_->AskStop(*stop);	}
+int MGL_EXPORT mgl_need_stop(HMGL gr)			{	return gr->NeedStop();	}
+int MGL_EXPORT mgl_need_stop_(uintptr_t *gr)	{	return _GR_->NeedStop();}
+void MGL_EXPORT mgl_set_event_func(HMGL gr, void (*func)(void *), void *par)
+{	gr->SetEventFunc(func,par);	}
+//---------------------------------------------------------------------------
