@@ -18,6 +18,8 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include <float.h>
+#include <math.h>
+#include <list>
 #include "mgl2/other.h"
 #include "mgl2/data.h"
 #include "mgl2/thread.h"
@@ -35,17 +37,15 @@ void MGL_EXPORT mgl_triplot_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HCD
 	long ss=gr->AddTexture(sch);
 	gr->SaveState(opt);	gr->SetPenPal("-");
 	static int cgid=1;	gr->StartGroup("TriPlot",cgid++);
-	mglPoint p1,p2,p3,q;
 
 	bool wire = mglchr(sch,'#');
 	long nc = a->GetNx();
 	if(nc!=n && nc>=m)	// colors per triangle
 	{
+		mglPoint p1,p2,p3,q;
 		gr->Reserve(m*3);
-#pragma omp parallel for private(p1,p2,p3,q)
 		for(long i=0;i<m;i++)
 		{
-			if(gr->Stop)	continue;
 			register long k1 = long(nums->v(0,i)+0.5);
 			p1 = mglPoint(x->v(k1), y->v(k1), z->v(k1));
 			register long k2 = long(nums->v(1,i)+0.5);
@@ -64,10 +64,8 @@ void MGL_EXPORT mgl_triplot_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HCD
 		gr->Reserve(n);
 		long *kk = new long[n];
 		mglPoint *pp = new mglPoint[n];
-#pragma omp parallel for
 		for(long i=0;i<m;i++)	// add averaged normales
 		{
-			if(gr->Stop)	continue;
 			register long k1 = long(nums->v(0,i)+0.5);
 			register long k2 = long(nums->v(1,i)+0.5);
 			register long k3 = long(nums->v(2,i)+0.5);
@@ -78,21 +76,14 @@ void MGL_EXPORT mgl_triplot_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HCD
 				q.Normalize();
 				// try be sure that in the same direction ...
 				if(q.z<0)	q *= -1;
-#pragma omp critical(quadplot)
-				{pp[k1] += q;	pp[k2] += q;	pp[k3] += q;}
+				pp[k1] += q;	pp[k2] += q;	pp[k3] += q;
 			}
 			else	pp[k1]=pp[k2]=pp[k3]=mglPoint(NAN,NAN);
 		}
-#pragma omp parallel for
 		for(long i=0;i<n;i++)	// add points
-		{
-			if(gr->Stop)	continue;
 			kk[i] = gr->AddPnt(mglPoint(x->v(i), y->v(i), z->v(i)), gr->GetC(ss,a->v(i)), pp[i]);
-		}
-#pragma omp parallel for
 		for(long i=0;i<m;i++)	// draw triangles
 		{
-			if(gr->Stop)	continue;
 			register long k1 = long(nums->v(0,i)+0.5);
 			register long k2 = long(nums->v(1,i)+0.5);
 			register long k3 = long(nums->v(2,i)+0.5);
@@ -155,10 +146,8 @@ void MGL_EXPORT mgl_quadplot_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HC
 	if(nc!=n && nc>=m)	// colors per triangle
 	{
 		gr->Reserve(m*4);
-#pragma omp parallel for private(p1,p2,p3,p4)
 		for(long i=0;i<m;i++)
 		{
-			if(gr->Stop)	continue;
 			register long k1 = long(nums->v(0,i)+0.5);
 			p1 = mglPoint(x->v(k1), y->v(k1), z->v(k1));
 			register long k2 = long(nums->v(1,i)+0.5);
@@ -180,10 +169,8 @@ void MGL_EXPORT mgl_quadplot_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HC
 		gr->Reserve(n);
 		long *kk = new long[n];
 		mglPoint *pp = new mglPoint[n];
-#pragma omp parallel for private(p1,p2,p3,p4)
 		for(long i=0;i<m;i++)	// add averaged normales
 		{
-			if(gr->Stop)	continue;
 			register long k1 = long(nums->v(0,i)+0.5);
 			p1 = mglPoint(x->v(k1), y->v(k1), z->v(k1));
 			register long k2 = long(nums->v(1,i)+0.5);
@@ -200,20 +187,13 @@ void MGL_EXPORT mgl_quadplot_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HC
 				mglPoint q2 = (p2-p4) ^ (p3-p4);	if(q2.z<0) q2*=-1;
 				mglPoint q3 = (p1-p2) ^ (p4-p2);	if(q3.z<0) q3*=-1;
 				mglPoint q4 = (p1-p4) ^ (p4-p3);	if(q4.z<0) q4*=-1;
-#pragma omp critical(quadplot)
-				{pp[k1] += q1;	pp[k2] += q2;	pp[k3] += q3;	pp[k4] += q4;}
+				pp[k1] += q1;	pp[k2] += q2;	pp[k3] += q3;	pp[k4] += q4;
 			}
 		}
-#pragma omp parallel for
 		for(long i=0;i<n;i++)	// add points
-		{
-			if(gr->Stop)	continue;
 			kk[i] = gr->AddPnt(mglPoint(x->v(i), y->v(i), z->v(i)),gr->GetC(ss,a->v(i)), pp[i]);
-		}
-#pragma omp parallel for
 		for(long i=0;i<m;i++)	// draw quads
 		{
-			if(gr->Stop)	continue;
 			register long k1 = floor(nums->v(0,i)+0.5);
 			register long k2 = floor(nums->v(1,i)+0.5);
 			register long k3 = floor(nums->v(2,i)+0.5);
@@ -261,49 +241,59 @@ void MGL_EXPORT mgl_quadplot_xy_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, u
 //	TriCont series
 //
 //-----------------------------------------------------------------------------
-void MGL_EXPORT mgl_tricont_xyzcv(HMGL gr, HCDT v, HCDT nums, HCDT x, HCDT y, HCDT z, HCDT a, const char *sch, const char *opt)
+#include "cont.hpp"
+//-----------------------------------------------------------------------------
+std::vector<mglSegment> MGL_NO_EXPORT mgl_tri_lines(mreal val, HCDT nums, HCDT a, HCDT x, HCDT y, HCDT z)
 {
 	long n = x->GetNx(), m = nums->GetNy();
-	if(mgl_check_trig(gr,nums,x,y,z,a,"TriCont"))	return;
-
-	long ss=gr->AddTexture(sch);
-	gr->SaveState(opt);
-	static int cgid=1;	gr->StartGroup("TriCont",cgid++);
-	bool zVal = !(mglchr(sch,'_'));
-	mglPoint p1,p2,p3;
-#pragma omp parallel for private(p1,p2,p3) collapse(2)
-	for(long k=0;k<v->GetNx();k++)	for(long i=0;i<m;i++)
+	std::vector<mglSegment> lines;
+	for(long i=0;i<m;i++)
 	{
-		if(gr->Stop)	continue;
 		register long k1 = long(nums->v(0,i)+0.1);	if(k1<0 || k1>=n)	continue;
 		register long k2 = long(nums->v(1,i)+0.1);	if(k2<0 || k2>=n)	continue;
 		register long k3 = long(nums->v(2,i)+0.1);	if(k3<0 || k3>=n)	continue;
-		register mreal val = v->v(k), c = gr->GetC(ss,val), d1,d2,d3;
-
-		d1 = mgl_d(val,a->v(k1),a->v(k2));
-		p1 = mglPoint(x->v(k1)*(1-d1)+x->v(k2)*d1, y->v(k1)*(1-d1)+y->v(k2)*d1,
-					  zVal?z->v(k1)*(1-d1)+z->v(k2)*d1:gr->Min.z);
-		d2 = mgl_d(val,a->v(k1),a->v(k3));
-		p2 = mglPoint(x->v(k1)*(1-d2)+x->v(k3)*d2, y->v(k1)*(1-d2)+y->v(k3)*d2,
-					  zVal?z->v(k1)*(1-d2)+z->v(k3)*d2:gr->Min.z);
-		d3 = mgl_d(val,a->v(k2),a->v(k3));
-		p3 = mglPoint(x->v(k2)*(1-d3)+x->v(k3)*d3, y->v(k2)*(1-d3)+y->v(k3)*d3,
-					  zVal?z->v(k2)*(1-d3)+z->v(k3)*d3:gr->Min.z);
+		register mreal v1 = a->v(k1), v2 = a->v(k2), v3 = a->v(k3);
+		register mreal d1 = mgl_d(val,v1,v2), d2 = mgl_d(val,v1,v3), d3 = mgl_d(val,v2,v3);
+		mglSegment line;
 		if(d1>=0 && d1<=1 && d2>=0 && d2<=1)
 		{
-			k1 = gr->AddPnt(p1,c);	k2 = gr->AddPnt(p2,c);
-			gr->line_plot(k1,k2);
+			line.p1 = mglPoint(x->v(k1)*(1-d1)+x->v(k2)*d1, y->v(k1)*(1-d1)+y->v(k2)*d1, z->v(k1)*(1-d1)+z->v(k2)*d1);
+			line.p2 = mglPoint(x->v(k1)*(1-d2)+x->v(k3)*d2, y->v(k1)*(1-d2)+y->v(k3)*d2, z->v(k1)*(1-d2)+z->v(k3)*d2);
 		}
 		else if(d1>=0 && d1<=1 && d3>=0 && d3<=1)
 		{
-			k1 = gr->AddPnt(p1,c);	k2 = gr->AddPnt(p3,c);
-			gr->line_plot(k1,k2);
+			line.p1 = mglPoint(x->v(k1)*(1-d1)+x->v(k2)*d1, y->v(k1)*(1-d1)+y->v(k2)*d1, z->v(k1)*(1-d1)+z->v(k2)*d1);
+			line.p2 = mglPoint(x->v(k2)*(1-d3)+x->v(k3)*d3, y->v(k2)*(1-d3)+y->v(k3)*d3, z->v(k2)*(1-d3)+z->v(k3)*d3);
 		}
 		else if(d3>=0 && d3<=1 && d2>=0 && d2<=1)
 		{
-			k1 = gr->AddPnt(p3,c);	k2 = gr->AddPnt(p2,c);
-			gr->line_plot(k1,k2);
+			line.p1 = mglPoint(x->v(k1)*(1-d2)+x->v(k3)*d2, y->v(k1)*(1-d2)+y->v(k3)*d2, z->v(k1)*(1-d2)+z->v(k3)*d2);
+			line.p2 = mglPoint(x->v(k2)*(1-d3)+x->v(k3)*d3, y->v(k2)*(1-d3)+y->v(k3)*d3, z->v(k2)*(1-d3)+z->v(k3)*d3);
 		}
+		if(line.p1!=line.p2)	lines.push_back(line);
+	}
+	return lines;
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_tricont_xyzcv(HMGL gr, HCDT v, HCDT nums, HCDT x, HCDT y, HCDT z, HCDT a, const char *sch, const char *opt)
+{
+	mglDataV zz(x->GetNx(),x->GetNy());
+	if(!z)	z = &zz;
+	if(mgl_check_trig(gr,nums,x,y,z,a,"TriCont"))	return;
+
+	gr->SaveState(opt);
+	static int cgid=1;	gr->StartGroup("TriCont",cgid++);
+	int text=0;
+	if(mglchr(sch,'t'))	text=1;
+	if(mglchr(sch,'T'))	text=2;
+	bool fixed=(mglchr(sch,'_')) || (gr->Min.z==gr->Max.z);
+	long s=gr->AddTexture(sch);
+	gr->SetPenPal(sch);
+
+	for(long k=0;k<v->GetNx();k++)
+	{
+		mreal v0 = v->v(k);		zz.Fill(fixed ? gr->Min.z : v0);
+		mgl_draw_curvs(gr,v0,gr->GetC(s,v0),text,mgl_get_curvs(gr,mgl_tri_lines(v0,nums,a,x,y,fixed?&zz:z)));
 	}
 }
 //-----------------------------------------------------------------------------
@@ -317,10 +307,10 @@ void MGL_EXPORT mgl_tricont_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HCD
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_tricont_xyc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
-{	mgl_tricont_xyzc(gr,nums,x,y,z,z,sch,opt);	}
+{	mgl_tricont_xyzc(gr,nums,x,y,0,z,sch,opt);	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_tricont_xycv(HMGL gr, HCDT v, HCDT nums, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
-{	mgl_tricont_xyzcv(gr,v,nums,x,y,z,z,sch,opt);	}
+{	mgl_tricont_xyzcv(gr,v,nums,x,y,0,z,sch,opt);	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_tricont_xyzcv_(uintptr_t *gr, uintptr_t *v, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *c, const char *sch, const char *opt,int l,int lo)
 {	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
@@ -347,6 +337,85 @@ void MGL_EXPORT mgl_tricont_xyc_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, u
 	delete []o;	delete []s;	}
 //-----------------------------------------------------------------------------
 //
+//	TriContV series
+//
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_tricontv_xyzcv(HMGL gr, HCDT v, HCDT nums, HCDT x, HCDT y, HCDT z, HCDT a, const char *sch, const char *opt)
+{
+	mglDataV zz(x->GetNx(),x->GetNy());
+	if(!z)	z = &zz;
+	if(mgl_check_trig(gr,nums,x,y,z,a,"TriContV"))	return;
+
+	gr->SaveState(opt);
+	static int cgid=1;	gr->StartGroup("TriContV",cgid++);
+	bool fixed=(mglchr(sch,'_')) || (gr->Min.z==gr->Max.z);
+	long s=gr->AddTexture(sch);
+	gr->SetPenPal(sch);
+
+	for(long k=0;k<v->GetNx();k++)
+	{
+		mreal v0 = v->v(k);		zz.Fill(fixed ? gr->Min.z : v0);
+		mreal dv = (gr->Max.c-gr->Min.c)/8, c = gr->GetC(s,v0);
+		if(k>0)	dv = v->v(k-1)-v->v(k);
+		else if(k<v->GetNx()-1)	dv = v->v(k)-v->v(k+1);
+		if(fixed)	dv=-dv;
+
+		const std::vector<mglSegment> curvs = mgl_get_curvs(gr,mgl_tri_lines(v0,nums,a,x,y,fixed?&zz:z));
+		for(size_t i=0;i<curvs.size();i++)
+		{
+			const std::list<mglPoint> &pp=curvs[i].pp;
+			long f2=-1,g2=-1;
+			for(std::list<mglPoint>::const_iterator it=pp.begin(); it != pp.end(); ++it)
+			{
+				mglPoint p=*it,q(p.y,-p.x);
+				long f1 = f2;	f2 = gr->AddPnt(p,c,q);	p.z+=dv;
+				long g1 = g2;	g2 = gr->AddPnt(p,c,q);
+				gr->quad_plot(f1,g1,f2,g2);
+			}
+		}
+	}
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_tricontv_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HCDT a, const char *sch, const char *opt)
+{
+	mreal r = gr->SaveState(opt);
+	long n = (mgl_isnan(r) || r<=0) ? 7:long(r+0.5);
+	mglData v(n);
+	for(long i=0;i<n;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(n+1);
+	mgl_tricontv_xyzcv(gr,&v,nums,x,y,z,a,sch,0);
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_tricontv_xyc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
+{	mgl_tricontv_xyzc(gr,nums,x,y,0,z,sch,opt);	}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_tricontv_xycv(HMGL gr, HCDT v, HCDT nums, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
+{	mgl_tricontv_xyzcv(gr,v,nums,x,y,0,z,sch,opt);	}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_tricontv_xyzcv_(uintptr_t *gr, uintptr_t *v, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *c, const char *sch, const char *opt,int l,int lo)
+{	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	mgl_tricontv_xyzcv(_GR_, _DA_(v), _DA_(nums), _DA_(x), _DA_(y), _DA_(z), _DA_(c), s, o);
+	delete []o;	delete []s;	}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_tricontv_xycv_(uintptr_t *gr, uintptr_t *v, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *sch, const char *opt,int l,int lo)
+{	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	mgl_tricontv_xycv(_GR_, _DA_(v), _DA_(nums), _DA_(x), _DA_(y), _DA_(z), s, o);
+	delete []o;	delete []s;	}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_tricontv_xyzc_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *c, const char *sch, const char *opt, int l,int lo)
+{	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	mgl_tricontv_xyzc(_GR_, _DA_(nums), _DA_(x), _DA_(y), _DA_(z), _DA_(c), s, o);
+	delete []o;	delete []s;	}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_tricontv_xyc_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *sch, const char *opt, int l,int lo)
+{	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	mgl_tricontv_xyc(_GR_, _DA_(nums), _DA_(x), _DA_(y), _DA_(z), s, o);
+	delete []o;	delete []s;	}
+//-----------------------------------------------------------------------------
+//
 //	Dots series
 //
 //-----------------------------------------------------------------------------
@@ -368,10 +437,8 @@ void MGL_EXPORT mgl_dots_ca(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT c, HCDT a, con
 	if(mk==0)	mk='.';
 	gr->Reserve(n);
 
-#pragma omp parallel for
 	for(long i=0;i<n;i+=d)
 	{
-		if(gr->Stop)	continue;
 		mglPoint p = mglPoint(x->vthr(i),y->vthr(i),z->vthr(i));
 		long pp = gr->AddPnt(p,gr->GetC(ss,c->vthr(i)),mglPoint(NAN),a?gr->GetA(a->vthr(i)):-1);
 		gr->mark_plot(pp, mk);
@@ -406,20 +473,18 @@ void MGL_EXPORT mgl_dots_ca_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_
 //-----------------------------------------------------------------------------
 long MGL_NO_EXPORT mgl_crust(long n,mglPoint *pp,long **nn,mreal ff);
 HMDT MGL_EXPORT mgl_triangulation_3d(HCDT x, HCDT y, HCDT z)
-{	// TODO: should be used s-hull or q-hull
+{
 	mglData *nums=0;
 	long n = x->GetNx(), m;
 	if(y->GetNx()!=n || z->GetNx()!=n)	return nums;
 	mglPoint *pp = new mglPoint[n];
 	long *nn=0;
-#pragma omp parallel for
 	for(long i=0;i<n;i++)	pp[i] = mglPoint(x->v(i), y->v(i), z->v(i));
 	m = mgl_crust(n,pp,&nn,0);
 
 	if(m>0)
 	{
 		nums=new mglData(3,m);
-#pragma omp parallel for
 		for(long i=0;i<3*m;i++)	nums->a[i]=nn[i];
 	}
 	delete []pp;	free(nn);	return nums;
@@ -433,18 +498,32 @@ HMDT MGL_EXPORT mgl_triangulation_2d(HCDT x, HCDT y)
 	if(y->GetNN()!=n)	return nums;
 	// use s-hull here
 	std::vector<Shx> pts;
-	std::vector<size_t> out;
+	std::vector<long> out;
 	Shx pt;
 
+	double mx = 0, my = 0;
 	for(long i=0;i<n;i++)
-	{	pt.r = x->vthr(i);	pt.c = y->vthr(i);	pt.id = i;	pts.push_back(pt);	}
+	{
+		register double t;
+		t = fabs(x->vthr(i));	if(t>mx)	mx=t;
+		t = fabs(y->vthr(i));	if(t>my)	my=t;
+	}
+	mx *= 1e-15;	my *= 1e-15;
+	for(long i=0;i<n;i++)
+	{
+		pt.r = x->vthr(i);	pt.c = y->vthr(i);
+		if(mgl_isbad(pt.r) || mgl_isbad(pt.c))	continue;
+		if(fabs(pt.r)<mx)	pt.r=0;
+		if(fabs(pt.c)<my)	pt.c=0;
+		pt.id = i;    pts.push_back(pt);
+	}
+
 	std::vector<Triad> triads;
 	if(de_duplicate(pts, out))
-		mglGlobalMess += "There are duplicated points for triangulation.\n";
+		mgl_set_global_warn("There are duplicated points for triangulation.");
 	s_hull_pro(pts, triads);
 	long m = triads.size();
 	nums=new mglData(3,m);
-#pragma omp parallel for
 	for(long i=0;i<m;i++)
 	{
 		nums->a[3*i]   = triads[i].a;
@@ -468,15 +547,17 @@ MGL_NO_EXPORT void *mgl_grid_t(void *par)
 	mglThreadD *t=(mglThreadD *)par;
 	long nx=t->p[0],ny=t->p[1];
 	mreal *b=t->a;
-	const mreal *x=t->b, *y=t->c, *d=t->d, *z=t->e;
+	const mreal *x=t->b, *y=t->c, *d=t->d;
+	HCDT zdat = (HCDT) t->v;
 #if !MGL_HAVE_PTHREAD
 #pragma omp parallel for
 #endif
 	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
-	{	// TODO check if rounding needed
-		register long k1 = long(d[3*i0]), k2 = long(d[3*i0+1]), k3 = long(d[3*i0+2]);
+	{
+		register long k1 = long(d[3*i0]+0.5), k2 = long(d[3*i0+1]+0.5), k3 = long(d[3*i0+2]+0.5);
 		mreal dxu,dxv,dyu,dyv;
-		mglPoint d1=mglPoint(x[k2]-x[k1],y[k2]-y[k1],z[k2]-z[k1]), d2=mglPoint(x[k3]-x[k1],y[k3]-y[k1],z[k3]-z[k1]), p;
+		mreal z1=zdat->vthr(k1), z2=zdat->vthr(k2), z3=zdat->vthr(k3);
+		mglPoint d1=mglPoint(x[k2]-x[k1],y[k2]-y[k1],z2-z1), d2=mglPoint(x[k3]-x[k1],y[k3]-y[k1],z3-z1), p;
 
 		dxu = d2.x*d1.y - d1.x*d2.y;
 		if(fabs(dxu)<1e-5) continue; // points lies on the same line
@@ -484,10 +565,10 @@ MGL_NO_EXPORT void *mgl_grid_t(void *par)
 		dyu = d2.x/dxu; dxu =-d2.y/dxu;
 
 		long x1,y1,x2,y2;
-		x1 = long(fmin(fmin(x[k1],x[k2]),x[k3])); // bounding box
-		y1 = long(fmin(fmin(y[k1],y[k2]),y[k3]));
-		x2 = long(fmax(fmax(x[k1],x[k2]),x[k3]));
-		y2 = long(fmax(fmax(y[k1],y[k2]),y[k3]));
+		x1 = long(mgl_min(mgl_min(x[k1],x[k2]),x[k3])); // bounding box
+		y1 = long(mgl_min(mgl_min(y[k1],y[k2]),y[k3]));
+		x2 = long(mgl_max(mgl_max(x[k1],x[k2]),x[k3]));
+		y2 = long(mgl_max(mgl_max(y[k1],y[k2]),y[k3]));
 		x1 = x1>0 ? x1:0; x2 = x2<nx ? x2:nx-1;
 		y1 = y1>0 ? y1:0; y2 = y2<ny ? y2:ny-1;
 		if((x1>x2) | (y1>y2)) continue;
@@ -499,7 +580,7 @@ MGL_NO_EXPORT void *mgl_grid_t(void *par)
 			xx = (i-x0); yy = (j-y0);
 			u = dxu*xx+dyu*yy; v = dxv*xx+dyv*yy;
 			if((u<0) | (v<0) | (u+v>1)) continue;
-			b[i+nx*j] = z[k1] + d1.z*u + d2.z*v;
+			b[i+nx*j] = z1 + d1.z*u + d2.z*v;
 		}
 	}
 	return 0;
@@ -509,25 +590,28 @@ void MGL_EXPORT mgl_data_grid_xy(HMDT d, HCDT xdat, HCDT ydat, HCDT zdat, mreal
 { // NOTE: only for mglData
 	const mglData *x = dynamic_cast<const mglData *>(xdat);
 	const mglData *y = dynamic_cast<const mglData *>(ydat);
-	const mglData *z = dynamic_cast<const mglData *>(zdat);
-	if(!x || !y || !z) return;
-	long n=x->GetNN();
-	if((n<3) || (y->GetNN()!=n) || (z->GetNN()!=n))	return;
+	long n=xdat->GetNN();
+	if((n<3) || (ydat->GetNN()!=n) || (zdat->GetNN()!=n))	return;
 
-	mglData *nums = mgl_triangulation_2d(x,y);
+	mglData *nums = mgl_triangulation_2d(xdat,ydat);
 	if(nums->nx<3)	{	delete nums;	return;	}
 	long nn = nums->ny, par[3]={d->nx,d->ny,d->nz};
-	mreal xx[4]={x1,0, y1,0};
-	if(d->nx>1) xx[1] = (d->nx-1.)/(x2-x1);
-	if(d->ny>1) xx[3] = (d->ny-1.)/(y2-y1);
+	mreal xx[4]={x1,(d->nx-1.)/(x2-x1), y1,(d->ny-1.)/(y2-y1)};
 
 	mreal *xc=new mreal[n], *yc=new mreal[n];
+	if(x && y)
 #pragma omp parallel for
-	for(long i=0;i<n;i++)	{	xc[i]=xx[1]*(x->a[i]-xx[0]);	yc[i]=xx[3]*(y->a[i]-xx[2]);	}
+		for(long i=0;i<n;i++)
+		{	xc[i]=xx[1]*(x->a[i]-xx[0]);	yc[i]=xx[3]*(y->a[i]-xx[2]);	}
+	else
 #pragma omp parallel for
-	for(long i=0;i<d->nx*d->ny*d->nz;i++) d->a[i] = NAN;
+		for(long i=0;i<n;i++)
+		{	xc[i]=xx[1]*(xdat->vthr(i)-xx[0]);	yc[i]=xx[3]*(ydat->vthr(i)-xx[2]);	}
+	long tmp = d->nx*d->ny*d->nz;
+#pragma omp parallel for
+	for(long i=0;i<tmp;i++) d->a[i] = NAN;
 
-	mglStartThread(mgl_grid_t,0,nn,d->a,xc,yc,par,0,nums->a,z->a);
+	mglStartThread(mgl_grid_t,0,nn,d->a,xc,yc,par,zdat,nums->a);
 	delete nums;	delete []xc;	delete []yc;
 }
 void MGL_EXPORT mgl_data_grid_xy_(uintptr_t *d, uintptr_t *x, uintptr_t *y, uintptr_t *z, mreal *x1, mreal *x2, mreal *y1, mreal *y2)
