@@ -27,6 +27,7 @@
 #include <gsl/gsl_sf.h>
 #include <gsl/gsl_rng.h>
 #include <gsl/gsl_errno.h>
+#include <sys/stat.h>
 #endif
 //-----------------------------------------------------------------------------
 //	constants for expression parsing
@@ -59,6 +60,7 @@ EQ_ELE,		// elliptic integral E(\phi,k) = \int_0^\phi dt   \sqrt((1 - k^2 \sin^2
 EQ_ELF,		// elliptic integral F(\phi,k) = \int_0^\phi dt 1/\sqrt((1 - k^2 \sin^2(t)))
 EQ_LP,		// Legendre polynomial P_l(x), (|x|<=1, l>=0)
 EQ_BETA,	// beta function B(x,y) = Gamma(x)*Gamma(y)/Gamma(x+y)
+EQ_GAMMA_INC,	// incomplete gamma function Gamma(a,x) = \int_x^\infty dt t^{a-1} \exp(-t) for x>=0.
 // normal functions of 1 argument
 EQ_SIN,		// sine function \sin(x).			!!! MUST BE FIRST 1-PLACE FUNCTION
 EQ_COS,		// cosine function \cos(x).
@@ -127,8 +129,8 @@ EQ_CL		// Clausen function
 #endif
 //-----------------------------------------------------------------------------
 int mglFormula::Error=0;
-bool mglCheck(char *str,int n);
-int mglFindInText(char *str,const char *lst);
+bool MGL_LOCAL_PURE mglCheck(char *str,int n);
+int MGL_LOCAL_PURE mglFindInText(char *str,const char *lst);
 //-----------------------------------------------------------------------------
 #if MGL_HAVE_GSL
 MGL_NO_EXPORT gsl_rng *mgl_rng=0;	// NOTE: should be deleted by gsl_rng_free() but I don't know where :(
@@ -148,7 +150,7 @@ void MGL_EXPORT mgl_srnd(long seed)
 }
 void MGL_EXPORT mgl_srnd_(int *seed)	{	mgl_srnd(*seed);	}
 //-----------------------------------------------------------------------------
-double MGL_EXPORT mgl_hypot(double x, double y)	{	return hypot(x,y);	}
+double MGL_EXPORT_CONST mgl_hypot(double x, double y)	{	return hypot(x,y);	}
 //-----------------------------------------------------------------------------
 #if MGL_HAVE_PTHREAD
 pthread_mutex_t mutexRnd;
@@ -266,6 +268,7 @@ mglFormula::mglFormula(const char *string)
 		{	Kod=EQ_A;	Res = str[0]-'a';	}
 		else if(!strcmp(str,"rnd")) Kod=EQ_RND;
 		else if(!strcmp(str,"pi")) Res=M_PI;
+		else if(!strcmp(str,"inf")) Res=INFINITY;
 		else Res=atof(str);				// this is number
 	}
 	else
@@ -376,6 +379,7 @@ mglFormula::mglFormula(const char *string)
 		else if(!strcmp(name,"y"))		Kod=EQ_BESY;
 		else if(!strcmp(name,"f"))		Kod=EQ_ELF;
 		else if(!strcmp(name,"gamma"))	Kod=EQ_GAMMA;
+		else if(!strcmp(name,"gamma_inc"))	Kod=EQ_GAMMA_INC;
 		else if(!strcmp(name,"ns"))		Kod=EQ_NS;
 		else if(!strcmp(name,"nc"))		Kod=EQ_NC;
 		else if(!strcmp(name,"nd"))		Kod=EQ_ND;
@@ -463,74 +467,75 @@ mreal mglFormula::CalcD(const mreal var[MGL_VS], char diff) const
 	return CalcDIn(diff-'a', var);
 }
 //-----------------------------------------------------------------------------
-double MGL_NO_EXPORT cand(double a,double b)	{return a&&b?1:0;}
-double MGL_NO_EXPORT cor(double a,double b)	{return a||b?1:0;}
-double MGL_NO_EXPORT ceq(double a,double b)	{return a==b?1:0;}
-double MGL_NO_EXPORT clt(double a,double b)	{return a<b?1:0;}
-double MGL_NO_EXPORT cgt(double a,double b)	{return a>b?1:0;}
-double MGL_NO_EXPORT add(double a,double b)	{return a+b;}
-double MGL_NO_EXPORT sub(double a,double b)	{return a-b;}
-double MGL_NO_EXPORT mul(double a,double b)	{return a&&b?a*b:0;}
-double MGL_NO_EXPORT del(double a,double b)	{return b?a/b:NAN;}
-double MGL_NO_EXPORT ipw(double a,double b)	{return fabs(b-int(b))<1e-5 ? mgl_ipow(a,int(b)) : pow(a,b);}
-double MGL_NO_EXPORT llg(double a,double b)	{return log(a)/log(b);}
+double MGL_LOCAL_CONST cand(double a,double b)	{return a&&b?1:0;}
+double MGL_LOCAL_CONST cor(double a,double b)	{return a||b?1:0;}
+double MGL_LOCAL_CONST ceq(double a,double b)	{return a==b?1:0;}
+double MGL_LOCAL_CONST clt(double a,double b)	{return a<b?1:0;}
+double MGL_LOCAL_CONST cgt(double a,double b)	{return a>b?1:0;}
+double MGL_LOCAL_CONST add(double a,double b)	{return a+b;}
+double MGL_LOCAL_CONST sub(double a,double b)	{return a-b;}
+double MGL_LOCAL_CONST mul(double a,double b)	{return a&&b?a*b:0;}
+double MGL_LOCAL_CONST del(double a,double b)	{return b?a/b:NAN;}
+double MGL_LOCAL_CONST ipw(double a,double b)	{return fabs(b-int(b))<1e-5 ? mgl_ipow(a,int(b)) : pow(a,b);}
+double MGL_LOCAL_CONST llg(double a,double b)	{return log(a)/log(b);}
 #if MGL_HAVE_GSL
-double MGL_NO_EXPORT gslEllE(double a,double b)	{return gsl_sf_ellint_E(a,b,GSL_PREC_SINGLE);}
-double MGL_NO_EXPORT gslEllF(double a,double b)	{return gsl_sf_ellint_F(a,b,GSL_PREC_SINGLE);}
-double MGL_NO_EXPORT gslLegP(double a,double b)	{return gsl_sf_legendre_Pl(int(a),b);}
-double MGL_NO_EXPORT gslEllEc(double a)	{return gsl_sf_ellint_Ecomp(a,GSL_PREC_SINGLE);}
-double MGL_NO_EXPORT gslEllFc(double a)	{return gsl_sf_ellint_Kcomp(a,GSL_PREC_SINGLE);}
-double MGL_NO_EXPORT gslAi(double a)	{return gsl_sf_airy_Ai(a,GSL_PREC_SINGLE);}
-double MGL_NO_EXPORT gslBi(double a)	{return gsl_sf_airy_Bi(a,GSL_PREC_SINGLE);}
-double MGL_NO_EXPORT gslAi_d(double a)	{return gsl_sf_airy_Ai_deriv(a,GSL_PREC_SINGLE);}
-double MGL_NO_EXPORT gslBi_d(double a)	{return gsl_sf_airy_Bi_deriv(a,GSL_PREC_SINGLE);}
+double MGL_LOCAL_CONST gslEllE(double a,double b)	{return gsl_sf_ellint_E(a,b,GSL_PREC_SINGLE);}
+double MGL_LOCAL_CONST gslEllF(double a,double b)	{return gsl_sf_ellint_F(a,b,GSL_PREC_SINGLE);}
+double MGL_LOCAL_CONST gslLegP(double a,double b)	{return gsl_sf_legendre_Pl(int(a),b);}
+double MGL_LOCAL_CONST gslEllEc(double a)	{return gsl_sf_ellint_Ecomp(a,GSL_PREC_SINGLE);}
+double MGL_LOCAL_CONST gslEllFc(double a)	{return gsl_sf_ellint_Kcomp(a,GSL_PREC_SINGLE);}
+double MGL_LOCAL_CONST gslAi(double a)	{return gsl_sf_airy_Ai(a,GSL_PREC_SINGLE);}
+double MGL_LOCAL_CONST gslBi(double a)	{return gsl_sf_airy_Bi(a,GSL_PREC_SINGLE);}
+double MGL_LOCAL_CONST gslAi_d(double a)	{return gsl_sf_airy_Ai_deriv(a,GSL_PREC_SINGLE);}
+double MGL_LOCAL_CONST gslBi_d(double a)	{return gsl_sf_airy_Bi_deriv(a,GSL_PREC_SINGLE);}
 #endif
-double MGL_NO_EXPORT sgn(double a)	{return a<0 ? -1:(a>0?1:0);}
-double MGL_NO_EXPORT stp(double a)	{return a>0 ? 1:0;}
-double MGL_NO_EXPORT arg(double a,double b)	{	return atan2(b,a);	}
-double MGL_NO_EXPORT mgz1(double)	{return 0;}
-double MGL_NO_EXPORT mgz2(double,double)	{return 0;}
+double MGL_LOCAL_CONST sgn(double a)	{return a<0 ? -1:(a>0?1:0);}
+double MGL_LOCAL_CONST stp(double a)	{return a>0 ? 1:0;}
+double MGL_LOCAL_CONST arg(double a,double b)	{	return atan2(b,a);	}
+double MGL_LOCAL_CONST mgz1(double)			{return NAN;}	// NOTE I think NAN value is more correct here than 0
+double MGL_LOCAL_CONST mgz2(double,double)	{return NAN;}	// NOTE I think NAN value is more correct here than 0
 #ifdef WIN32
-double MGL_NO_EXPORT asinh(double x)	{	return log(x+sqrt(x*x+1.));	}
-double MGL_NO_EXPORT acosh(double x)	{	return x>1 ? log(x+sqrt(x*x-1.)) : NAN;	}
-double MGL_NO_EXPORT atanh(double x)	{	return fabs(x)<1 ? log((1.+x)/(1.-x))/2 : NAN;	}
+double MGL_LOCAL_CONST asinh(double x)	{	return log(x+sqrt(x*x+1.));	}
+double MGL_LOCAL_CONST acosh(double x)	{	return x>1 ? log(x+sqrt(x*x-1.)) : NAN;	}
+double MGL_LOCAL_CONST atanh(double x)	{	return fabs(x)<1 ? log((1.+x)/(1.-x))/2 : NAN;	}
 #endif
 //-----------------------------------------------------------------------------
 typedef double (*func_1)(double);
 typedef double (*func_2)(double, double);
+//-----------------------------------------------------------------------------
+static const mreal z2[EQ_SIN-EQ_LT] = {3,3,3,3,0,3,3,0,0,0,0,0,NAN,0
+#if MGL_HAVE_GSL
+	,3,NAN, 3,NAN, 0,0,3,1,3
+#else
+	,0,0,0,0,0,0,0,0,0
+#endif
+};
+static const func_2 f2[EQ_SIN-EQ_LT] = {clt,cgt,ceq,cor,cand,add,sub,mul,del,ipw,pow,fmod,llg,arg,hypot
+#if MGL_HAVE_GSL
+	,gsl_sf_bessel_Jnu,gsl_sf_bessel_Ynu,
+	gsl_sf_bessel_Inu,gsl_sf_bessel_Knu,
+	gslEllE,gslEllF,gslLegP,gsl_sf_beta,gsl_sf_gamma_inc,
+#else
+	,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2
+#endif
+};
+static const func_1 f1[EQ_SN-EQ_SIN] = {sin,cos,tan,asin,acos,atan,sinh,cosh,tanh,
+			asinh,acosh,atanh,sqrt,exp,log,log10,sgn,stp,floor,fabs
+#if MGL_HAVE_GSL
+	,gsl_sf_dilog,gslEllEc,gslEllFc,gslAi,gslBi,gsl_sf_erf,
+	gsl_sf_expint_3,gsl_sf_expint_Ei,gsl_sf_expint_E1,gsl_sf_expint_E2,
+	gsl_sf_Si,gsl_sf_Ci,gsl_sf_gamma,gsl_sf_psi,gsl_sf_lambert_W0,
+	gsl_sf_lambert_Wm1,gsl_sf_sinc,gsl_sf_zeta,gsl_sf_eta,gslAi_d,gslBi_d,
+	gsl_sf_dawson
+#else
+	,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,
+	mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1
+#endif
+};
+//-----------------------------------------------------------------------------
 // evaluation of embedded (included) expressions
 mreal mglFormula::CalcIn(const mreal *a1) const
 {
-	mreal z2[EQ_SIN-EQ_LT] = {3,3,3,3,0,3,3,0,0,0,0,0,NAN,0
-#if MGL_HAVE_GSL
-			,3,NAN, 3,NAN, 0,0,3,1
-#else
-			,0,0,0,0,0,0,0,0
-#endif
-		};
-	func_2 f2[EQ_SIN-EQ_LT] = {clt,cgt,ceq,cor,cand,add,sub,mul,del,ipw,pow,fmod,llg,arg,hypot
-#if MGL_HAVE_GSL
-			,gsl_sf_bessel_Jnu,gsl_sf_bessel_Ynu,
-			gsl_sf_bessel_Inu,gsl_sf_bessel_Knu,
-			gslEllE,gslEllF,gslLegP,gsl_sf_beta
-#else
-			,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2
-#endif
-		};
-	func_1 f1[EQ_SN-EQ_SIN] = {sin,cos,tan,asin,acos,atan,sinh,cosh,tanh,
-					asinh,acosh,atanh,sqrt,exp,log,log10,sgn,stp,floor,fabs
-#if MGL_HAVE_GSL
-			,gsl_sf_dilog,gslEllEc,gslEllFc,gslAi,gslBi,gsl_sf_erf,
-			gsl_sf_expint_3,gsl_sf_expint_Ei,gsl_sf_expint_E1,gsl_sf_expint_E2,
-			gsl_sf_Si,gsl_sf_Ci,gsl_sf_gamma,gsl_sf_psi,gsl_sf_lambert_W0,
-			gsl_sf_lambert_Wm1,gsl_sf_sinc,gsl_sf_zeta,gsl_sf_eta,gslAi_d,gslBi_d,
-			gsl_sf_dawson
-#else
-			,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,
-			mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1
-#endif
-		};
-//	if(Error)	return 0;
 	if(Kod<EQ_LT)
 	{
 		if(Kod==EQ_RND)	return mgl_rnd();
@@ -548,7 +553,7 @@ mreal mglFormula::CalcIn(const mreal *a1) const
 			return mgl_isfin(b) ? b : NAN;
 		}
 		else if(Kod<EQ_SN)
-		{	a = f1[Kod-EQ_SIN](a);	return mgl_isfin(a)?a:NAN;	}	
+		{	a = f1[Kod-EQ_SIN](a);	return mgl_isfin(a)?a:NAN;	}
 #if MGL_HAVE_GSL
 		else if(Kod<=EQ_DC)
 		{
@@ -576,79 +581,82 @@ mreal mglFormula::CalcIn(const mreal *a1) const
 	return NAN;
 }
 //-----------------------------------------------------------------------------
-double MGL_NO_EXPORT mgp(double ,double )	{return 1;}
-double MGL_NO_EXPORT mgm(double ,double )	{return -1;}
-double MGL_NO_EXPORT mul1(double ,double b)	{return b;}
-double MGL_NO_EXPORT mul2(double a,double )	{return a;}
-double MGL_NO_EXPORT div1(double ,double b)	{return b?1/b:NAN;}
-double MGL_NO_EXPORT div2(double a,double b)	{return b?-a/(b*b):NAN;}
-double MGL_NO_EXPORT ipw1(double a,double b)	{return b*(fabs(b-int(b))<1e-5 ? mgl_ipow(a,int(b-1)) : pow(a,b-1));}
-double MGL_NO_EXPORT pow1(double a,double b)	{return b*pow(a,b-1);}
-double MGL_NO_EXPORT pow2(double a,double b)	{return log(a)*pow(a,b);}
-double MGL_NO_EXPORT llg1(double a,double b)	{return 1/(a*log(b));}
-double MGL_NO_EXPORT llg2(double a,double b)	{return -log(a)/(b*log(b)*log(b));}
-double MGL_NO_EXPORT cos_d(double a)	{return -sin(a);}
-double MGL_NO_EXPORT tan_d(double a)	{return 1./(cos(a)*cos(a));}
-double MGL_NO_EXPORT asin_d(double a)	{return 1./sqrt(1.-a*a);}
-double MGL_NO_EXPORT acos_d(double a)	{return -1./sqrt(1.-a*a);}
-double MGL_NO_EXPORT atan_d(double a)	{return 1./(1.+a*a);}
-double MGL_NO_EXPORT tanh_d(double a)	{return 1./(cosh(a)*cosh(a));}
-double MGL_NO_EXPORT atanh_d(double a){return 1./(1.-a*a);}
-double MGL_NO_EXPORT asinh_d(double a){return 1./sqrt(1.+a*a);}
-double MGL_NO_EXPORT acosh_d(double a){return 1./sqrt(a*a-1.);}
-double MGL_NO_EXPORT sqrt_d(double a)	{return 0.5/sqrt(a);}
-double MGL_NO_EXPORT log10_d(double a){return M_LN10/a;}
-double MGL_NO_EXPORT log_d(double a)	{return 1./a;}
-double MGL_NO_EXPORT erf_d(double a)	{return 2*exp(-a*a)/sqrt(M_PI);}
-double MGL_NO_EXPORT dilog_d(double a){return log(a)/(1.-a);}
-double MGL_NO_EXPORT ei_d(double a)	{return exp(a)/a;}
-double MGL_NO_EXPORT si_d(double a)	{return a?sin(a)/a:1;}
-double MGL_NO_EXPORT ci_d(double a)	{return cos(a)/a;}
-double MGL_NO_EXPORT exp3_d(double a)	{return exp(-a*a*a);}
-double MGL_NO_EXPORT e1_d(double a)	{return exp(-a)/a;}
-double MGL_NO_EXPORT sinc_d(double a)	{return a ? (cos(M_PI*a)/a-sin(M_PI*a)/(M_PI*a*a)) : 0;}
+double MGL_LOCAL_CONST mgzz(double,double)	{return 0;}
+double MGL_LOCAL_CONST mgp(double ,double )	{return 1;}
+double MGL_LOCAL_CONST mgm(double ,double )	{return -1;}
+double MGL_LOCAL_CONST mul1(double ,double b)	{return b;}
+double MGL_LOCAL_CONST mul2(double a,double )	{return a;}
+double MGL_LOCAL_CONST div1(double ,double b)	{return b?1/b:NAN;}
+double MGL_LOCAL_CONST div2(double a,double b)	{return b?-a/(b*b):NAN;}
+double MGL_LOCAL_CONST ipw1(double a,double b)	{return b*(fabs(b-int(b))<1e-5 ? mgl_ipow(a,int(b-1)) : pow(a,b-1));}
+double MGL_LOCAL_CONST pow1(double a,double b)	{return b*pow(a,b-1);}
+double MGL_LOCAL_CONST pow2(double a,double b)	{return log(a)*pow(a,b);}
+double MGL_LOCAL_CONST llg1(double a,double b)	{return 1/(a*log(b));}
+double MGL_LOCAL_CONST llg2(double a,double b)	{return -log(a)/(b*log(b)*log(b));}
+double MGL_LOCAL_CONST cos_d(double a)	{return -sin(a);}
+double MGL_LOCAL_CONST tan_d(double a)	{return 1./(cos(a)*cos(a));}
+double MGL_LOCAL_CONST asin_d(double a)	{return 1./sqrt(1.-a*a);}
+double MGL_LOCAL_CONST acos_d(double a)	{return -1./sqrt(1.-a*a);}
+double MGL_LOCAL_CONST atan_d(double a)	{return 1./(1.+a*a);}
+double MGL_LOCAL_CONST tanh_d(double a)	{return 1./(cosh(a)*cosh(a));}
+double MGL_LOCAL_CONST atanh_d(double a){return 1./(1.-a*a);}
+double MGL_LOCAL_CONST asinh_d(double a){return 1./sqrt(1.+a*a);}
+double MGL_LOCAL_CONST acosh_d(double a){return 1./sqrt(a*a-1.);}
+double MGL_LOCAL_CONST sqrt_d(double a)	{return 0.5/sqrt(a);}
+double MGL_LOCAL_CONST log10_d(double a){return M_LN10/a;}
+double MGL_LOCAL_CONST log_d(double a)	{return 1./a;}
+double MGL_LOCAL_CONST erf_d(double a)	{return 2*exp(-a*a)/sqrt(M_PI);}
+double MGL_LOCAL_CONST dilog_d(double a){return log(a)/(1.-a);}
+double MGL_LOCAL_CONST ei_d(double a)	{return exp(a)/a;}
+double MGL_LOCAL_CONST si_d(double a)	{return a?sin(a)/a:1;}
+double MGL_LOCAL_CONST ci_d(double a)	{return cos(a)/a;}
+double MGL_LOCAL_CONST exp3_d(double a)	{return exp(-a*a*a);}
+double MGL_LOCAL_CONST e1_d(double a)	{return exp(-a)/a;}
+double MGL_LOCAL_CONST sinc_d(double a)	{return a ? (cos(M_PI*a)/a-sin(M_PI*a)/(M_PI*a*a)) : 0;}
 #if MGL_HAVE_GSL
-double MGL_NO_EXPORT e2_d(double a)	{return -gsl_sf_expint_E1(a);}
-double MGL_NO_EXPORT gslJnuD(double a,double b)	{return 0.5*(gsl_sf_bessel_Jnu(a-1,b)-gsl_sf_bessel_Jnu(a+1,b));}
-double MGL_NO_EXPORT gslYnuD(double a,double b)	{return 0.5*(gsl_sf_bessel_Ynu(a-1,b)-gsl_sf_bessel_Ynu(a+1,b));}
-double MGL_NO_EXPORT gslKnuD(double a,double b)	{return -(a*gsl_sf_bessel_Knu(a,b)/b +gsl_sf_bessel_Knu(a-1,b));}
-double MGL_NO_EXPORT gslInuD(double a,double b)	{return -(a*gsl_sf_bessel_Inu(a,b)/b -gsl_sf_bessel_Inu(a-1,b));}
-double MGL_NO_EXPORT gslEllE1(double a,double b)	{return sqrt(1.-sin(a)*sin(a)*b);}
-double MGL_NO_EXPORT gslEllE2(double a,double b)	{return (gsl_sf_ellint_E(a,b,GSL_PREC_SINGLE) - gsl_sf_ellint_F(a,b,GSL_PREC_SINGLE))/(2.*b);}
-double MGL_NO_EXPORT gslEllF1(double a,double b)	{return 1./sqrt(1.-sin(a)*sin(a)*b);}
-double MGL_NO_EXPORT gslEllF2(double a,double b)	{return (gsl_sf_ellint_E(a,b,GSL_PREC_SINGLE) - gsl_sf_ellint_F(a,b,GSL_PREC_SINGLE)*(1.-b))/(2*b*(1.-b)) - sin(2.*a)/(sqrt(1.-sin(a)*sin(a)*b)*2.*(1.-b));}
-double MGL_NO_EXPORT gslE_d(double a)	{return (gsl_sf_ellint_Ecomp(a,GSL_PREC_SINGLE) - gsl_sf_ellint_Kcomp(a,GSL_PREC_SINGLE))/(2.*a);}
-double MGL_NO_EXPORT gslK_d(double a)	{return (gsl_sf_ellint_Ecomp(a,GSL_PREC_SINGLE) - (1.-a)*gsl_sf_ellint_Kcomp(a,GSL_PREC_SINGLE))/(2.*a*(1.-a));}
-double MGL_NO_EXPORT gamma_d(double a)	{return gsl_sf_psi(a)*gsl_sf_gamma(a);}
+double MGL_LOCAL_CONST e2_d(double a)	{return -gsl_sf_expint_E1(a);}
+double MGL_LOCAL_CONST gslJnuD(double a,double b)	{return 0.5*(gsl_sf_bessel_Jnu(a-1,b)-gsl_sf_bessel_Jnu(a+1,b));}
+double MGL_LOCAL_CONST gslYnuD(double a,double b)	{return 0.5*(gsl_sf_bessel_Ynu(a-1,b)-gsl_sf_bessel_Ynu(a+1,b));}
+double MGL_LOCAL_CONST gslKnuD(double a,double b)	{return -(a*gsl_sf_bessel_Knu(a,b)/b +gsl_sf_bessel_Knu(a-1,b));}
+double MGL_LOCAL_CONST gslInuD(double a,double b)	{return -(a*gsl_sf_bessel_Inu(a,b)/b -gsl_sf_bessel_Inu(a-1,b));}
+double MGL_LOCAL_CONST gslEllE1(double a,double b)	{return sqrt(1.-sin(a)*sin(a)*b);}
+double MGL_LOCAL_CONST gslEllE2(double a,double b)	{return (gsl_sf_ellint_E(a,b,GSL_PREC_SINGLE) - gsl_sf_ellint_F(a,b,GSL_PREC_SINGLE))/(2.*b);}
+double MGL_LOCAL_CONST gslEllF1(double a,double b)	{return 1./sqrt(1.-sin(a)*sin(a)*b);}
+double MGL_LOCAL_CONST gslEllF2(double a,double b)	{return (gsl_sf_ellint_E(a,b,GSL_PREC_SINGLE) - gsl_sf_ellint_F(a,b,GSL_PREC_SINGLE)*(1.-b))/(2*b*(1.-b)) - sin(2.*a)/(sqrt(1.-sin(a)*sin(a)*b)*2.*(1.-b));}
+double MGL_LOCAL_CONST gslE_d(double a)	{return (gsl_sf_ellint_Ecomp(a,GSL_PREC_SINGLE) - gsl_sf_ellint_Kcomp(a,GSL_PREC_SINGLE))/(2.*a);}
+double MGL_LOCAL_CONST gslK_d(double a)	{return (gsl_sf_ellint_Ecomp(a,GSL_PREC_SINGLE) - (1.-a)*gsl_sf_ellint_Kcomp(a,GSL_PREC_SINGLE))/(2.*a*(1.-a));}
+double MGL_LOCAL_CONST gamma_d(double a)	{return gsl_sf_psi(a)*gsl_sf_gamma(a);}
 #endif
+double MGL_LOCAL_CONST ginc_d(double a, double x)	{return -exp(-x)*pow(x,a-1);}
+//-----------------------------------------------------------------------------
+static const func_2 f21[EQ_SIN-EQ_LT] = {mgzz,mgzz,mgzz, mgzz,mgzz,mgp, mgp,mul1,div1, ipw1,pow1,mgp,llg1, mgz2
+#if MGL_HAVE_GSL
+	,mgz2,mgz2,mgz2, mgz2,gslEllE1,gslEllF1, mgz2,mgz2,mgz2
+#else
+	,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2
+#endif
+};
+static const func_2 f22[EQ_SIN-EQ_LT] = {mgzz,mgzz,mgzz,mgzz,mgzz,mgp,mgm,mul2,div2,pow2,pow2,mgz2,llg2, mgz2
+#if MGL_HAVE_GSL
+	,gslJnuD,gslYnuD,gslInuD,gslKnuD,gslEllE2,gslEllF2,mgz2/*gslLegP*/,mgz2,ginc_d
+#else
+	,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2
+#endif
+};
+static const func_1 f11[EQ_SN-EQ_SIN] = {cos,cos_d,tan_d,asin_d,acos_d,atan_d,cosh,sinh,tanh_d,
+	asinh_d,acosh_d,atanh_d,sqrt_d,exp,log_d,log10_d,mgz1,mgz1,mgz1,sgn
+#if MGL_HAVE_GSL
+	,dilog_d,gslE_d,gslK_d,gslAi_d,gslBi_d,erf_d,exp3_d,ei_d,e1_d,e2_d,
+	si_d,ci_d,gamma_d,gsl_sf_psi_1,mgz1,mgz1,sinc_d,mgz1,mgz1,mgz1,mgz1,mgz1
+#else
+	,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,
+	mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1
+#endif
+};
 //-----------------------------------------------------------------------------
 // evaluation of derivative of embedded (included) expressions
 mreal mglFormula::CalcDIn(int id, const mreal *a1) const
 {
-	func_2 f21[EQ_SIN-EQ_LT] = {mgz2,mgz2,mgz2, mgz2,mgz2,mgp, mgp,mul1,div1, ipw1,pow1,mgp,llg1, mgz2
-#if MGL_HAVE_GSL
-			,mgz2,mgz2,mgz2, mgz2,gslEllE1,gslEllF2, mgz2,mgz2
-#else
-			,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2
-#endif
-		};
-		func_2 f22[EQ_SIN-EQ_LT] = {mgz2,mgz2,mgz2,mgz2,mgz2,mgp,mgm,mul2,div2,pow2,pow2,mgz2,llg2, mgz2
-#if MGL_HAVE_GSL
-			,gslJnuD,gslYnuD,gslInuD,gslKnuD,gslEllE2,gslEllF2,mgz2/*gslLegP*/,mgz2
-#else
-			,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2
-#endif
-		};
-	func_1 f11[EQ_SN-EQ_SIN] = {cos,cos_d,tan_d,asin_d,acos_d,atan_d,cosh,sinh,tanh_d,
-					asinh_d,acosh_d,atanh_d,sqrt_d,exp,log_d,log10_d,mgz1,mgz1,mgz1,sgn
-#if MGL_HAVE_GSL
-			,dilog_d,gslE_d,gslK_d,gslAi_d,gslBi_d,erf_d,exp3_d,ei_d,e1_d,e2_d,
-			si_d,ci_d,gamma_d,gsl_sf_psi_1,mgz1,mgz1,sinc_d,mgz1,mgz1,mgz1,mgz1,mgz1
-#else
-			,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,
-			mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1,mgz1
-#endif
-		};
 	if(Kod<EQ_LT)	return (Kod==EQ_A && id==(int)Res)?1:0;
 
 	double a = Left->CalcIn(a1), d = Left->CalcDIn(id,a1);
@@ -690,7 +698,7 @@ mreal mglFormula::CalcDIn(int id, const mreal *a1) const
 }
 //-----------------------------------------------------------------------------
 // Check braces correctness
-bool MGL_NO_EXPORT mglCheck(char *str,int n)
+bool MGL_LOCAL_PURE mglCheck(char *str,int n)
 {
 	register long s = 0,i;
 	for(i=0;i<n;i++)
@@ -703,7 +711,7 @@ bool MGL_NO_EXPORT mglCheck(char *str,int n)
 }
 //-----------------------------------------------------------------------------
 // Try to find one of symbols lst in the string str
-int MGL_NO_EXPORT mglFindInText(char *str,const char *lst)
+int MGL_LOCAL_PURE mglFindInText(char *str,const char *lst)
 {
 	register long l=0,r=0,i;//,j,len=strlen(lst);
 	for(i=strlen(str)-1;i>=0;i--)
@@ -716,12 +724,12 @@ int MGL_NO_EXPORT mglFindInText(char *str,const char *lst)
 }
 //-----------------------------------------------------------------------------
 HMEX MGL_EXPORT mgl_create_expr(const char *expr)	{	return new mglFormula(expr);	}
-void MGL_EXPORT mgl_delete_expr(HMEX ex)	{	delete ex;	}
-double MGL_EXPORT mgl_expr_eval(HMEX ex, double x, double y,double z)
+void MGL_EXPORT mgl_delete_expr(HMEX ex)	{	if(ex)	delete ex;	}
+double MGL_EXPORT_PURE mgl_expr_eval(HMEX ex, double x, double y,double z)
 {	return ex->Calc(x,y,z);	}
 double MGL_EXPORT mgl_expr_eval_v(HMEX ex, mreal *var)
 {	return ex->Calc(var);	}
-double MGL_EXPORT mgl_expr_diff(HMEX ex, char dir, double x, double y,double z)
+double MGL_EXPORT_PURE mgl_expr_diff(HMEX ex, char dir, double x, double y,double z)
 {	return ex->CalcD(dir,x,y,z);	}
 double MGL_EXPORT mgl_expr_diff_v(HMEX ex, char dir, mreal *var)
 {	return ex->CalcD(var, dir);		}
