@@ -27,27 +27,23 @@ void MGL_NO_EXPORT mgl_mesh_plot(mglBase *gr, long *pos, long n, long m, int how
 {
 	int d = gr->MeshNum>0 ? gr->MeshNum+1 : n*m, dx = n>d?n/d:1, dy = m>d?m/d:1;
 	// NOTE: number of lines in each direction can be reduced too
-	if(how&1)
-#pragma omp parallel for
-		for(long j=0;j<m;j+=dy)
-		{
-			register long s,i;
-			for(s=i=0;i<n-1;i++)	if(pos[n*j+i]>=0 && pos[n*j+i+1]>=0)	s++;
-			d = gr->FaceNum>0 ? gr->FaceNum+1 : n;	s = s>d?s/d:1;
-			for(i=0;i<n-s;i+=s)
-				gr->line_plot(pos[n*j+i],pos[n*j+i+s]);
+	if(how&1)	for(long j=0;j<m;j+=dy)
+	{
+		register long s,i;
+		for(s=i=0;i<n-1;i++)	if(pos[n*j+i]>=0 && pos[n*j+i+1]>=0)	s++;
+		d = gr->FaceNum>0 ? gr->FaceNum+1 : n;	s = s>d?s/d:1;
+		for(i=0;i<n-s;i+=s)
+			gr->line_plot(pos[n*j+i],pos[n*j+i+s]);
 
-		}
-	if(how&2)
-#pragma omp parallel for
-		for(long i=0;i<n;i+=dx)
-		{
-			register long s,j;
-			for(s=j=0;j<m-1;j++)	if(pos[n*j+i]>=0 && pos[n*j+i+n]>=0)	s++;
-			d = gr->FaceNum>0 ? gr->FaceNum+1 : n;	s = s>d?s/d:1;
-			for(j=0;j<m-s;j+=s)
-				gr->line_plot(pos[n*j+i],pos[n*j+i+s*n]);
-		}
+	}
+	if(how&2)	for(long i=0;i<n;i+=dx)
+	{
+		register long s,j;
+		for(s=j=0;j<m-1;j++)	if(pos[n*j+i]>=0 && pos[n*j+i+n]>=0)	s++;
+		d = gr->FaceNum>0 ? gr->FaceNum+1 : n;	s = s>d?s/d:1;
+		for(j=0;j<m-s;j+=s)
+			gr->line_plot(pos[n*j+i],pos[n*j+i+s*n]);
+	}
 }
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_surf_plot(mglBase *gr, long *pos, long n, long m)
@@ -62,7 +58,6 @@ void MGL_NO_EXPORT mgl_surf_plot(mglBase *gr, long *pos, long n, long m)
 		int d = gr->FaceNum+1,ns=n*s/((n-1)*(m-1)),ms=m*s/((n-1)*(m-1));
 		dx = ns>d?ns/d:1;		dy = ms>d?ms/d:1;
 	}
-#pragma omp parallel for collapse(2)
 	for(long j=0;j<m-dy;j+=dy)	for(long i=0;i<n-dx;i+=dx)
 		gr->quad_plot(pos[n*j+i],pos[n*j+i+dx],pos[n*j+i+n*dy],pos[n*j+i+n*dy+dx]);
 }
@@ -71,46 +66,42 @@ void MGL_NO_EXPORT mgl_surf_plot(mglBase *gr, long *pos, long n, long m)
 //	Plot by formulas series
 //
 //-----------------------------------------------------------------------------
+mglData MGL_NO_EXPORT mglFormulaCalc(const char *str, const std::vector<mglDataA*> &head);
+//-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_fsurf(HMGL gr, const char *eqZ, const char *sch, const char *opt)
-{	// TODO: Add strong function variation analysis ???
+{	// NOTE Strong function variation analysis can be added here
 	if(eqZ==0 || eqZ[0]==0)	return;		// nothing to plot
 	mreal r = gr->SaveState(opt);
 	long n = (mgl_isnan(r) || r<=0) ? 100:long(r+0.5);
-	mglData z(n,n);
-	mglFormula *eq = new mglFormula(eqZ);
-	mreal dx = (gr->Max.x - gr->Min.x)/(n-1.), dy = (gr->Max.y - gr->Min.y)/(n-1.);
-#pragma omp parallel for collapse(2)
-	for(long j=0;j<n;j++)	for(long i=0;i<n;i++)
-	{
-		if(gr->Stop)	continue;
-		z.a[i+n*j] = eq->Calc(gr->Min.x+i*dx, gr->Min.y+j*dy);
-	}
+	mglData z(n,n),res;
+	mglDataV x(n,n,1, gr->Min.x,gr->Max.x,'x');	x.s=L"x";
+	mglDataV y(n,n,1, gr->Min.y,gr->Max.y,'y');	y.s=L"y";
+	mglDataV t(n,n);	t.s=L"#$mgl";
+	std::vector<mglDataA*> list;
+	list.push_back(&x);	list.push_back(&y);	list.push_back(&t);
+	res.Set(mglFormulaCalc(eqZ,list));
+	if(res.nx==1 && res.ny==1)	z = res.a[0];	else	z = res;
 	mgl_surf(gr, &z, sch,0);
-	delete eq;
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_fsurf_xyz(HMGL gr, const char *eqX, const char *eqY, const char *eqZ, const char *sch, const char *opt)
-{	// TODO: Add strong function variation analisys ???
+{	// NOTE Strong function variation analysis can be added here
 	if(eqZ==0 || eqZ[0]==0)	return;		// nothing to plot
 	mreal r = gr->SaveState(opt);
 	long n = (mgl_isnan(r) || r<=0) ? 100:long(r+0.5);
-	mglData x(n,n), y(n,n), z(n,n);
-	if(n<=0)	n=100;
-	mglFormula *ex, *ey, *ez;
-	ex = new mglFormula(eqX ? eqX : "u");
-	ey = new mglFormula(eqY ? eqY : "v");
-	ez = new mglFormula(eqZ);
-#pragma omp parallel for collapse(2)
-	for(long j=0;j<n;j++)	for(long i=0;i<n;i++)
-	{
-		if(gr->Stop)	continue;
-		register mreal u = j/(n-1.), v = i/(n-1.);
-		x.a[i+n*j] = ex->Calc(0,v,0,u);
-		y.a[i+n*j] = ey->Calc(0,v,0,u);
-		z.a[i+n*j] = ez->Calc(0,v,0,u);
-	}
+	mglData z(n,n), x(n,n), y(n,n), res;
+	mglDataV u(n,n,1, 0,1,'x');	u.s=L"u";
+	mglDataV v(n,n,1, 0,1,'y');	v.s=L"v";
+	mglDataV t(n,n);	t.s=L"#$mgl";
+	std::vector<mglDataA*> list;
+	list.push_back(&u);	list.push_back(&v);	list.push_back(&t);
+	res.Set(mglFormulaCalc(eqX,list));
+	if(res.nx==1 && res.ny==1)	x = res.a[0];	else	x = res;
+	res.Set(mglFormulaCalc(eqY,list));
+	if(res.nx==1 && res.ny==1)	y = res.a[0];	else	y = res;
+	res.Set(mglFormulaCalc(eqZ,list));
+	if(res.nx==1 && res.ny==1)	z = res.a[0];	else	z = res;
 	mgl_surf_xy(gr,&x,&y,&z,sch,0);
-	delete ex;	delete ey;	delete ez;
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_fsurf_(uintptr_t *gr, const char *fy, const char *stl, const char *opt, int ly, int ls, int lo)
@@ -148,10 +139,9 @@ void MGL_EXPORT mgl_mesh_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 
 	for(long k=0;k<z->GetNz();k++)
 	{
-#pragma omp parallel for collapse(2)
+		if(gr->NeedStop())	break;
 		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	continue;
 			register mreal zz = z->v(i,j,k);
 			pos[i+n*j] = gr->AddPnt(mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, zz),gr->GetC(ss,zz));
 		}
@@ -163,7 +153,7 @@ void MGL_EXPORT mgl_mesh_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 void MGL_EXPORT mgl_mesh(HMGL gr, HCDT z, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(z->GetNx()), y(z->GetNy());
+	mglDataV x(z->GetNx()), y(z->GetNy());
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_mesh_xy(gr,&x,&y,z,sch,0);
@@ -197,10 +187,9 @@ void MGL_EXPORT mgl_fall_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 
 	for(long k=0;k<z->GetNz();k++)
 	{
-#pragma omp parallel for collapse(2)
+		if(gr->NeedStop())	break;
 		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	continue;
 			register mreal zz = z->v(i,j,k);
 			pos[i+n*j] = gr->AddPnt(mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, zz),gr->GetC(ss,zz));
 		}
@@ -212,7 +201,7 @@ void MGL_EXPORT mgl_fall_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 void MGL_EXPORT mgl_fall(HMGL gr, HCDT z, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(z->GetNx()), y(z->GetNy());
+	mglDataV x(z->GetNx()), y(z->GetNy());
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_fall_xy(gr,&x,&y,z,sch,0);
@@ -246,13 +235,10 @@ void MGL_EXPORT mgl_grid_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 
 	for(long k=0;k<z->GetNz();k++)
 	{
+		if(gr->NeedStop())	break;
 		if(z->GetNz()>1)	zVal = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(k)/(z->GetNz()-1);
-#pragma omp parallel for collapse(2)
 		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
-		{
-			if(gr->Stop)	continue;
 			pos[i+n*j] = gr->AddPnt(mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, zVal),gr->CDef);
-		}
 		mgl_mesh_plot(gr,pos,n,m,3);
 	}
 	delete []pos;	gr->EndGroup();
@@ -261,7 +247,7 @@ void MGL_EXPORT mgl_grid_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 void MGL_EXPORT mgl_grid(HMGL gr, HCDT z,const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(z->GetNx()), y(z->GetNy());
+	mglDataV x(z->GetNx()), y(z->GetNy());
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_grid_xy(gr,&x,&y,z,sch,0);
@@ -296,10 +282,9 @@ void MGL_EXPORT mgl_surf_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 	mglPoint p,q,s,xx,yy;
 	for(long k=0;k<z->GetNz();k++)
 	{
-#pragma omp parallel for private(p,q,s,xx,yy) collapse(2)
+		if(gr->NeedStop())	break;
 		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	continue;
 			xx = GetX(x,i,j,k);		yy = GetY(y,i,j,k);
 			p = mglPoint(xx.x, yy.x, z->v(i,j,k));
 			q = mglPoint(xx.y, yy.y, z->dvx(i,j,k));
@@ -307,13 +292,11 @@ void MGL_EXPORT mgl_surf_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 			pos[i+n*j] = gr->AddPnt(p,gr->GetC(ss,p.z),q^s);
 		}
 		if(sch && mglchr(sch,'.'))
-#pragma omp parallel for
 			for(long i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
 		else	mgl_surf_plot(gr,pos,n,m);
-		if(wire && !gr->Stop)
+		if(wire)
 		{
 			gr->SetPenPal("k-");
-#pragma omp parallel for
 			for(long i=0;i<n*m;i++)	pos[i] = gr->CopyNtoC(pos[i],gr->CDef);
 			mgl_mesh_plot(gr,pos,n,m,3);
 		}
@@ -324,7 +307,7 @@ void MGL_EXPORT mgl_surf_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 void MGL_EXPORT mgl_surf(HMGL gr, HCDT z, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(z->GetNx()), y(z->GetNy());
+	mglDataV x(z->GetNx()), y(z->GetNy());
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_surf_xy(gr,&x,&y,z,sch,0);
@@ -360,12 +343,11 @@ void MGL_EXPORT mgl_belt_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 	mglPoint p1,p2,q,s,xx,yy;
 	for(long k=0;k<z->GetNz();k++)
 	{
+		if(gr->NeedStop())	break;
 		if(how)	for(long i=0;i<n-dx;i+=dx)
 		{
-#pragma omp parallel for private(p1,p2,xx,yy,q,s)
 			for(long j=0;j<m;j++)
 			{
-				if(gr->Stop)	continue;
 				xx = GetX(x,i,j,k);		yy = GetY(y,i,j,k);
 				p1 = mglPoint(xx.x, yy.x, z->v(i,j,k));
 				s = mglPoint(xx.z, yy.z, z->dvy(i,j,k));
@@ -379,10 +361,8 @@ void MGL_EXPORT mgl_belt_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 		}
 		else	for(long j=0;j<m-dy;j+=dy)
 		{
-#pragma omp parallel for private(p1,p2,xx,yy,q,s)
 			for(long i=0;i<n;i++)	// ñîçäàåì ìàññèâ òî÷åê
 			{
-				if(gr->Stop)	continue;
 				xx = GetX(x,i,j,k);		yy = GetY(y,i,j,k);
 				p1 = mglPoint(xx.x, yy.x, z->v(i,j,k));
 				q = mglPoint(xx.y, yy.y, z->dvx(i,j,k));
@@ -401,7 +381,7 @@ void MGL_EXPORT mgl_belt_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 void MGL_EXPORT mgl_belt(HMGL gr, HCDT z, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(z->GetNx()), y(z->GetNy());
+	mglDataV x(z->GetNx()), y(z->GetNy());
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_belt_xy(gr,&x,&y,z,sch,0);
@@ -439,25 +419,22 @@ void MGL_EXPORT mgl_dens_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 	mreal zz, c;
 	for(long k=0;k<z->GetNz();k++)
 	{
+		if(gr->NeedStop())	break;
 		if(z->GetNz()>1)
 			zVal = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(k)/(z->GetNz()-1);
-#pragma omp parallel for private(p,c,zz) collapse(2)
 		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)	// ñîçäàåì ìàññèâ òî÷åê
 		{
-			if(gr->Stop)	continue;
 			p = mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, zVal);
 			zz = z->v(i,j,k);	c = gr->GetC(ss,zz);
 			if(mgl_isnan(zz))	p.x = NAN;
 			pos[i+n*j] = gr->AddPnt(p,c,s);
 		}
 		if(sch && mglchr(sch,'.'))
-#pragma omp parallel for
 			for(long i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
 		else	mgl_surf_plot(gr,pos,n,m);
-		if(wire && !gr->Stop)
+		if(wire)
 		{
 			gr->SetPenPal("k-");
-#pragma omp parallel for
 			for(long i=0;i<n*m;i++)	pos[i] = gr->CopyNtoC(pos[i],gr->CDef);
 			mgl_mesh_plot(gr,pos,n,m,3);
 		}
@@ -468,7 +445,7 @@ void MGL_EXPORT mgl_dens_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 void MGL_EXPORT mgl_dens(HMGL gr, HCDT z, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(z->GetNx()), y(z->GetNy());
+	mglDataV x(z->GetNx()), y(z->GetNy());
 	x.Fill(gr->Min.x, gr->Max.x);
 	y.Fill(gr->Min.y, gr->Max.y);
 	mgl_dens_xy(gr,&x,&y,z,sch,0);
@@ -523,10 +500,9 @@ void MGL_EXPORT mgl_surfc_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT c, const char
 	mglPoint p,q,s,xx,yy;
 	for(long k=0;k<z->GetNz();k++)
 	{
-#pragma omp parallel for private(p,xx,yy,q,s) collapse(2)
+		if(gr->NeedStop())	break;
 		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	continue;
 			xx = GetX(x,i,j,k);		yy = GetY(y,i,j,k);
 			p = mglPoint(xx.x, yy.x, z->v(i,j,k));
 			q = mglPoint(xx.y, yy.y, z->dvx(i,j,k));
@@ -534,13 +510,11 @@ void MGL_EXPORT mgl_surfc_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT c, const char
 			pos[i+n*j] = gr->AddPnt(p,gr->GetC(ss,c->v(i,j,k)),q^s);
 		}
 		if(sch && mglchr(sch,'.'))
-#pragma omp parallel for
 			for(long i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
 		else	mgl_surf_plot(gr,pos,n,m);
-		if(wire && !gr->Stop)
+		if(wire)
 		{
 			gr->SetPenPal("k-");
-#pragma omp parallel for
 			for(long i=0;i<n*m;i++)	pos[i] = gr->CopyNtoC(pos[i],gr->CDef);
 			mgl_mesh_plot(gr,pos,n,m,3);
 		}
@@ -551,7 +525,7 @@ void MGL_EXPORT mgl_surfc_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT c, const char
 void MGL_EXPORT mgl_surfc(HMGL gr, HCDT z, HCDT c, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(z->GetNx()), y(z->GetNy());
+	mglDataV x(z->GetNx()), y(z->GetNy());
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_surfc_xy(gr,&x,&y,z,c,sch,0);
@@ -586,10 +560,9 @@ void MGL_EXPORT mgl_surfa_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT c, const char
 	mglPoint p,q,s,xx,yy;
 	for(k=0;k<z->GetNz();k++)
 	{
-#pragma omp parallel for private(p,xx,yy,q,s) collapse(2)
+		if(gr->NeedStop())	break;
 		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	continue;
 			xx = GetX(x,i,j,k);		yy = GetY(y,i,j,k);
 			mreal vv = z->v(i,j,k);	p = mglPoint(xx.x, yy.x, vv);
 			q = mglPoint(xx.y, yy.y, z->dvx(i,j,k));
@@ -597,13 +570,11 @@ void MGL_EXPORT mgl_surfa_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT c, const char
 			pos[i+n*j] = gr->AddPnt(p,gr->GetC(ss,vv),q^s,gr->GetA(c->v(i,j,k)));
 		}
 		if(sch && mglchr(sch,'.'))
-#pragma omp parallel for
 			for(long i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
 		else	mgl_surf_plot(gr,pos,n,m);
-		if(wire && !gr->Stop)
+		if(wire)
 		{
 			gr->SetPenPal("k-");
-#pragma omp parallel for
 			for(long i=0;i<n*m;i++)	pos[i] = gr->CopyNtoC(pos[i],gr->CDef);
 			mgl_mesh_plot(gr,pos,n,m,3);
 		}
@@ -614,7 +585,7 @@ void MGL_EXPORT mgl_surfa_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT c, const char
 void MGL_EXPORT mgl_surfa(HMGL gr, HCDT z, HCDT c, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(z->GetNx()), y(z->GetNy());
+	mglDataV x(z->GetNx()), y(z->GetNy());
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_surfa_xy(gr,&x,&y,z,c,sch,0);
@@ -654,9 +625,9 @@ void MGL_EXPORT mgl_boxs_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 	long k1,k2,k3,k4,k5,k6,k7,k8;
 	for(k=0;k<z->GetNz();k++)
 	{
+		if(gr->NeedStop())	break;
 		for(i=0;i<n;i+=dx)	for(j=0;j<m;j+=dy)
 		{
-			if(gr->Stop)	continue;
 			zz = z->v(i,j,k);		c  = gr->GetC(ss,zz);
 			xx = GetX(x,i,j,k);		yy = GetY(y,i,j,k);
 			x1 = i<lx-dx ? GetX(x,i+dx,j,k).x:NAN;
@@ -721,7 +692,7 @@ void MGL_EXPORT mgl_boxs_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 void MGL_EXPORT mgl_boxs(HMGL gr, HCDT z, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(z->GetNx()+1), y(z->GetNy()+1);
+	mglDataV x(z->GetNx()+1), y(z->GetNy()+1);
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_boxs_xy(gr,&x,&y,z,sch,0);
@@ -757,10 +728,9 @@ void MGL_EXPORT mgl_tile_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 	mglPoint s=mglPoint(0,0,1);
 	for(long k=0;k<z->GetNz();k++)
 	{
-#pragma omp parallel for collapse(2)
+		if(gr->NeedStop())	break;
 		for(long j=0;j<m;j+=dx)	for(long i=0;i<n;i+=dy)
 		{
-			if(gr->Stop)	continue;
 			register mreal zz = z->v(i,j,k), c = gr->GetC(ss,zz);
 			register mreal xx = GetX(x,i,j,k).x, yy = GetY(y,i,j,k).x;
 			register long k1 = gr->AddPnt(mglPoint(xx,yy,zz),c,s);
@@ -782,7 +752,7 @@ void MGL_EXPORT mgl_tile_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 void MGL_EXPORT mgl_tile(HMGL gr, HCDT z, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(z->GetNx()+1), y(z->GetNy()+1);
+	mglDataV x(z->GetNx()+1), y(z->GetNy()+1);
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_tile_xy(gr,&x,&y,z,sch,0);
@@ -819,10 +789,9 @@ void MGL_EXPORT mgl_tiles_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT s, const char
 	mreal x1,x2,x3,x4,y1,y2,y3,y4;
 	for(long k=0;k<z->GetNz();k++)
 	{
-#pragma omp parallel for private(x1,x2,x3,x4,y1,y2,y3,y4) collapse(2)
+		if(gr->NeedStop())	break;
 		for(long j=0;j<m;j+=dx)	for(long i=0;i<n;i+=dy)
 		{
-			if(gr->Stop)	continue;
 			register mreal zz = z->v(i,j,k), c = gr->GetC(cc,zz);
 			register mreal ss = (1-gr->GetA(s->v(i,j,k)))/2, sm = 1-ss;
 
@@ -849,7 +818,7 @@ void MGL_EXPORT mgl_tiles_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT s, const char
 void MGL_EXPORT mgl_tiles(HMGL gr, HCDT z, HCDT s, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(z->GetNx()+1), y(z->GetNy()+1);
+	mglDataV x(z->GetNx()+1), y(z->GetNy()+1);
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_tiles_xy(gr,&x,&y,z,s,sch,0);
@@ -874,12 +843,12 @@ void MGL_EXPORT mgl_map_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const char
 	long n=ax->GetNx(),m=ax->GetNy();
 	if(mgl_check_dim2(gr,x,y,ax,ay,"Map"))	return;
 
-	bool both = x->GetNx()==n && y->GetNx()==n && x->GetNy()==m && y->GetNy()==m;
+	bool nboth = !(x->GetNx()==n && y->GetNx()==n && x->GetNy()==m && y->GetNy()==m);
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("Map",cgid++);
 
 	long ss = gr->AddTexture(mgl_have_color(sch)?sch:"rgb",2);
-	long s = both ? n:1;
+	long s = nboth ?1:n;
 
 	mglPoint t=mglPoint(NAN);
 	long *pos = new long[n*m];
@@ -888,7 +857,6 @@ void MGL_EXPORT mgl_map_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const char
 #pragma omp parallel for collapse(2)
 	for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 	{
-		if(gr->Stop)	continue;
 		register long s1 = i>0 ? 1:0, s2 = i<n-1 ? 1:0;
 		register mreal xdx = (ax->v(i+s2,j)-ax->v(i-s1,j))/(GetX(x,i+s2,j).x-GetX(x,i-s1,j).x);
 		register mreal ydx = (ay->v(i+s2,j)-ay->v(i-s1,j))/(GetX(x,i+s2,j).x-GetX(x,i-s1,j).x);
@@ -898,22 +866,21 @@ void MGL_EXPORT mgl_map_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const char
 		xdx = xdx*ydy - xdy*ydx;	// Jacobian
 		register mreal xx,yy;
 
-		if(both)
-		{
-			xx = (x->v(i,j) - gr->Min.x)/(gr->Max.x - gr->Min.x);
-			yy = (y->v(i,j) - gr->Min.y)/(gr->Max.y - gr->Min.y);
-		}
-		else
+		if(nboth)
 		{
 			xx = (x->v(i) - gr->Min.x)/(gr->Max.x - gr->Min.x);
 			yy = (y->v(j) - gr->Min.y)/(gr->Max.y - gr->Min.y);
 		}
+		else
+		{
+			xx = (x->v(i,j) - gr->Min.x)/(gr->Max.x - gr->Min.x);
+			yy = (y->v(i,j) - gr->Min.y)/(gr->Max.y - gr->Min.y);
+		}
 		if(xx<0)	xx=0;	if(xx>=1)	xx=1/MGL_FEPSILON;
 		if(yy<0)	yy=0;	if(yy>=1)	yy=1/MGL_FEPSILON;
 		pos[i+n*j] = gr->AddPnt(mglPoint(ax->v(i,j), ay->v(i,j), xdx),gr->GetC(ss,xx,false),t,yy);
 	}
 	if(sch && mglchr(sch,'.'))
-#pragma omp parallel for
 		for(long i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
 	else	mgl_surf_plot(gr,pos,n,m);
 	delete []pos;	gr->EndGroup();
@@ -922,7 +889,7 @@ void MGL_EXPORT mgl_map_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const char
 void MGL_EXPORT mgl_map(HMGL gr, HCDT ax, HCDT ay, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(ax->GetNx()), y(ax->GetNy());
+	mglDataV x(ax->GetNx()), y(ax->GetNy());
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_map_xy(gr,&x,&y,ax,ay,sch,0);
