@@ -28,17 +28,18 @@
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_traj_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay, HCDT az, const char *sch, const char *opt)
 {
-	long m,mx,my,mz,nx,ny,nz,n=ax->GetNx(),pal;
+	long n=ax->GetNx(),pal;
 	if(mgl_check_dim1(gr,x,z,y,ax,"Traj"))	return;
 	if(mgl_check_dim1(gr,ax,az,ay,0,"Traj"))	return;
 
 	mreal len=gr->SaveState(opt);	if(mgl_isnan(len))	len = 0;
 	static int cgid=1;	gr->StartGroup("Traj",cgid++);
 
-	register long i, j;
 	// find maximum
-	i = ax->GetNy()>ay->GetNy() ? ax->GetNy():ay->GetNy();	j = z->GetNy()>az->GetNy() ? z->GetNy():az->GetNy();
-	m = x->GetNy()>y->GetNy() ? x->GetNy():y->GetNy();		if(i>m)	m=i;	if(j>m)	m=j;
+	long m = x->GetNy()>y->GetNy() ? x->GetNy():y->GetNy();
+	long i = ax->GetNy()>ay->GetNy() ? ax->GetNy():ay->GetNy();
+	long j = z->GetNy()>az->GetNy() ? z->GetNy():az->GetNy();
+	if(i>m)	m=i;	if(j>m)	m=j;
 	gr->SetPenPal(sch,&pal);	gr->Reserve(4*n*m);
 
 	mglPoint p1,p2;
@@ -49,15 +50,14 @@ void MGL_EXPORT mgl_traj_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay,
 		xm = xm>da ? xm : da;
 	}
 	xm = 1./(xm ? sqrt(xm):1);*/
-	for(j=0;j<m;j++) // start prepare arrows
+	for(long j=0;j<m;j++) // start prepare arrows
 	{
+		if(gr->NeedStop())	break;
 		gr->NextColor(pal);
-		nx = j<x->GetNy() ? j:0;	ny = j<y->GetNy() ? j:0;	nz = j<z->GetNy() ? j:0;
-		mx = j<ax->GetNy() ? j:0;	my = j<ay->GetNy() ? j:0;	mz = j<az->GetNy() ? j:0;
-#pragma omp parallel for private(p1,p2)
+		long nx = j<x->GetNy() ? j:0, ny = j<y->GetNy() ? j:0, nz = j<z->GetNy() ? j:0;
+		long mx = j<ax->GetNy() ? j:0,my = j<ay->GetNy() ? j:0,mz = j<az->GetNy() ? j:0;
 		for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	continue;
 			p1 = mglPoint(x->v(i,nx), y->v(i,ny), z->v(i,nz));
 			p2 = mglPoint(ax->v(i,mx),ay->v(i,my),az->v(i,mz));
 			mreal dd = p2.norm();
@@ -80,7 +80,7 @@ void MGL_EXPORT mgl_traj_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay,
 void MGL_EXPORT mgl_traj_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData z(x->GetNx()), az(x->GetNx());	z.Fill(gr->Min.z,gr->Min.z);
+	mglDataV z(x->GetNx()), az(x->GetNx());	z.Fill(gr->AdjustZMin());
 	mgl_traj_xyz(gr,x,y,&z,ax,ay,&az,sch,0);
 }
 //-----------------------------------------------------------------------------
@@ -148,11 +148,10 @@ void MGL_EXPORT mgl_vect_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const cha
 
 	for(long k=0;k<l;k++)
 	{
+		if(gr->NeedStop())	break;
 		if(ax->GetNz()>1)	zVal = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(k)/(ax->GetNz()-1);
-#pragma omp parallel for private(d,v,p1,p2) collapse(2)
 		for(long j=0;j<m;j+=ty)	for(long i=0;i<n;i+=tx)
 		{
-			if(gr->Stop)	continue;
 			d = mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, zVal);
 			v = mglPoint(ax->v(i,j,k),ay->v(i,j,k));
 			mreal dd = v.norm(), c1, c2;
@@ -177,7 +176,7 @@ void MGL_EXPORT mgl_vect_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const cha
 void MGL_EXPORT mgl_vect_2d(HMGL gr, HCDT ax, HCDT ay, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(ax->GetNx()), y(ax->GetNy());
+	mglDataV x(ax->GetNx()), y(ax->GetNy());
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_vect_xy(gr,&x,&y,ax,ay,sch,0);
@@ -247,26 +246,28 @@ void MGL_EXPORT mgl_vect_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay,
 	ca /= (n*m*l)/(tx*ty*tz);
 	xm = xm?1./xm:0;	cm = cm?1./cm:0;
 
-#pragma omp parallel for private(d,v,p1,p2) collapse(3)
-	for(long k=0;k<l;k+=tz)	for(long i=0;i<n;i+=tx)	for(long j=0;j<m;j+=ty)
+	for(long k=0;k<l;k+=tz)
 	{
-		if(gr->Stop)	continue;
-		d=mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, GetZ(z,i,j,k).x);
-		v = mglPoint(ax->v(i,j,k),ay->v(i,j,k),az->v(i,j,k));
-		mreal dd = v.norm(),c1,c2;
-		v *= cm*(fix?(dd>dm ? 1./dd : 0) : xm);
+		if(gr->NeedStop())	break;
+		for(long i=0;i<n;i+=tx)	for(long j=0;j<m;j+=ty)
+		{
+			d=mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, GetZ(z,i,j,k).x);
+			v = mglPoint(ax->v(i,j,k),ay->v(i,j,k),az->v(i,j,k));
+			mreal dd = v.norm(),c1,c2;
+			v *= cm*(fix?(dd>dm ? 1./dd : 0) : xm);
 
-		if(end)		{	p1 = d-v;	p2 = d;	}
-		else if(beg)	{	p1 = d;	p2 = d+v;	}
-		else		{	p1=d-v/2.;	p2=d+v/2.;	}
-		if(grd)	{	c1=gr->GetC(ss,dd*xm-0.5,false);	c2=gr->GetC(ss,dd*xm,false);	}
-		else	c1 = c2 = gr->GetC(ss,dd*xm,false);
-		long n1=gr->AddPnt(p1,c1),	n2=gr->AddPnt(p2,c2);
-		// allow vectors outside bounding box
-		if(n1<0 && n2>=0)	n1=gr->AddPnt(p1,c1,mglPoint(NAN),-1,2);
-		if(n2<0 && n1>=0)	n2=gr->AddPnt(p2,c2,mglPoint(NAN),-1,2);
-		if(dot)	{	gr->line_plot(n1,n2);	gr->mark_plot(n1,'.');	}
-		else	gr->vect_plot(n1,n2);
+			if(end)		{	p1 = d-v;	p2 = d;	}
+			else if(beg)	{	p1 = d;	p2 = d+v;	}
+			else		{	p1=d-v/2.;	p2=d+v/2.;	}
+			if(grd)	{	c1=gr->GetC(ss,dd*xm-0.5,false);	c2=gr->GetC(ss,dd*xm,false);	}
+			else	c1 = c2 = gr->GetC(ss,dd*xm,false);
+			long n1=gr->AddPnt(p1,c1),	n2=gr->AddPnt(p2,c2);
+			// allow vectors outside bounding box
+			if(n1<0 && n2>=0)	n1=gr->AddPnt(p1,c1,mglPoint(NAN),-1,2);
+			if(n2<0 && n1>=0)	n2=gr->AddPnt(p2,c2,mglPoint(NAN),-1,2);
+			if(dot)	{	gr->line_plot(n1,n2);	gr->mark_plot(n1,'.');	}
+			else	gr->vect_plot(n1,n2);
+		}
 	}
 	gr->EndGroup();
 }
@@ -274,7 +275,7 @@ void MGL_EXPORT mgl_vect_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay,
 void MGL_EXPORT mgl_vect_3d(HMGL gr, HCDT ax, HCDT ay, HCDT az, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(ax->GetNx()), y(ax->GetNy()), z(ax->GetNz());
+	mglDataV x(ax->GetNx()), y(ax->GetNy()), z(ax->GetNz());
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	z.Fill(gr->Min.z,gr->Max.z);
@@ -395,103 +396,6 @@ void MGL_NO_EXPORT mgl_get_slice(_mgl_vec_slice &s, HCDT x, HCDT y, HCDT z, HCDT
 	}
 }
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_get_slice_md(_mgl_vec_slice &s, const mglData *x, const mglData *y, const mglData *z, const mglData *ax, const mglData *ay, const mglData *az, char dir, mreal d, bool both)
-{
-	long n=ax->nx,m=ax->ny,l=ax->nz, nx=1,ny=1,p;
-
-	if(dir=='x')	{	nx = m;	ny = l;	if(d<0)	d = n/2.;	}
-	if(dir=='y')	{	nx = n;	ny = l;	if(d<0)	d = m/2.;	}
-	if(dir=='z')	{	nx = n;	ny = m;	if(d<0)	d = l/2.;	}
-	s.x.Create(nx,ny);	s.y.Create(nx,ny);	s.z.Create(nx,ny);
-	s.ax.Create(nx,ny);	s.ay.Create(nx,ny);	s.az.Create(nx,ny);
-	p = long(d);	d -= p;
-	if(dir=='x' && p>=n-1)	{	d+=p-n+2;	p=n-2;	}
-	if(dir=='y' && p>=m-1)	{	d+=p-m+2.;	p=m-2;	}
-	if(dir=='z' && p>=l-1)	{	d+=p-l+2;	p=l-2;	}
-
-	if(both)
-	{
-		if(dir=='x')
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
-			{
-				register long i0 = i+nx*j, i1 = p+n*(i+m*j);
-				s.x.a[i0] = x->a[i1]*(1-d) + x->a[i1+1]*d;
-				s.y.a[i0] = y->a[i1]*(1-d) + y->a[i1+1]*d;
-				s.z.a[i0] = z->a[i1]*(1-d) + z->a[i1+1]*d;
-				s.ax.a[i0] = ax->a[i1]*(1-d) + ax->a[i1+1]*d;
-				s.ay.a[i0] = ay->a[i1]*(1-d) + ay->a[i1+1]*d;
-				s.az.a[i0] = az->a[i1]*(1-d) + az->a[i1+1]*d;
-			}
-		if(dir=='y')
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
-			{
-				register long i0 = i+nx*j, i1 = i+n*(p+m*j);
-				s.x.a[i0] = x->a[i1]*(1-d) + x->a[i1+n]*d;
-				s.y.a[i0] = y->a[i1]*(1-d) + y->a[i1+n]*d;
-				s.z.a[i0] = z->a[i1]*(1-d) + z->a[i1+n]*d;
-				s.ax.a[i0] = ax->a[i1]*(1-d) + ax->a[i1+n]*d;
-				s.ay.a[i0] = ay->a[i1]*(1-d) + ay->a[i1+n]*d;
-				s.az.a[i0] = az->a[i1]*(1-d) + az->a[i1+n]*d;
-			}
-		if(dir=='z')
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
-			{
-				register long i0 = i+nx*j, i1 = i+n*(j+m*p);
-				s.x.a[i0] = x->a[i1]*(1-d) + x->a[i1+n*m]*d;
-				s.y.a[i0] = y->a[i1]*(1-d) + y->a[i1+n*m]*d;
-				s.z.a[i0] = z->a[i1]*(1-d) + z->a[i1+n*m]*d;
-				s.ax.a[i0] = ax->a[i1]*(1-d) + ax->a[i1+n*m]*d;
-				s.ay.a[i0] = ay->a[i1]*(1-d) + ay->a[i1+n*m]*d;
-				s.az.a[i0] = az->a[i1]*(1-d) + az->a[i1+n*m]*d;
-			}
-	}
-	else	// x, y, z -- vectors
-	{
-		if(dir=='x')
-		{
-			mreal v = x->a[p]*(1-d)+x->a[p+1]*d;
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
-			{
-				register long i0 = i+nx*j, i1 = p+n*(i+m*j);	s.x.a[i0] = v;
-				s.y.a[i0] = y->a[i];	s.z.a[i0] = z->a[j];
-				s.ax.a[i0] = ax->a[i1]*(1-d) + ax->a[i1+1]*d;
-				s.ay.a[i0] = ay->a[i1]*(1-d) + ay->a[i1+1]*d;
-				s.az.a[i0] = az->a[i1]*(1-d) + az->a[i1+1]*d;
-			}
-		}
-		if(dir=='y')
-		{
-			mreal v = y->a[p]*(1-d)+y->a[p+1]*d;
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
-			{
-				register long i0 = i+nx*j, i1 = i+n*(p+m*j);	s.y.a[i0] = v;
-				s.x.a[i0] = x->a[i];	s.z.a[i0] = z->a[j];
-				s.ax.a[i0] = ax->a[i1]*(1-d) + ax->a[i1+n]*d;
-				s.ay.a[i0] = ay->a[i1]*(1-d) + ay->a[i1+n]*d;
-				s.az.a[i0] = az->a[i1]*(1-d) + az->a[i1+n]*d;
-			}
-		}
-		if(dir=='z')
-		{
-			mreal v = z->a[p]*(1-d)+z->a[p+1]*d;
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
-			{
-				register long i0 = i+nx*j, i1 = i+n*(j+m*p);	s.z.a[i0] = v;
-				s.x.a[i0] = x->a[i];	s.y.a[i0] = y->a[j];
-				s.ax.a[i0] = ax->a[i1]*(1-d) + ax->a[i1+n*m]*d;
-				s.ay.a[i0] = ay->a[i1]*(1-d) + ay->a[i1+n*m]*d;
-				s.az.a[i0] = az->a[i1]*(1-d) + az->a[i1+n*m]*d;
-			}
-		}
-	}
-}
-//-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_vect3_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay, HCDT az, const char *sch, double sVal, const char *opt)
 {
 	bool both = mgl_isboth(x,y,z,ax);
@@ -511,16 +415,7 @@ void MGL_EXPORT mgl_vect3_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay,
 	long ss = gr->AddTexture(sch);
 
 	_mgl_vec_slice s;
-	const mglData *mx = dynamic_cast<const mglData *>(x);
-	const mglData *my = dynamic_cast<const mglData *>(y);
-	const mglData *mz = dynamic_cast<const mglData *>(z);
-	const mglData *max = dynamic_cast<const mglData *>(ax);
-	const mglData *may = dynamic_cast<const mglData *>(ay);
-	const mglData *maz = dynamic_cast<const mglData *>(az);
-	if(mx&&my&&mz&&max&&may&&maz)
-		mgl_get_slice_md(s,mx,my,mz,max,may,maz,dir,sVal,both);
-	else
-		mgl_get_slice(s,x,y,z,ax,ay,az,dir,sVal,both);
+	mgl_get_slice(s,x,y,z,ax,ay,az,dir,sVal,both);
 
 	long n=s.ax.nx,m=s.ax.ny, tx=1,ty=1;
 	if(gr->MeshNum>1)	{	tx=(n-1)/(gr->MeshNum-1);	ty=(m-1)/(gr->MeshNum-1);	}
@@ -555,10 +450,8 @@ void MGL_EXPORT mgl_vect3_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay,
 	ca /= (n*m)/(tx*ty);
 	xm = xm?1./xm:0;	cm = cm?1./cm:0;
 
-#pragma omp parallel for private(d,v,p1,p2) collapse(2)
 	for(long i=0;i<n;i+=tx)	for(long j=0;j<m;j+=ty)
 	{
-		if(gr->Stop)	continue;
 		register long i0 = i+n*j;
 		d = mglPoint(s.x.a[i0], s.y.a[i0], s.z.a[i0]);
 		v = mglPoint(s.ax.a[i0], s.ay.a[i0], s.az.a[i0]);
@@ -583,7 +476,7 @@ void MGL_EXPORT mgl_vect3_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay,
 void MGL_EXPORT mgl_vect3(HMGL gr, HCDT ax, HCDT ay, HCDT az, const char *sch, double sVal, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(ax->GetNx()), y(ax->GetNy()),z(ax->GetNz());
+	mglDataV x(ax->GetNx()), y(ax->GetNy()),z(ax->GetNz());	// NOTE mglDataV here is useless
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	z.Fill(gr->Min.z,gr->Max.z);
@@ -608,10 +501,9 @@ void MGL_EXPORT mgl_vect3_(uintptr_t *gr, uintptr_t *ax, uintptr_t *ay, uintptr_
 void MGL_NO_EXPORT flow(mglBase *gr, double zVal, double u, double v, const mglData &x, const mglData &y, const mglData &ax, const mglData &ay, long ss, bool vv)
 {
 	long n=10*(ax.nx+ax.ny);
-	bool both = x.nx==ax.nx && y.nx==ax.nx && x.ny==ax.ny && y.ny==ax.ny;
+	bool nboth = x.nx*x.ny!=ax.nx*ax.ny || y.nx*y.ny!=ax.nx*ax.ny;
 
 	mglPoint *pp = new mglPoint[n], dp;
-	mreal *cc = new mreal[n];
 	mglPoint dx(1/fabs(gr->Max.x-gr->Min.x),1/fabs(gr->Max.y-gr->Min.y),1/fabs(gr->Max.z-gr->Min.z));
 
 	mreal dt = 0.5/(ax.nx > ax.ny ? ax.nx : ax.ny),e,f,g,ff[4],gg[4],h,s=1;
@@ -619,26 +511,25 @@ void MGL_NO_EXPORT flow(mglBase *gr, double zVal, double u, double v, const mglD
 	register long k=0,m;
 	bool end = false;
 	do{
-		if(gr->Stop)	{	delete []pp;	delete []cc;	return;	}
-		pp[k].x = both ? x.Spline1(u,v,0):x.Spline1(u,0,0);
-		pp[k].y = both ? y.Spline1(u,v,0):y.Spline1(v,0,0);
+		pp[k].x = nboth ? x.Spline1(u,0,0):x.Spline1(u,v,0);
+		pp[k].y = nboth ? y.Spline1(v,0,0):y.Spline1(u,v,0);
 		pp[k].z = zVal;
 		for(m=0;m<k-1;m++)	// determines encircle
 			if(mgl_norm((pp[k]-pp[m])/dx)<dt/10.)	{	end = true;	break;	}
-		f = ax.Linear1(u,v,0);	g = ay.Linear1(u,v,0);
-		h = hypot(f,g);	cc[k] = gr->GetC(ss,s*h);
+		f = ax.Spline1(u,v,0);	g = ay.Spline1(u,v,0);
+		h = hypot(f,g);	pp[k].c = gr->GetC(ss,s*h);
 		if(h<1e-5)	break;	// stationary point
 		k++;
 		// find next point by midpoint method
 		h+=1;	ff[0]=f*dt/h;	gg[0]=g*dt/h;
 		e = u+ff[0]/2;	h = v+gg[0]/2;
-		f = ax.Linear1(e,h,0);	g = ay.Linear1(e,h,0);
+		f = ax.Spline1(e,h,0);	g = ay.Spline1(e,h,0);
 		h = 1+hypot(f,g);	ff[1]=f*dt/h;	gg[1]=g*dt/h;
 		e = u+ff[1]/2;	h = v+gg[1]/2;
-		f = ax.Linear1(e,h,0);	g = ay.Linear1(e,h,0);
+		f = ax.Spline1(e,h,0);	g = ay.Spline1(e,h,0);
 		h = 1+hypot(f,g);	ff[2]=f*dt/h;	gg[2]=g*dt/h;
 		e = u+ff[2];	h = v+gg[2];
-		f = ax.Linear1(e,h,0);	g = ay.Linear1(e,h,0);
+		f = ax.Spline1(e,h,0);	g = ay.Spline1(e,h,0);
 		h = 1+hypot(f,g);	ff[3]=f*dt/h;	gg[3]=g*dt/h;
 		u += ff[0]/6+ff[1]/3+ff[2]/3+ff[3]/6;
 		v += gg[0]/6+gg[1]/3+gg[2]/3+gg[3]/6;
@@ -647,11 +538,11 @@ void MGL_NO_EXPORT flow(mglBase *gr, double zVal, double u, double v, const mglD
 	} while(!end);
 	if(k>1)
 	{
-		long i,j,jj,a=long(1./fabs(dt));
-		gr->Reserve(k);		j = gr->AddPnt(pp[0],cc[0]);
-		for(i=1;i<k;i++)
+		long j,a=long(1./fabs(dt));
+		gr->Reserve(k);		j = gr->AddPnt(pp[0],pp[0].c);
+		for(long i=1;i<k;i++)
 		{
-			jj=j;	j = gr->AddPnt(pp[i],cc[i]);
+			long jj=j;	j = gr->AddPnt(pp[i],pp[i].c);
 			if(vv && i%a==0)
 			{
 				if(dt<0)	gr->vect_plot(j,jj,a/5);
@@ -660,12 +551,11 @@ void MGL_NO_EXPORT flow(mglBase *gr, double zVal, double u, double v, const mglD
 			else	gr->line_plot(jj,j);
 		}
 	}
-	delete []pp;	delete []cc;
+	delete []pp;
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_flow_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const char *sch, const char *opt)
 {
-	mreal u,v;
 	if(mgl_check_dim2(gr,x,y,ax,ay,"Flow"))	return;
 
 	mreal r = gr->SaveState(opt);
@@ -681,11 +571,13 @@ void MGL_EXPORT mgl_flow_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const cha
 
 	for(long k=0;k<ax->GetNz();k++)
 	{
+		if(gr->NeedStop())	break;
 		if(ax->GetNz()>1)	zVal = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(k)/(ax->GetNz()-1);
-#pragma omp parallel for private(u,v) collapse(2)
+//#pragma omp parallel for collapse(2)
 		for(long i=0;i<num;i++)	for(int s=-1;s<=1;s+=2)
 		{
-			if(gr->Stop)	continue;
+			mreal u,v;
+			if(gr->NeedStop())	{	i=num;	s=2;	continue;	}
 			u = 0;	v = (i+1.)/(num+1.);
 			flow(gr, zVal, s*u, s*v, xx, yy, bx, by,ss,vv);
 			u = 1;	v = (i+1.)/(num+1.);
@@ -709,7 +601,7 @@ void MGL_EXPORT mgl_flow_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const cha
 void MGL_EXPORT mgl_flow_2d(HMGL gr, HCDT ax, HCDT ay, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(ax->GetNx()), y(ax->GetNy());
+	mglDataV x(ax->GetNx()), y(ax->GetNy());	// NOTE mglDataV here is useless
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_flow_xy(gr,&x,&y,ax,ay,sch,0);
@@ -726,10 +618,10 @@ void MGL_EXPORT mgl_flow_2d_(uintptr_t *gr, uintptr_t *ax, uintptr_t *ay, const
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_flowp_xy(HMGL gr, double x0, double y0, double z0, HCDT x, HCDT y, HCDT ax, HCDT ay, const char *sch, const char *opt)
 {
-	mglPoint p(x0,y0,z0);
+	if(mgl_isnan(z0))	z0 = gr->Min.z;
 	mreal u,v;
 	long n=ax->GetNx(), m=ax->GetNy();
-	bool both = x->GetNx()==n && y->GetNx()==n && x->GetNy()==m && y->GetNy()==m;
+	bool nboth = x->GetNx()*x->GetNy()!=n*m || y->GetNx()*y->GetNy()!=n*m;
 	if(mgl_check_dim2(gr,x,y,ax,ay,"FlowP"))	return;
 
 	gr->SaveState(opt);
@@ -743,38 +635,38 @@ void MGL_EXPORT mgl_flowp_xy(HMGL gr, double x0, double y0, double z0, HCDT x, H
 	long i0=0,j0=0;
 	for(i=0;i<n;i++)	for(j=0;j<m;j++)	// first find closest
 	{
-		d = both ? hypot(x->v(i,j)-p.x,y->v(i,j)-p.y) : hypot(x->v(i)-p.x,y->v(j)-p.y);
+		d = nboth ? hypot(x->v(i)-x0,y->v(j)-y0) : hypot(x->v(i,j)-x0,y->v(i,j)-y0);
 		if(d<dm)	{	i0=i;	j0=j;	dm=d;	}
 	}
 	if(dm==0)	{	u = i0/mreal(n);	v = j0/mreal(m);	}	// we find it
 	else
 	{
 		mreal dxu,dxv,dyu,dyv, dx, dy;
-		if(both)
+		if(nboth)
 		{
-			dx = x->v(i0,j0)-p.x;	dy = y->v(i0,j0)-p.y;
+			dx = x->v(i0)-x0;	dy = y->v(j0)-y0;
+			dxu= x->dvx(i0);	dyv= y->dvx(j0);
+			u = (i0+dx/dxu)/n;	v = (j0+dy/dyv)/m;
+		}
+		else
+		{
+			dx = x->v(i0,j0)-x0;	dy = y->v(i0,j0)-y0;
 			dxu= x->dvx(i0,j0);		dyu= y->dvx(i0,j0);
 			dxv= x->dvy(i0,j0);		dyv= y->dvy(i0,j0);
 			d = dxv*dyu-dxu*dyv;
 			u = (i0+(dxv*dy-dx*dyv)/d)/n;
 			v = (j0-(dxu*dy-dx*dyu)/d)/m;
 		}
-		else
-		{
-			dx = x->v(i0)-p.x;	dy = y->v(j0)-p.y;
-			dxu= x->dvx(i0);	dyv= y->dvx(j0);
-			u = (i0+dx/dxu)/n;	v = (j0+dy/dyv)/m;
-		}
 	}
 	mglData xx(x), yy(y), bx(ax), by(ay);
-	flow(gr, p.z, u, v, xx, yy, bx, by,ss,vv);
+	flow(gr, z0, u, v, xx, yy, bx, by,ss,vv);
 	gr->EndGroup();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_flowp_2d(HMGL gr, double x0, double y0, double z0, HCDT ax, HCDT ay, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(ax->GetNx()), y(ax->GetNy());
+	mglDataV x(ax->GetNx()), y(ax->GetNy());
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_flowp_xy(gr,x0,y0,z0,&x,&y,ax,ay,sch,0);
@@ -798,9 +690,8 @@ void flow(mglBase *gr, double u, double v, double w, const mglData &x, const mgl
 {
 	static long n=10*(ax.nx+ax.ny);
 	long nn = ax.nx*ax.ny*ax.nz;
-	bool both = x.nx*x.ny*x.nz==nn && y.nx*y.ny*y.nz==nn && z.nx*z.ny*z.nz==nn;
+	bool nboth = x.nx*x.ny*x.nz!=nn || y.nx*y.ny*y.nz!=nn || z.nx*z.ny*z.nz!=nn;
 	mglPoint *pp = new mglPoint[n], dp;
-	mreal *cc = new mreal[n];
 	mglPoint dx(1/fabs(gr->Max.x-gr->Min.x),1/fabs(gr->Max.y-gr->Min.y),1/fabs(gr->Max.z-gr->Min.z));
 
 	nn = (ax.nx > ax.ny ? ax.nx : ax.ny);
@@ -811,29 +702,28 @@ void flow(mglBase *gr, double u, double v, double w, const mglData &x, const mgl
 	register long k=0,m;
 	bool end = false;
 	do{
-		if(gr->Stop)	{	delete []pp;	delete []cc;	return;	}
-		pp[k].x = both ? x.Spline1(u,v,w):x.Spline1(u,0,0);
-		pp[k].y = both ? y.Spline1(u,v,w):y.Spline1(v,0,0);
-		pp[k].z = both ? z.Spline1(u,v,w):z.Spline1(w,0,0);
+		pp[k].x = nboth ? x.Spline1(u,0,0):x.Spline1(u,v,w);
+		pp[k].y = nboth ? y.Spline1(v,0,0):y.Spline1(u,v,w);
+		pp[k].z = nboth ? z.Spline1(w,0,0):z.Spline1(u,v,w);
 		for(m=0;m<k-1;m++)	// determines encircle
 			if(mgl_norm((pp[k]-pp[m])/dx)<dt/10.)	{	end = true;	break;	}
-		e = ax.Linear1(u,v,w);	f = ay.Linear1(u,v,w);	g = az.Linear1(u,v,w);
-		h = sqrt(e*e+f*f+g*g);	cc[k] = gr->GetC(ss,s*h);
+		e = ax.Spline1(u,v,w);	f = ay.Spline1(u,v,w);	g = az.Spline1(u,v,w);
+		h = sqrt(e*e+f*f+g*g);	pp[k].c = gr->GetC(ss,s*h);
 		if(h<1e-5)	break;	// stationary point
 		k++;
 		// find next point by midpoint method
 		h+=1;	ee[0]=e*dt/h;	ff[0]=f*dt/h;	gg[0]=g*dt/h;
 		u1 = u+ee[0]/2;	v1 = v+ff[0]/2;	w1 = w+gg[0]/2;
-		e = ax.Linear1(u1,v1,w1);	f = ay.Linear1(u1,v1,w1);
-		g = az.Linear1(u1,v1,w1);	h = 1+sqrt(e*e+f*f+g*g);
+		e = ax.Spline1(u1,v1,w1);	f = ay.Spline1(u1,v1,w1);
+		g = az.Spline1(u1,v1,w1);	h = 1+sqrt(e*e+f*f+g*g);
 		ee[1]=e*dt/h;	ff[1]=f*dt/h;	gg[1]=g*dt/h;
 		u1 = u+ee[1]/2;	v1 = v+ff[1]/2;	w1 = w+gg[1]/2;
-		e = ax.Linear1(u1,v1,w1);	f = ay.Linear1(u1,v1,w1);
-		g = az.Linear1(u1,v1,w1);	h = 1+sqrt(e*e+f*f+g*g);
+		e = ax.Spline1(u1,v1,w1);	f = ay.Spline1(u1,v1,w1);
+		g = az.Spline1(u1,v1,w1);	h = 1+sqrt(e*e+f*f+g*g);
 		ee[2]=e*dt/h;	ff[2]=f*dt/h;	gg[2]=g*dt/h;
 		u1 = u+ee[2];	v1 = v+ff[2];	w1 = w+gg[2];
-		e = ax.Linear1(u1,v1,w1);	f = ay.Linear1(u1,v1,w1);
-		g = az.Linear1(u1,v1,w1);	h = 1+sqrt(e*e+f*f+g*g);
+		e = ax.Spline1(u1,v1,w1);	f = ay.Spline1(u1,v1,w1);
+		g = az.Spline1(u1,v1,w1);	h = 1+sqrt(e*e+f*f+g*g);
 		ee[3]=e*dt/h;	ff[3]=f*dt/h;	gg[3]=g*dt/h;
 		u += ee[0]/6+ee[1]/3+ee[2]/3+ee[3]/6;
 		v += ff[0]/6+ff[1]/3+ff[2]/3+ff[3]/6;
@@ -843,21 +733,21 @@ void flow(mglBase *gr, double u, double v, double w, const mglData &x, const mgl
 	} while(!end);
 	if(k>1)
 	{
-		long i,j,jj,a=long(1./fabs(dt));
+		long j,a=long(1./fabs(dt));
 		mreal rr = mgl_norm(gr->Max-gr->Min)*gr->BarWidth/25, ll;
 		mglPoint q1,q2,l;
-		long n1=-1,n2=-1,n3=-1,n4=-1, m1=-1,m2=-1,m3=-1,m4=-1;
+		long n1=-1,n2=-1,n3=-1,n4=-1;
 
-		gr->Reserve(4*k);	j = gr->AddPnt(pp[0],cc[0]);
+		gr->Reserve(4*k);	j = gr->AddPnt(pp[0],pp[0].c);
 		l = pp[1] - pp[0];	l /= mgl_norm(l);
 		q1 = mglPoint(l.y,-l.x,0);	ll = mgl_norm(q1);
 		if(ll)	q1 /= ll;	else	q1 = mglPoint(0,1,0);
 		q2 = q1^l;
 		if(xo)	{	n1 = gr->AddPnt(pp[0],-1,q2);	n2 = gr->AddPnt(pp[0]+rr*q1,-1,q2);	}
 		if(zo)	{	n3 = gr->AddPnt(pp[0],-1,q1);	n4 = gr->AddPnt(pp[0]+rr*q2,-1,q1);	}
-		for(i=1;i<k;i++)
+		for(long i=1;i<k;i++)
 		{
-			jj=j;	j = gr->AddPnt(pp[i],cc[i]);
+			long jj=j;	j = gr->AddPnt(pp[i],pp[i].c);
 			if(vv && i%a==0)
 			{
 				if(dt<0)	gr->vect_plot(j,jj,a/5);
@@ -866,19 +756,18 @@ void flow(mglBase *gr, double u, double v, double w, const mglData &x, const mgl
 			else	gr->line_plot(jj,j);
 			l = pp[i]-pp[i-1];		l /= mgl_norm(l);
 			q1 -= l*(l*q1);	q1/= mgl_norm(q1);	q2 = q1^l;
-			m1 = n1;	m2 = n2;	m3 = n3;	m4 = n4;
+			long m1 = n1, m2 = n2, m3 = n3, m4 = n4;
 			if(xo)
 			{	n1 = gr->AddPnt(pp[i],-1,q2);	n2 = gr->AddPnt(pp[i]+rr*q1,-1,q2);	gr->quad_plot(n1,n2,m1,m2);	}
 			if(zo)
 			{	n3 = gr->AddPnt(pp[i],-1,q1);	n4 = gr->AddPnt(pp[i]+rr*q2,-1,q1);	gr->quad_plot(n3,n4,m3,m4);	}
 		}
 	}
-	delete []pp;	delete []cc;
+	delete []pp;
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_flow_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay, HCDT az, const char *sch, const char *opt)
 {
-	mreal u,v,w;
 	if(mgl_check_vec3(gr,x,y,z,ax,ay,az,"Flow3"))	return;
 
 	mreal r = gr->SaveState(opt);
@@ -889,10 +778,11 @@ void MGL_EXPORT mgl_flow_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay,
 	bool vv = mglchr(sch,'v'), xo = mglchr(sch,'x'), zo = mglchr(sch,'z');
 
 	mglData xx(x), yy(y), zz(z), bx(ax), by(ay), bz(az);
-#pragma omp parallel for private(u,v,w) collapse(3)
+//#pragma omp parallel for collapse(3)
 	for(long i=0;i<num;i++)	for(long j=0;j<num;j++)	for(int s=-1;s<=1;s+=2)
 	{
-		if(gr->Stop)	continue;
+		mreal u,v,w;
+		if(gr->NeedStop())	{	i=j=num;	s=2;	continue;	}
 		u = (i+1.)/(num+1.);	v = (j+1.)/(num+1.);	w = 0;
 		flow(gr, s*u, s*v, s*w, xx, yy, zz, bx, by, bz,ss,vv,xo,zo);
 		u = (i+1.)/(num+1.);	v = (j+1.)/(num+1.);	w = 1;
@@ -921,7 +811,7 @@ void MGL_EXPORT mgl_flow_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay,
 void MGL_EXPORT mgl_flow_3d(HMGL gr, HCDT ax, HCDT ay, HCDT az, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(ax->GetNx()), y(ax->GetNy()), z(ax->GetNz());
+	mglDataV x(ax->GetNx()), y(ax->GetNy()), z(ax->GetNz());	// NOTE mglDataV here is useless
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	z.Fill(gr->Min.z,gr->Max.z);
@@ -942,7 +832,7 @@ void MGL_EXPORT mgl_flowp_xyz(HMGL gr, double x0, double y0, double z0, HCDT x,
 	mglPoint p(x0,y0,z0);
 	mreal u,v,w;
 	long n=ax->GetNx(),m=ax->GetNy(),l=ax->GetNz();
-	bool both = x->GetNx()*x->GetNy()*x->GetNz()==n*m*l && y->GetNx()*y->GetNy()*y->GetNz()==n*m*l && z->GetNx()*z->GetNy()*z->GetNz()==n*m*l;
+	bool nboth = !(x->GetNN()==n*m*l && y->GetNN()==n*m*l && z->GetNN()==n*m*l);
 	if(mgl_check_vec3(gr,x,y,z,ax,ay,az,"FlowP3"))	return;
 
 	gr->SaveState(opt);
@@ -957,10 +847,10 @@ void MGL_EXPORT mgl_flowp_xyz(HMGL gr, double x0, double y0, double z0, HCDT x,
 	mreal dx,dy,dz;
 	for(i=0;i<n;i++)	for(j=0;j<m;j++)	for(k=0;k<l;k++)	// first find closest
 	{
-		if(both)
-		{	dx = x->v(i,j,k)-p.x;	dy = y->v(i,j,k)-p.y;	dz = x->v(i,j,k)-p.z;	}
-		else
+		if(nboth)
 		{	dx = x->v(i)-p.x;	dy = y->v(j)-p.y;	dz = x->v(k)-p.z;	}
+		else
+		{	dx = x->v(i,j,k)-p.x;	dy = y->v(i,j,k)-p.y;	dz = x->v(i,j,k)-p.z;	}
 		d = sqrt(dx*dx+dy*dy+dz*dz);
 		if(d<dm)	{	i0=i;	j0=j;	k0=k;	dm=d;	}
 	}
@@ -969,7 +859,13 @@ void MGL_EXPORT mgl_flowp_xyz(HMGL gr, double x0, double y0, double z0, HCDT x,
 	else
 	{
 		mreal dxu,dxv,dxw,dyu,dyv,dyw,dzu,dzv,dzw;
-		if(both)
+		if(nboth)
+		{
+			dx = x->v(i0)-p.x;	dy = y->v(j0)-p.y;	dz = z->v(k0)-p.z;
+			dxu= x->dvx(i0);	dyv= y->dvx(j0);	dzw= z->dvx(k0);
+			u = (i0+dx/dxu)/n;	v = (j0+dy/dyv)/m;	w = (k0+dz/dzw)/m;
+		}
+		else
 		{
 			dx = x->v(i0,j0,k0)-p.x;	dy = y->v(i0,j0,k0)-p.y;	dz = z->v(i0,j0,k0)-p.z;
 			dxu= x->dvx(i0,j0,k0);		dyu= y->dvx(i0,j0,k0);		dzu= z->dvx(i0,j0,k0);
@@ -980,12 +876,6 @@ void MGL_EXPORT mgl_flowp_xyz(HMGL gr, double x0, double y0, double z0, HCDT x,
 			v = (j0-(dx*(dyw*dzu-dyu*dzw)+dxu*(dy*dzw-dyw*dz)+dxw*(dyu*dz-dy*dzu))/d)/m;
 			w = (i0+(dx*(dyv*dzu-dyu*dzv)+dxu*(dy*dzv-dyv*dz)+dxv*(dyu*dz-dy*dzu))/d)/l;
 		}
-		else
-		{
-			dx = x->v(i0)-p.x;	dy = y->v(j0)-p.y;	dz = z->v(k0)-p.z;
-			dxu= x->dvx(i0);	dyv= y->dvx(j0);	dzw= z->dvx(k0);
-			u = (i0+dx/dxu)/n;	v = (j0+dy/dyv)/m;	w = (k0+dz/dzw)/m;
-		}
 	}
 	mglData xx(x), yy(y), zz(z), bx(ax), by(ay), bz(az);
 	flow(gr, u, v, w, xx, yy, zz, bx, by, bz,ss,vv,xo,zo);
@@ -995,7 +885,7 @@ void MGL_EXPORT mgl_flowp_xyz(HMGL gr, double x0, double y0, double z0, HCDT x,
 void MGL_EXPORT mgl_flowp_3d(HMGL gr, double x0, double y0, double z0, HCDT ax, HCDT ay, HCDT az, const char *sch, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(ax->GetNx()), y(ax->GetNy()), z(ax->GetNz());
+	mglDataV x(ax->GetNx()), y(ax->GetNy()), z(ax->GetNz());	// NOTE mglDataV here is useless
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	z.Fill(gr->Min.z,gr->Max.z);
@@ -1021,7 +911,7 @@ void MGL_EXPORT mgl_grad_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT phi, const ch
 	mglData ax(phi), ay,az,xx,yy,zz;
 	ay.Set(ax);	az.Set(ax);	xx.Set(ax);	yy.Set(ax);	zz.Set(ax);
 	long n=xx.nx, m=xx.ny, l=xx.nz, nn = n*m*l;
-	if(x->GetNx()*x->GetNy()*x->GetNz()==nn && y->GetNx()*y->GetNy()*y->GetNz()==nn && z->GetNx()*z->GetNy()*z->GetNz()==nn)
+	if(x->GetNN()==nn && y->GetNN()==nn && z->GetNN()==nn)
 	{	xx.Set(x);	yy.Set(y);	zz.Set(z);	}	// nothing to do
 	else if(x->GetNx()==n && y->GetNx()==m && z->GetNx()==l)
 #pragma omp parallel for collapse(3)
@@ -1052,7 +942,7 @@ void MGL_EXPORT mgl_grad_xy(HMGL gr, HCDT x, HCDT y, HCDT phi, const char *sch,
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_grad(HMGL gr, HCDT phi, const char *sch, const char *opt)
 {
-	mglData x(phi->GetNx()), y(phi->GetNy()), z(phi->GetNz());
+	mglDataV x(phi->GetNx()), y(phi->GetNy()), z(phi->GetNz());
 	gr->SaveState(opt);
 	x.Fill(gr->Min.x,gr->Max.x);	y.Fill(gr->Min.y,gr->Max.y);	z.Fill(gr->Min.z,gr->Max.z);
 	if(phi->GetNz()==1)	mgl_grad_xy(gr,&x,&y,phi,sch,0);
@@ -1081,7 +971,7 @@ void MGL_EXPORT mgl_grad_(uintptr_t *gr, uintptr_t *ph, const char *sch, const c
 void MGL_NO_EXPORT flowr(mglBase *gr, double zVal, double u, double v, const mglData &x, const mglData &y, const mglData &ax, const mglData &ay, double r0,long sc)
 {
 	long n=10*(ax.nx+ax.ny);
-	bool both = x.nx==ax.nx && y.nx==ax.nx && x.ny==ax.ny && y.ny==ax.ny;
+	bool nboth = x.nx*x.ny!=ax.nx*ax.ny || y.nx*y.ny!=ax.nx*ax.ny;
 
 	mglPoint *pp = new mglPoint[n], dp;
 	mreal *cc = new mreal[n];
@@ -1093,13 +983,12 @@ void MGL_NO_EXPORT flowr(mglBase *gr, double zVal, double u, double v, const mgl
 	register long k=0,m;
 	bool end = false;
 	do{
-		if(gr->Stop)	{	delete []pp;	delete []cc;	return;	}
-		pp[k].x = both ? x.Spline1(u,v,0):x.Spline1(u,0,0);
-		pp[k].y = both ? y.Spline1(u,v,0):y.Spline1(v,0,0);
+		pp[k].x = nboth ? x.Spline1(u,0,0):x.Spline1(u,v,0);
+		pp[k].y = nboth ? y.Spline1(v,0,0):y.Spline1(u,v,0);
 		pp[k].z = zVal;
 		for(m=0;m<k-1;m++)	// determines encircle
 			if(mgl_norm((pp[k]-pp[m])/dx)<dt/10.)	{	end = true;	break;	}
-		f = ax.Linear1(u,v,0);	g = ay.Linear1(u,v,0);
+		f = ax.Spline1(u,v,0);	g = ay.Spline1(u,v,0);
 		h = hypot(f,g);	cc[k] = gr->GetC(sc,s*h);
 		pp[k].c = r0>0 ? r0*sqrt(1e-2+ss*h*h)/2 : -r0/sqrt(1e-2+ss*h*h)/5;
 		if(h<1e-5)	break;	// stationary point
@@ -1107,13 +996,13 @@ void MGL_NO_EXPORT flowr(mglBase *gr, double zVal, double u, double v, const mgl
 		// find next point by midpoint method
 		h+=1;	ff[0]=f*dt/h;	gg[0]=g*dt/h;
 		e = u+ff[0]/2;	h = v+gg[0]/2;
-		f = ax.Linear1(e,h,0);	g = ay.Linear1(e,h,0);	h = 1+hypot(f,g);
+		f = ax.Spline1(e,h,0);	g = ay.Spline1(e,h,0);	h = 1+hypot(f,g);
 		ff[1]=f*dt/h;	gg[1]=g*dt/h;
 		e = u+ff[1]/2;	h = v+gg[1]/2;
-		f = ax.Linear1(e,h,0);	g = ay.Linear1(e,h,0);	h = 1+hypot(f,g);
+		f = ax.Spline1(e,h,0);	g = ay.Spline1(e,h,0);	h = 1+hypot(f,g);
 		ff[2]=f*dt/h;	gg[2]=g*dt/h;
 		e = u+ff[2];	h = v+gg[2];
-		f = ax.Linear1(e,h,0);	g = ay.Linear1(e,h,0);	h = 1+hypot(f,g);
+		f = ax.Spline1(e,h,0);	g = ay.Spline1(e,h,0);	h = 1+hypot(f,g);
 		ff[3]=f*dt/h;	gg[3]=g*dt/h;
 		u += ff[0]/6+ff[1]/3+ff[2]/3+ff[3]/6;
 		v += gg[0]/6+gg[1]/3+gg[2]/3+gg[3]/6;
@@ -1161,7 +1050,6 @@ void MGL_NO_EXPORT flowr(mglBase *gr, double zVal, double u, double v, const mgl
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_pipe_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const char *sch, double r0, const char *opt)
 {
-	mreal u,v;
 	if(mgl_check_dim2(gr,x,y,ax,ay,"Pipe"))	return;
 
 	mreal r = gr->SaveState(opt);
@@ -1177,11 +1065,13 @@ void MGL_EXPORT mgl_pipe_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const cha
 	mglData xx(x), yy(y), bx(ax), by(ay);
 	for(long k=0;k<ax->GetNz();k++)
 	{
+		if(gr->NeedStop())	break;
 		if(ax->GetNz()>1)	zVal = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(k)/(ax->GetNz()-1);
-#pragma omp parallel for private(u,v) collapse(2)
+//#pragma omp parallel for collapse(2)
 		for(long i=0;i<num;i++)	for(int s=-1;s<=1;s+=2)
 		{
-			if(gr->Stop)	continue;
+			mreal u,v;
+			if(gr->NeedStop())	{	i=num;	s=2;	continue;	}
 			u = 0;	v = (i+1.)/(num+1.);
 			flowr(gr, zVal, s*u, s*v, xx, yy, bx, by,r0,ss);
 			u = 1;	v = (i+1.)/(num+1.);
@@ -1205,7 +1095,7 @@ void MGL_EXPORT mgl_pipe_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const cha
 void MGL_EXPORT mgl_pipe_2d(HMGL gr, HCDT ax, HCDT ay, const char *sch, double r0, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(ax->GetNx()), y(ax->GetNy());
+	mglDataV x(ax->GetNx()), y(ax->GetNy());	// NOTE mglDataV here is useless
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_pipe_xy(gr,&x,&y,ax,ay,sch,r0,0);
@@ -1228,7 +1118,7 @@ void flowr(mglBase *gr, double u, double v, double w, const mglData &x, const mg
 {
 	static long n=10*(ax.nx+ax.ny);
 	long nn = ax.nx*ax.ny*ax.nz;
-	bool both = x.nx*x.ny*x.nz==nn && y.nx*y.ny*y.nz==nn && z.nx*z.ny*z.nz==nn;
+	bool nboth = x.nx*x.ny*x.nz!=nn || y.nx*y.ny*y.nz!=nn || z.nx*z.ny*z.nz!=nn;
 	mglPoint *pp = new mglPoint[n], dp;
 	mreal *cc = new mreal[n];
 	mglPoint dx(1/fabs(gr->Max.x-gr->Min.x),1/fabs(gr->Max.y-gr->Min.y),1/fabs(gr->Max.z-gr->Min.z));
@@ -1243,13 +1133,12 @@ void flowr(mglBase *gr, double u, double v, double w, const mglData &x, const mg
 	register long k=0,m;
 	bool end = false;
 	do{
-		if(gr->Stop)	{	delete []pp;	delete []cc;	return;	}
-		pp[k].x = both ? x.Spline1(u,v,w):x.Spline1(u,0,0);
-		pp[k].y = both ? y.Spline1(u,v,w):y.Spline1(v,0,0);
-		pp[k].z = both ? z.Spline1(u,v,w):z.Spline1(w,0,0);
+		pp[k].x = nboth ? x.Spline1(u,0,0):x.Spline1(u,v,w);
+		pp[k].y = nboth ? y.Spline1(v,0,0):y.Spline1(u,v,w);
+		pp[k].z = nboth ? z.Spline1(w,0,0):z.Spline1(u,v,w);
 		for(m=0;m<k-1;m++)	// determines encircle
 			if(mgl_norm((pp[k]-pp[m])/dx)<dt/10.)	{	end = true;	break;	}
-		e = ax.Linear1(u,v,w);	f = ay.Linear1(u,v,w);	g = az.Linear1(u,v,w);
+		e = ax.Spline1(u,v,w);	f = ay.Spline1(u,v,w);	g = az.Spline1(u,v,w);
 		h = sqrt(e*e+f*f+g*g);	cc[k] = gr->GetC(sc,s*h);
 		pp[k].c = r0>0 ? r0*sqrt(1e-2+ss*h*h)/2 : -r0/sqrt(1e-2+ss*h*h)/5;
 		if(h<1e-5)	break;	// stationary point
@@ -1257,16 +1146,16 @@ void flowr(mglBase *gr, double u, double v, double w, const mglData &x, const mg
 		// find next point by midpoint method
 		h+=1;	ee[0]=e*dt/h;	ff[0]=f*dt/h;	gg[0]=g*dt/h;
 		u1 = u+ee[0]/2;	v1 = v+ff[0]/2;	w1 = w+gg[0]/2;
-		e = ax.Linear1(u1,v1,w1);	f = ay.Linear1(u1,v1,w1);
-		g = az.Linear1(u1,v1,w1);	h = 1+sqrt(e*e+f*f+g*g);
+		e = ax.Spline1(u1,v1,w1);	f = ay.Spline1(u1,v1,w1);
+		g = az.Spline1(u1,v1,w1);	h = 1+sqrt(e*e+f*f+g*g);
 		ee[1]=e*dt/h;	ff[1]=f*dt/h;	gg[1]=g*dt/h;
 		u1 = u+ee[1]/2;	v1 = v+ff[1]/2;	w1 = w+gg[1]/2;
-		e = ax.Linear1(u1,v1,w1);	f = ay.Linear1(u1,v1,w1);
-		g = az.Linear1(u1,v1,w1);	h = 1+sqrt(e*e+f*f+g*g);
+		e = ax.Spline1(u1,v1,w1);	f = ay.Spline1(u1,v1,w1);
+		g = az.Spline1(u1,v1,w1);	h = 1+sqrt(e*e+f*f+g*g);
 		ee[2]=e*dt/h;	ff[2]=f*dt/h;	gg[2]=g*dt/h;
 		u1 = u+ee[2];	v1 = v+ff[2];	w1 = w+gg[2];
-		e = ax.Linear1(u1,v1,w1);	f = ay.Linear1(u1,v1,w1);
-		g = az.Linear1(u1,v1,w1);	h = 1+sqrt(e*e+f*f+g*g);
+		e = ax.Spline1(u1,v1,w1);	f = ay.Spline1(u1,v1,w1);
+		g = az.Spline1(u1,v1,w1);	h = 1+sqrt(e*e+f*f+g*g);
 		ee[3]=e*dt/h;	ff[3]=f*dt/h;	gg[3]=g*dt/h;
 		u += ee[0]/6+ee[1]/3+ee[2]/3+ee[3]/6;
 		v += ff[0]/6+ff[1]/3+ff[2]/3+ff[3]/6;
@@ -1315,7 +1204,6 @@ void flowr(mglBase *gr, double u, double v, double w, const mglData &x, const mg
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_pipe_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay, HCDT az, const char *sch, double r0, const char *opt)
 {
-	mreal u,v,w;
 	if(mgl_check_vec3(gr,x,y,z,ax,ay,az,"Vect"))	return;
 
 	mreal r = gr->SaveState(opt);
@@ -1327,10 +1215,11 @@ void MGL_EXPORT mgl_pipe_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay,
 	bool cnt=!mglchr(sch,'#');
 
 	mglData xx(x), yy(y), zz(z), bx(ax), by(ay), bz(az);
-#pragma omp parallel for private(u,v,w) collapse(3)
+//#pragma omp parallel for collapse(3)
 	for(long i=0;i<num;i++)	for(long j=0;j<num;j++)	for(int s=-1;s<=1;s+=2)
 	{
-		if(gr->Stop)	continue;
+		mreal u,v,w;
+		if(gr->NeedStop())	{	i=j=num;	s=2;	continue;	}
 		u = (i+1.)/(num+1.);	v = (j+1.)/(num+1.);	w = 0;
 		flowr(gr, s*u, s*v, s*w, xx, yy, zz, bx, by, bz,r0,ss);
 		u = (i+1.)/(num+1.);	v = (j+1.)/(num+1.);	w = 1;
@@ -1359,7 +1248,7 @@ void MGL_EXPORT mgl_pipe_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay,
 void MGL_EXPORT mgl_pipe_3d(HMGL gr, HCDT ax, HCDT ay, HCDT az, const char *sch, double r0, const char *opt)
 {
 	gr->SaveState(opt);
-	mglData x(ax->GetNx()), y(ax->GetNy()), z(ax->GetNz());
+	mglDataV x(ax->GetNx()), y(ax->GetNy()), z(ax->GetNz());	// NOTE mglDataV here is useless
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	z.Fill(gr->Min.z,gr->Max.z);
