@@ -47,7 +47,7 @@ void addDataPanel(QWidget *wnd, QWidget *w, QString name);
 void deleteDat(void *o)		{	if(o)	delete ((DatPanel *)o);	}
 void refreshData(QWidget *w)	{	((DatPanel *)w)->refresh();	}
 //-----------------------------------------------------------------------------
-QWidget *newDataWnd(InfoDialog *inf, QWidget *wnd, mglVar *v)
+QWidget *newDataWnd(InfoDialog *inf, QWidget *wnd, mglDataA *v)
 {
 	DatPanel *t = new DatPanel(inf);
 	if(v)	t->setVar(v);
@@ -73,18 +73,19 @@ DatPanel::DatPanel(InfoDialog *inf, QWidget *parent) : QWidget(parent)
 	setWindowIcon(QPixmap(table_xpm));
 }
 //-----------------------------------------------------------------------------
-DatPanel::~DatPanel()	{	if(var)	var->o = 0;	}
+DatPanel::~DatPanel()	{	if(var && var->o==this)	var->o = 0;	}
 //-----------------------------------------------------------------------------
 void DatPanel::refresh()
 {
 	bool rc = false;
 	if(!var)	return;
 	infoDlg->allowRefresh=false;
-	if(nx!=var->nx)	{	nx = var->nx;	tab->setColumnCount(nx);	rc=true;	}
-	if(ny!=var->ny)	{	ny = var->ny;	tab->setRowCount(ny);	rc=true;	}
-	if(kz>=var->nz)	{	kz = 0;			emit sliceChanged(0);	}
-	if(nz!=var->ny)	{	nz = var->nz;	emit nzChanged(nz);		}
-	id = QString(var->id.c_str());
+	if(nx!=var->GetNx())	{	nx = var->GetNx();	tab->setColumnCount(nx);	rc=true;	}
+	if(ny!=var->GetNy())	{	ny = var->GetNy();	tab->setRowCount(ny);	rc=true;	}
+	if(kz>=var->GetNz())	{	kz = 0;	emit sliceChanged(0);	}
+	if(nz!=var->GetNz())	{	nz = var->GetNz();	emit nzChanged(nz);		}
+	const mglData *dd = dynamic_cast<const mglData *>(var);	if(dd)	id = QString(dd->id.c_str());
+	const mglDataC *dc = dynamic_cast<const mglDataC *>(var);	if(dc)	id = QString(dc->id.c_str());
 	if(nz==1 && ny>1 && !id.isEmpty())
 	{
 		QStringList head;
@@ -112,8 +113,9 @@ void DatPanel::refresh()
 	}
 	for(i=0;i<nx;i++)	for(j=0;j<ny;j++)
 	{
-		f = var->GetVal(i,j,kz);
+		f = var->v(i,j,kz);
 		if(mgl_isnan(f))	s = "nan";
+		else if(mgl_isbad(f))	s=f>0?"inf":"-inf";
 		else	s.sprintf("%g",f);
 		tab->item(j,i)->setText(s);
 	}
@@ -125,7 +127,7 @@ void DatPanel::refresh()
 	ready = true;
 }
 //-----------------------------------------------------------------------------
-void DatPanel::setVar(mglVar *v)
+void DatPanel::setVar(mglDataA *v)
 {
 	ready = false;
 	if(var)	var->o = 0;
@@ -160,13 +162,18 @@ void DatPanel::putValue(int r, int c)
 	if(!var || r<0 || c<0 || r>=ny || c>=nx || !ready)	return;
 	QString s = tab->item(r,c)->text().toLower();
 	mreal f;
-	f = s=="nan" ? NAN : s.toDouble();
-	if(f!=var->GetVal(c,r,kz))
+	if(s=="nan")	f=NAN;
+	else if(s=="inf")	f=INFINITY;
+	else if(s=="-inf")	f=-INFINITY;
+	else	f = s.toDouble();
+	if(f!=var->v(c,r,kz))
 	{
-		if(mgl_isnan(f))	s="nan";	else	s.sprintf("%g", f);
+		if(mgl_isnan(f))	s="nan";
+		else if(mgl_isbad(f))	s=f>0?"inf":"-inf";
+		else	s.sprintf("%g", f);
 		tab->item(r,c)->setText(s);
 	}
-	var->SetVal(f,c,r,kz);
+	var->set_v(f,c,r,kz);
 	infoDlg->refresh();
 }
 //-----------------------------------------------------------------------------
@@ -193,6 +200,7 @@ void DatPanel::save()
 //-----------------------------------------------------------------------------
 void DatPanel::load()
 {
+	mglData *d = dynamic_cast<mglData *>(var);	if(!d)	return;
 	QString fn = QFileDialog::getOpenFileName(this, tr("UDAV - Load data"), "",
 				tr("Data files (*.dat)\nHDF5 files (*.h5 *.hdf)\nPNG files (*.png)\nAll files (*.*)"));
 	if(fn.isEmpty())	return;
@@ -201,15 +209,15 @@ void DatPanel::load()
 	{
 		bool ok;
 		QString s = QInputDialog::getText(this, tr("UDAV - Import PNG"), tr("Enter color scheme for picture"), QLineEdit::Normal, MGL_DEF_SCH, &ok);
-		if(ok)	var->Import(fn.toStdString().c_str(), s.toStdString().c_str());
+		if(ok)	d->Import(fn.toStdString().c_str(), s.toStdString().c_str());
 	}
 	else if(ext=="h5" || ext=="hdf")
 	{
 		bool ok;
 		QString s = QInputDialog::getText(this, tr("UDAV - Read from HDF"), tr("Enter data name"), QLineEdit::Normal, QString::fromStdWString(var->s), &ok);
-		if(ok)	var->ReadHDF(fn.toStdString().c_str(), s.toStdString().c_str());
+		if(ok)	d->ReadHDF(fn.toStdString().c_str(), s.toStdString().c_str());
 	}
-	else 	var->Read(fn.toStdString().c_str());
+	else 	d->Read(fn.toStdString().c_str());
 	refresh();
 }
 //-----------------------------------------------------------------------------
@@ -243,7 +251,7 @@ void DatPanel::paste()
 		{
 			t = s.section('\t',j,j,QString::SectionSkipEmpty);
 			if(t.isEmpty())	{	j=nx;	continue;	}
-			var->SetVal(t.toDouble(),j+c,i+r,kz);
+			var->set_v(t.toDouble(),j+c,i+r,kz);
 		}
 	}
 	refresh();
@@ -278,7 +286,12 @@ void DatPanel::inrange()
 	QString v1("-1"), v2("1"), dir("x");
 	if(sizesDialog(tr("UDAV - Fill data"), tr("Enter range for data and direction of filling"), tr("From"), tr("To"), tr("Direction"), v1, v2, dir))
 	{
-		var->Fill(v1.toDouble(), v2.toDouble(), dir[0].toLatin1());
+		mglData *d = dynamic_cast<mglData *>(var);
+		if(d)	d->Fill(v1.toDouble(), v2.toDouble(), dir[0].toLatin1());
+		mglDataC *dc = dynamic_cast<mglDataC *>(var);
+		if(dc)	dc->Fill(v1.toDouble(), v2.toDouble(), dir[0].toLatin1());
+		mglDataV *dv = dynamic_cast<mglDataV *>(var);
+		if(dv)	dv->Fill(v1.toDouble(), v2.toDouble(), dir[0].toLatin1());
 		refresh();
 	}
 }
@@ -288,7 +301,8 @@ void DatPanel::norm()
 	QString v1("0"), v2("1"), how;
 	if(sizesDialog(tr("UDAV - Normalize data"), tr("Enter range for final data"), tr("From"), tr("To"), tr("Symmetrical?"), v1, v2, how))
 	{
-		var->Norm(v1.toDouble(), v2.toDouble(), (how=="on" || how.contains('s')));
+		mglData *d = dynamic_cast<mglData *>(var);
+		if(d)	d->Norm(v1.toDouble(), v2.toDouble(), (how=="on" || how.contains('s')));
 		refresh();
 	}
 }
@@ -298,7 +312,8 @@ void DatPanel::normsl()
 	QString v1("0"), v2("1"), dir("z");
 	if(sizesDialog(tr("UDAV - Normalize by slice"), tr("Enter range for final data"), tr("From"), tr("To"), tr("Direction"), v1, v2, dir))
 	{
-		var->NormSl(v1.toDouble(), v2.toDouble(), dir[0].toLatin1());
+		mglData *d = dynamic_cast<mglData *>(var);
+		if(d)	d->NormSl(v1.toDouble(), v2.toDouble(), dir[0].toLatin1());
 		refresh();
 	}
 }
@@ -308,7 +323,10 @@ void DatPanel::create()
 	QString mx, my("1"), mz("1");
 	if(sizesDialog(tr("UDAV - Clear data"), tr("Enter new data sizes"), tr("X-size"), tr("Y-size"), tr("Z-size"), mx, my, mz))
 	{
-		var->Create(mx.toInt(), my.toInt(), mz.toInt());
+		mglData *d = dynamic_cast<mglData *>(var);
+		if(d)	d->Create(mx.toInt(), my.toInt(), mz.toInt());
+		mglDataC *c = dynamic_cast<mglDataC *>(var);
+		if(c)	c->Create(mx.toInt(), my.toInt(), mz.toInt());
 		refresh();	updateDataItems();
 	}
 }
@@ -319,7 +337,8 @@ void DatPanel::reSize()
 	mx.sprintf("%d",nx);	my.sprintf("%d",ny);	mz.sprintf("%d",nz);
 	if(sizesDialog(tr("UDAV - Resize data"), tr("Enter new data sizes"), tr("X-size"), tr("Y-size"), tr("Z-size"), mx, my, mz))
 	{
-		var->Set(var->Resize(mx.toInt(), my.toInt(), mz.toInt()));
+		mglData *d = dynamic_cast<mglData *>(var);
+		if(d)	d->Set(d->Resize(mx.toInt(), my.toInt(), mz.toInt()));
 		refresh();	updateDataItems();
 	}
 }
@@ -329,7 +348,8 @@ void DatPanel::squize()
 	QString mx("1"), my("1"), mz("1");
 	if(sizesDialog(tr("UDAV - Squeeze data"), tr("Enter step of saved points. For example, '1' save all, '2' save each 2nd point, '3' save each 3d and so on."), tr("X-direction"), tr("Y-direction"), tr("Z-direction"), mx, my, mz))
 	{
-		var->Squeeze(mx.toInt(), my.toInt(), mz.toInt());
+		mglData *d = dynamic_cast<mglData *>(var);
+		if(d)	d->Squeeze(mx.toInt(), my.toInt(), mz.toInt());
 		refresh();	updateDataItems();
 	}
 }
@@ -339,7 +359,8 @@ void DatPanel::crop()
 	QString n1("1"), n2("1"), dir;
 	if(sizesDialog(tr("UDAV - Crop data"), tr("Enter range of saved date."), tr("From"), tr("To"), tr("Direction"), n1, n2, dir))
 	{
-		var->Squeeze(n1.toInt(), n2.toInt(), dir[0].toLatin1());
+		mglData *d = dynamic_cast<mglData *>(var);
+		if(d)	d->Squeeze(n1.toInt(), n2.toInt(), dir[0].toLatin1());
 		refresh();	updateDataItems();
 	}
 }
@@ -350,7 +371,8 @@ void DatPanel::rearrange()
 	mx.sprintf("%d",nx);	my.sprintf("%d",ny);	mz.sprintf("%d",nz);
 	if(sizesDialog(tr("UDAV - Rearrange data"), tr("Enter new data sizes"), tr("X-size"), tr("Y-size"), tr("Z-size"), mx, my, mz))
 	{
-		var->Rearrange(mx.toInt(), my.toInt(), mz.toInt());
+		mglData *d = dynamic_cast<mglData *>(var);
+		if(d)	d->Rearrange(mx.toInt(), my.toInt(), mz.toInt());
 		refresh();	updateDataItems();
 	}
 }
@@ -379,9 +401,8 @@ void DatPanel::hist()
 	bool res = d->exec();
 	if(res && !v1->text().isEmpty() && !v2->text().isEmpty() && !id->text().isEmpty())
 	{
-		mglVar *vv = parser.AddVar(id->text().toStdString().c_str());
-		if(!vv)	return;
-		vv->Set(var->Hist(nm->value(), v1->text().toDouble(), v2->text().toDouble()));
+		mglData *vv = parser.AddVar(id->text().toStdString().c_str());
+		vv->Set(mglData(true,mgl_data_hist(var, nm->value(), v1->text().toDouble(), v2->text().toDouble(),0)));
 		updateDataItems();
 	}
 }
@@ -435,14 +456,9 @@ bool DatPanel::sizesDialog(const QString &cap, const QString &lab, const QString
 	return res;
 }
 //-----------------------------------------------------------------------------
-#include "xpm/plot.xpm"
 #include "xpm/size.xpm"
-//#include "xpm/smth.xpm"
 #include "xpm/crop.xpm"
 #include "xpm/squize.xpm"
-//#include "xpm/sum.xpm"
-//#include "xpm/func.xpm"
-//#include "xpm/swap.xpm"
 #include "xpm/hist.xpm"
 #include "xpm/oper_dir.xpm"
 #include "xpm/oper_of.xpm"
@@ -515,7 +531,7 @@ void DatPanel::newdat()
 	{
 		mglGraph gr;
 		parser.Execute(&gr,mgl.toStdString().c_str());
-		opers += mgl+"\n";
+		if(k>=6)	opers += mgl+"\n";
 		updateDataItems();
 	}
 }
@@ -599,13 +615,13 @@ void DatPanel::toolTop(QBoxLayout *l)
 
 	// file menu
 	o = menu->addMenu(tr("File"));
-	a = new QAction(QPixmap(":/xpm/document-open.png"), tr("Load data"), this);
+	a = new QAction(QPixmap(":/png/document-open.png"), tr("Load data"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(load()));
 	a->setToolTip(tr("Load data from file. Data will be deleted only\nat exit but UDAV will not ask to save it (Ctrl+Shift+O)."));
 	a->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_O);	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/document-save.png"), tr("Save data"), this);
+	a = new QAction(QPixmap(":/png/document-save.png"), tr("Save data"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(save()));
 	a->setToolTip(tr("Save data to a file (Ctrl+Shift+S)."));
 	a->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_S);	o->addAction(a);
@@ -618,19 +634,19 @@ void DatPanel::toolTop(QBoxLayout *l)
 //	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
 
-	a = new QAction(QPixmap(plot_xpm), tr("Plot data"), this);
+	a = new QAction(QPixmap(":/png/office-chart-line.png"), tr("Plot data"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(plot()));
 	a->setToolTip(tr("Plot data in new script window. You may select the kind\nof plot, its style and so on."));
 	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/edit-copy.png"), tr("Copy data"), this);
+	a = new QAction(QPixmap(":/png/edit-copy.png"), tr("Copy data"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(copy()));
 	a->setToolTip(tr("Copy range of numbers to clipboard (Ctrl+Shift+C)."));
 	a->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_C);	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/edit-paste.png"), tr("Paste data"), this);
+	a = new QAction(QPixmap(":/png/edit-paste.png"), tr("Paste data"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(copy()));
 	a->setToolTip(tr("Paste range of numbers from clipboard (Ctrl+Shift+P)."));
 	a->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_V);	o->addAction(a);
@@ -638,13 +654,13 @@ void DatPanel::toolTop(QBoxLayout *l)
 
 	// navigation menu
 	o = menu->addMenu(tr("Navigate"));
-	a = new QAction(QPixmap(":/xpm/go-first.png"), tr("First slice"), this);
+	a = new QAction(QPixmap(":/png/go-first.png"), tr("First slice"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(first()));
 	a->setToolTip(tr("Go to the first data slice for 3D data."));
 	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/go-previous.png"), tr("Prev. slice"), this);
+	a = new QAction(QPixmap(":/png/go-previous.png"), tr("Prev. slice"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(prev()));
 	a->setToolTip(tr("Go to the previous data slice for 3D data."));
 	o->addAction(a);
@@ -662,13 +678,13 @@ void DatPanel::toolTop(QBoxLayout *l)
 	a->setToolTip(tr("Go to the specified data slice for 3D data."));
 	o->addAction(a);
 
-	a = new QAction(QPixmap(":/xpm/go-next.png"), tr("Next slice"), this);
+	a = new QAction(QPixmap(":/png/go-next.png"), tr("Next slice"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(next()));
 	a->setToolTip(tr("Go to the next data slice for 3D data."));
 	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/go-last.png"), tr("Last slice"), this);
+	a = new QAction(QPixmap(":/png/go-last.png"), tr("Last slice"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(last()));
 	a->setToolTip(tr("Go to the last data slice for 3D data."));
 	o->addAction(a);
@@ -683,7 +699,7 @@ void DatPanel::toolLeft(QBoxLayout *l)
 
 	// size menu
 	o = menu->addMenu(tr("Sizes"));
-	a = new QAction(QPixmap(":/xpm/document-new.png"), tr("Create new"), this);
+	a = new QAction(QPixmap(":/png/document-new.png"), tr("Create new"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(create()));
 	a->setToolTip(tr("Recreate the data with new sizes and fill it by zeros (Ctrl+Shift+N)."));
 	a->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_N);	o->addAction(a);
@@ -725,7 +741,7 @@ void DatPanel::toolLeft(QBoxLayout *l)
 	a->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_H);	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-/*	a = new QAction(QPixmap(":/xpm/view-refresh.png"), tr("Refresh"), this);
+/*	a = new QAction(QPixmap(":/png/view-refresh.png"), tr("Refresh"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(refresh()));
 	a->setToolTip(tr("Refresh data values."));
 	o->addAction(a);
@@ -749,6 +765,11 @@ void DatPanel::toolLeft(QBoxLayout *l)
 	o->addAction(a);*/
 
 	l->addStretch(1);
+
+	a = new QAction(QPixmap(":/png/tab-close.png"), tr("Close tab"), this);
+	connect(a, SIGNAL(triggered()), this, SLOT(close()));
+	a->setToolTip(tr("Close this data tab."));
+	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 }
 //-----------------------------------------------------------------------------
 QString DatPanel::dataName()	{	return QString::fromStdWString(var->s);	}
