@@ -32,7 +32,7 @@
 //-----------------------------------------------------------------------------
 extern bool mglAutoSave;
 extern mglParse parser;
-QWidget *newDataWnd(InfoDialog *inf, QWidget *wnd, mglVar *v);
+QWidget *newDataWnd(InfoDialog *inf, QWidget *wnd, mglDataA *v);
 void refreshData(QWidget *w);
 //-----------------------------------------------------------------------------
 QWidget *createMemPanel(QWidget *p)	// NOTE: parent should be MainWindow
@@ -57,23 +57,23 @@ MemPanel::MemPanel(QWidget *parent) : QWidget(parent)
 	infoDlg->setModal(true);	infoDlg->allowRefresh=false;
 
 	v = new QVBoxLayout(this);	h = new QHBoxLayout();	v->addLayout(h);
-	b = new QToolButton(this);	b->setIcon(QPixmap(":/xpm/document-new.png"));
+	b = new QToolButton(this);	b->setIcon(QPixmap(":/png/document-new.png"));
 	b->setToolTip(tr("Create new data array"));		h->addWidget(b);
 	connect(b, SIGNAL(clicked()), this, SLOT(newTable()));
 	b = new QToolButton(this);	b->setIcon(QPixmap(table_xpm));
 	b->setToolTip(tr("Edit selected data array"));	h->addWidget(b);
 	connect(b, SIGNAL(clicked()), this, SLOT(editData()));
-	b = new QToolButton(this);	b->setIcon(QPixmap(":/xpm/edit-delete.png"));
+	b = new QToolButton(this);	b->setIcon(QPixmap(":/png/edit-delete.png"));
 	b->setToolTip(tr("Delete selected data array"));		h->addWidget(b);
 	connect(b, SIGNAL(clicked()), this, SLOT(delData()));
 	b = new QToolButton(this);	b->setIcon(QPixmap(preview_xpm));
 	b->setToolTip(tr("Properties of selected data array"));	h->addWidget(b);
 	connect(b, SIGNAL(clicked()), this, SLOT(infoData()));
-	b = new QToolButton(this);	b->setIcon(QPixmap(":/xpm/view-refresh.png"));
+	b = new QToolButton(this);	b->setIcon(QPixmap(":/png/view-refresh.png"));
 	b->setToolTip(tr("Update list of data arrays"));		h->addWidget(b);
 	connect(b, SIGNAL(clicked()), this, SLOT(refresh()));
 	h->addStretch(1);
-	b = new QToolButton(this);	b->setIcon(QPixmap(":/xpm/edit-delete.png"));
+	b = new QToolButton(this);	b->setIcon(QPixmap(":/png/edit-clear.png"));
 	b->setToolTip(tr("Delete ALL data arrays"));	h->addWidget(b);
 	connect(b, SIGNAL(clicked()), this, SLOT(delAllData()));
 
@@ -98,7 +98,7 @@ void MemPanel::newTable()
 	QString name = QInputDialog::getText(this, tr("UDAV - New variable"),
 				tr("Enter name for new variable"), QLineEdit::Normal, "", &ok);
 	if(!ok || name.isEmpty())	return;
-	mglVar *v = parser.AddVar(name.toStdString().c_str());
+	mglData *v = parser.AddVar(name.toStdString().c_str());
 	QWidget *t;
 	if(v->o)	t = (QWidget *)v->o;
 	else		t = newDataWnd(infoDlg,wnd,v);
@@ -111,7 +111,7 @@ void MemPanel::editData(int n)
 	if(tab->rowCount()<1)	return;
 	if(n<0)	n = tab->currentRow();
 	if(n<0)	n = 0;
-	mglVar *v = parser.FindVar(tab->item(n,0)->text().toStdString().c_str());
+	mglDataA *v = parser.FindVar(tab->item(n,0)->text().toStdString().c_str());
 	if(!v)	return;
 	QWidget *t;
 	if(v->o)	t = (QWidget *)v->o;
@@ -124,7 +124,7 @@ void MemPanel::delData()
 	if(tab->rowCount()<1)	return;
 	int	n = tab->currentRow();
 	if(n<0)	n = 0;
-	mglVar *v = parser.FindVar(tab->item(n,0)->text().toStdString().c_str());
+	mglDataA *v = parser.FindVar(tab->item(n,0)->text().toStdString().c_str());
 	if(!v && v->o)	((QWidget *)v->o)->close();
 	parser.DeleteVar(tab->item(n,0)->text().toStdString().c_str());
 	refresh();
@@ -143,7 +143,7 @@ void MemPanel::infoData()
 	if(tab->rowCount()<1)	return;
 	int	n = tab->currentRow();
 	if(n<0)	n = 0;
-	mglVar *v = parser.FindVar(tab->item(n,0)->text().toStdString().c_str());
+	mglDataA *v = parser.FindVar(tab->item(n,0)->text().toStdString().c_str());
 	if(!v)	return;
 	infoDlg->setVar(v);
 	QString s = QString::fromStdWString(v->s);
@@ -154,29 +154,29 @@ void MemPanel::infoData()
 //-----------------------------------------------------------------------------
 void MemPanel::refresh()
 {
-	mglVar *v = parser.FindVar("");
-	int n = 0;
-	while(v)	{	v = v->next;	n++;	}
-	tab->setRowCount(n);
-	v = parser.FindVar("");	n = 0;
+	long n = parser.GetNumVar(), m=0;
+	for(long i=0;i<n;i++)	if(parser.GetVar(i))	m++;
+	tab->setRowCount(m);
 	QString s;
 	QTableWidgetItem *it;
 	Qt::ItemFlags flags=Qt::ItemIsSelectable|Qt::ItemIsEnabled;
-	while(v)
+	for(long i=m=0;i<n;i++)
 	{
+		mglDataA *v = parser.GetVar(i);
+		if(!v)	continue;
 		s = QString::fromStdWString(v->s);
 		it = new QTableWidgetItem(s);
-		tab->setItem(n,0,it);	it->setFlags(flags);
-		s.sprintf("%ld * %ld * %ld", v->nx, v->ny, v->nz);
+		tab->setItem(m,0,it);	it->setFlags(flags);
+		s.sprintf("%ld * %ld * %ld", v->GetNx(), v->GetNy(), v->GetNz());
 		it = new QTableWidgetItem(s);
-		tab->setItem(n,1,it);	it->setFlags(flags);
+		tab->setItem(m,1,it);	it->setFlags(flags);
 		it->setTextAlignment(Qt::AlignHCenter|Qt::AlignVCenter);
-		s.sprintf("%12ld", v->nx*v->ny*v->nz*sizeof(mreal));
+		s.sprintf("%12ld", v->GetNN()*sizeof(mreal));
 		it = new QTableWidgetItem(s);
-		tab->setItem(n,2,it);	it->setFlags(flags);
+		tab->setItem(m,2,it);	it->setFlags(flags);
 		it->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
 		if(v->o)	refreshData((QWidget *)v->o);
-		v = v->next;	n++;
+		m++;
 	}
 	tab->sortItems(colSort);
 }
