@@ -54,7 +54,7 @@ DataOpenDialog::DataOpenDialog(QWidget *parent) : QDialog(parent)
 
 	a = new QHBoxLayout;	o->addLayout(a);
 	l = new QLabel(tr("Data name"));	a->addWidget(l);
-	char buf[32];	snprintf(buf,32,"mgl_%d",numDataOpened);
+	char buf[32];	snprintf(buf,32,"mgl_%d",numDataOpened);	buf[31]=0;
 	name = new QLineEdit(buf,this);		a->addWidget(name);
 
 	rA = new QRadioButton(tr("Auto detect data sizes"), this);
@@ -114,9 +114,10 @@ void DataOpenDialog::prepareResult()
 {
 	code = "";	numDataOpened++;	data = name->text();
 	// prepare unique value of name for next time
-	char buf[32];	snprintf(buf,32,"mgl_%d",numDataOpened);	name->setText(buf);
-	mglVar *v = parser.AddVar(data.toStdString().c_str());
-	bool dd=0;
+	char buf[32];	snprintf(buf,32,"mgl_%d",numDataOpened);
+	buf[31]=0;	name->setText(buf);
+	mglData *v = parser.AddVar(data.toStdString().c_str());
+	int dd=0;
 	if(rA->isChecked())	//	auto sizes
 	{
 		setlocale(LC_NUMERIC, "C");	v->Read(file.toStdString().c_str());	setlocale(LC_NUMERIC, "");
