@@ -31,14 +31,20 @@
 #include <QMenuBar>
 #include <QMessageBox>
 #include <QTextEdit>
+#include <QTextBlock>
 
 #include <QMdiArea>
+#include "udav_wnd.h"
 #include "mgl2/qmathgl.h"
 #include "plot_pnl.h"
 #include "anim_dlg.h"
 #include "style_dlg.h"
+#include "newcmd_dlg.h"
+#include "subplot_dlg.h"
+
 extern bool mglAutoSave;
 extern bool mglHighlight;
+extern bool mglDotsRefr;
 extern mglParse parser;
 int animDelay=500;
 void raisePanel(QWidget *w);
@@ -48,10 +54,14 @@ PlotPanel::PlotPanel(QWidget *parent) : QWidget(parent)
 	gifOn = jpgOn = false;
 	animDialog = new AnimParam(this);	animPos = -1;
 	stlDialog = new StyleDialog(this);
-	printer = new QPrinter;		curPos = -1;
+	newCmdDlg = new NewCmdDialog(this);
+	subplotDlg = new SubplotDialog(this);
+	printer = new QPrinter;		curPos = subId = -1;
 	timer = new QTimer(this);
 	connect(timer, SIGNAL(timeout()), this, SLOT(next()));
 	connect(animDialog, SIGNAL(putText(const QString &)), this, SLOT(animText(const QString &)));
+	connect(newCmdDlg, SIGNAL(result(const QString&, bool)), this, SLOT(putCmd(const QString&)));
+	connect(subplotDlg, SIGNAL(result(const QString&)), this, SLOT(insCmd(const QString&)));
 
 	menu = new QMenu(tr("Graphics"),this);
 	popup = new QMenu(this);
@@ -59,6 +69,7 @@ PlotPanel::PlotPanel(QWidget *parent) : QWidget(parent)
 	draw = new mglDrawScript(parser.Self());
 	mgl_set_flag(mgl->getGraph(),1,MGL_SHOW_POS);	mgl->setDraw(draw);
 	connect(mgl,SIGNAL(askStyle(int)),this,SLOT(setStyle(int)));
+	connect(mgl,SIGNAL(objChanged(int)),this,SLOT(setCurPos(int)));
 
 	QBoxLayout *v,*h,*m;
 	v = new QVBoxLayout(this);
@@ -76,7 +87,9 @@ PlotPanel::~PlotPanel()	{	delete printer;	}
 void PlotPanel::setStyle(int id)
 {	if(stlDialog->exec())	mgl->setStyle(id, stlDialog->getStyle());	}
 //-----------------------------------------------------------------------------
-void PlotPanel::animText(const QString &txt)		{	animPutText(txt);	}
+void PlotPanel::setSubId(int id)	{	subId = id;	}
+//-----------------------------------------------------------------------------
+void PlotPanel::animText(const QString &txt)	{	animPutText(txt);	}
 //-----------------------------------------------------------------------------
 void PlotPanel::setCurPos(int pos)
 {
@@ -89,7 +102,9 @@ void PlotPanel::stop()	{	parser.Stop();	mgl->stop();	}
 void PlotPanel::execute()
 {
 	if(mglAutoSave)	save();
+	mgl->setDotsPreview(mglDotsRefr);
 	raisePanel(this);
+	objId = subId = -1;
 	emit clearWarn();
 	QTime t;	t.start();
 	mgl_set_facenum(mgl->getGraph(),0);
@@ -256,7 +271,7 @@ void PlotPanel::animParseText(const QString &txt)
 	}
 }
 //-----------------------------------------------------------------------------
-#include "xpm/wire.xpm"
+//#include "xpm/wire.xpm"
 #include "xpm/text.xpm"
 #include "xpm/line.xpm"
 #include "xpm/curve.xpm"
@@ -264,15 +279,18 @@ void PlotPanel::animParseText(const QString &txt)
 #include "xpm/mark_s.xpm"
 #include "xpm/mark_a.xpm"
 #include "xpm/mark_d.xpm"
+#include "xpm/arc.xpm"
+#include "xpm/polygon.xpm"
+#include "xpm/box.xpm"
 //-----------------------------------------------------------------------------
 void PlotPanel::toolTop(QBoxLayout *l)
 {
-	QAction *a;
+	QAction *a, *aa;
 	QMenu *o=menu, *oo;
 	QToolButton *bb;
 
 	// graphics menu
-	a = new QAction(QPixmap(":/xpm/alpha.png"), tr("Alpha"), this);
+	a = new QAction(QPixmap(":/png/alpha.png"), tr("Alpha"), this);
 	a->setShortcut(Qt::CTRL+Qt::Key_T);	a->setCheckable(true);
 	connect(a, SIGNAL(toggled(bool)), mgl, SLOT(setAlpha(bool)));
 	connect(mgl, SIGNAL(alphaChanged(bool)), a, SLOT(setChecked(bool)));
@@ -280,7 +298,7 @@ void PlotPanel::toolTop(QBoxLayout *l)
 	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/weather-clear.png"), tr("Light"), this);
+	a = new QAction(QPixmap(":/png/weather-clear.png"), tr("Light"), this);
 	a->setShortcut(Qt::CTRL+Qt::Key_L);	a->setCheckable(true);
 	connect(a, SIGNAL(toggled(bool)), mgl, SLOT(setLight(bool)));
 	connect(mgl, SIGNAL(lightChanged(bool)), a, SLOT(setChecked(bool)));
@@ -288,21 +306,21 @@ void PlotPanel::toolTop(QBoxLayout *l)
 	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(wire_xpm), tr("Grid"), this);
+	a = new QAction(QPixmap(":/png/view-grid.png"), tr("Grid"), this);
 	a->setShortcut(Qt::CTRL+Qt::Key_G);	a->setCheckable(true);
 	connect(a, SIGNAL(toggled(bool)), mgl, SLOT(setGrid(bool)));
 	a->setToolTip(tr("Switch on/off grid of absolute coordinates (Ctrl+G)."));
 	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/object-rotate-right.png"), tr("Rotate by mouse"), this);
+	a = new QAction(QPixmap(":/png/transform-move.png"), tr("Rotate by mouse"), this);
 	a->setCheckable(true);
 	connect(a, SIGNAL(toggled(bool)), mgl, SLOT(setRotate(bool)));
 	connect(mgl, SIGNAL(rotateChanged(bool)), a, SLOT(setChecked(bool)));
 	a->setToolTip(tr("Switch on/off mouse handling of the graphics\n(rotation, shifting, zooming and perspective)."));
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-/*	a = new QAction(QPixmap(":/xpm/zoom-fit-best.png"), tr("Zoom by mouse"), this);
+/*	a = new QAction(QPixmap(":/png/zoom-fit-best.png"), tr("Zoom by mouse"), this);
 	a->setCheckable(true);
 	connect(a, SIGNAL(toggled(bool)), mgl, SLOT(setZoom(bool)));
 	connect(mgl, SIGNAL(zoomChanged(bool)), a, SLOT(setChecked(bool)));
@@ -310,91 +328,149 @@ void PlotPanel::toolTop(QBoxLayout *l)
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);*/
 
 	o->addSeparator();
-	a = new QAction(QPixmap(":/xpm/zoom-original.png"), tr("Restore"), this);
+	a = new QAction(QPixmap(":/png/zoom-original.png"), tr("Restore"), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(restore()));
 	a->setToolTip(tr("Restore default graphics rotation, zoom and perspective (Ctrl+Space)."));
 	a->setShortcut(Qt::CTRL+Qt::Key_Space);
 	o->addAction(a);	popup->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/view-refresh.png"), tr("Redraw"), this);
+	a = new QAction(QPixmap(":/png/view-refresh.png"), tr("Redraw"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(execute()));
 	a->setToolTip(tr("Execute script and redraw graphics (F5)."));
 	a->setShortcut(Qt::Key_F5);
 	o->addAction(a);	popup->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(tr("Adjust size"), this);
+	a = new QAction(QPixmap(":/png/view-fullscreen.png"), tr("Adjust size"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(adjust()));
 	a->setToolTip(tr("Change canvas size to fill whole region (F6)."));
 	a->setShortcut(Qt::Key_F6);		o->addAction(a);
 
-	a = new QAction(tr("Reload"), this);
+	a = new QAction(QPixmap(":/png/document-revert.png"), tr("Reload"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(pressF9()));
 	a->setToolTip(tr("Restore status for 'once' command and reload data (F9)."));
 	a->setShortcut(Qt::Key_F9);	o->addAction(a);	popup->addAction(a);
 
-	a = new QAction(QPixmap(":/xpm/process-stop.png"), tr("Stop"), this);
+	a = new QAction(QPixmap(":/png/process-stop.png"), tr("Stop"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(stop()));
 	a->setToolTip(tr("Stop script execution (F7)."));
 	a->setShortcut(Qt::Key_F7);	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/edit-copy.png"), tr("Copy plot"), this);
+	a = new QAction(QPixmap(":/png/edit-copy.png"), tr("Copy plot"), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(copy()));
 	a->setToolTip(tr("Copy graphics to clipboard (Ctrl+Shift+G)."));
 	a->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_G);
 	o->addAction(a);	popup->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/edit-copy.png"), tr("Copy click coor."), this);
+	a = new QAction(QPixmap(":/png/edit-copy.png"), tr("Copy click coor."), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(copyClickCoor()));
 	a->setToolTip(tr("Copy coordinates of last mouse click to clipboard."));
 	o->addAction(a);	popup->addAction(a);
 
-	l->addStretch(1);
-	oo = new QMenu(tr("Primitives ..."),this);
-	a = new QAction(QPixmap(line_xpm), tr("Add line"), this);
-	connect(a, SIGNAL(triggered()), mgl, SLOT(addLine()));
-	connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
-	a->setToolTip(tr("Add line which properties can be changed later by mouse."));
-	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);	oo->addAction(a);
+//	l->addStretch(1);
+	{
+		oo = new QMenu(tr("Primitives ..."),this);
+		aa=a = new QAction(QPixmap(line_xpm), tr("Add line"), this);
+		connect(a, SIGNAL(triggered()), mgl, SLOT(addLine()));
+		connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
+		a->setToolTip(tr("Add line which properties can be changed later by mouse."));
+		oo->addAction(a);
 
-	a = new QAction(QPixmap(curve_xpm), tr("Add curve"), this);
-	connect(a, SIGNAL(triggered()), mgl, SLOT(addCurve()));
-	connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
-	a->setToolTip(tr("Add curve which properties can be changed later by mouse."));
-	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);	oo->addAction(a);
+		a = new QAction(QPixmap(arc_xpm), tr("Add arc"), this);
+		connect(a, SIGNAL(triggered()), mgl, SLOT(addArc()));
+		connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
+		a->setToolTip(tr("Add arc which properties can be changed later by mouse."));
+		oo->addAction(a);
 
-	a = new QAction(QPixmap(mark_s_xpm), tr("Add rect"), this);
-	connect(a, SIGNAL(triggered()), mgl, SLOT(addRect()));
-	connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
-	a->setToolTip(tr("Add rectangle which properties can be changed later by mouse."));
-	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);	oo->addAction(a);
+		a = new QAction(QPixmap(curve_xpm), tr("Add curve"), this);
+		connect(a, SIGNAL(triggered()), mgl, SLOT(addCurve()));
+		connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
+		a->setToolTip(tr("Add curve which properties can be changed later by mouse."));
+		oo->addAction(a);
 
-	a = new QAction(QPixmap(mark_d_xpm), tr("Add rhombus"), this);
-	connect(a, SIGNAL(triggered()), mgl, SLOT(addRhomb()));
-	connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
-	a->setToolTip(tr("Add rhombus which properties can be changed later by mouse."));
-	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);	oo->addAction(a);
+		a = new QAction(QPixmap(mark_s_xpm), tr("Add rect"), this);
+		connect(a, SIGNAL(triggered()), mgl, SLOT(addRect()));
+		connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
+		a->setToolTip(tr("Add rectangle which properties can be changed later by mouse."));
+		oo->addAction(a);
 
-	a = new QAction(QPixmap(mark_o_xpm), tr("Add ellipse"), this);
-	connect(a, SIGNAL(triggered()), mgl, SLOT(addEllipse()));
-	connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
-	a->setToolTip(tr("Add ellipse which properties can be changed later by mouse."));
-	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);	oo->addAction(a);
+		a = new QAction(QPixmap(mark_d_xpm), tr("Add rhombus"), this);
+		connect(a, SIGNAL(triggered()), mgl, SLOT(addRhomb()));
+		connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
+		a->setToolTip(tr("Add rhombus which properties can be changed later by mouse."));
+		oo->addAction(a);
 
-	a = new QAction(QPixmap(mark_a_xpm), tr("Add mark"), this);
-	connect(a, SIGNAL(triggered()), mgl, SLOT(addMark()));
-	connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
-	a->setToolTip(tr("Add marker which properties can be changed later by mouse."));
-	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);	oo->addAction(a);
+		a = new QAction(QPixmap(mark_o_xpm), tr("Add ellipse"), this);
+		connect(a, SIGNAL(triggered()), mgl, SLOT(addEllipse()));
+		connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
+		a->setToolTip(tr("Add ellipse which properties can be changed later by mouse."));
+		oo->addAction(a);
+
+		a = new QAction(QPixmap(polygon_xpm), tr("Add polygon"), this);
+		connect(a, SIGNAL(triggered()), mgl, SLOT(addPolygon()));
+		connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
+		a->setToolTip(tr("Add ellipse which properties can be changed later by mouse."));
+		oo->addAction(a);
+
+		a = new QAction(QPixmap(mark_a_xpm), tr("Add mark"), this);
+		connect(a, SIGNAL(triggered()), mgl, SLOT(addMark()));
+		connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
+		a->setToolTip(tr("Add marker which properties can be changed later by mouse."));
+		oo->addAction(a);
+
+		a = new QAction(QPixmap(text_xpm), tr("Add text"), this);
+		connect(a, SIGNAL(triggered()), mgl, SLOT(addText()));
+		connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
+		a->setToolTip(tr("Add text which properties can be changed later by mouse."));
+		oo->addAction(a);
+
+		bb = new QToolButton(this);	l->addWidget(bb);
+		bb->setDefaultAction(aa);	bb->setMenu(oo);
+		bb->setPopupMode(QToolButton::MenuButtonPopup);
+	}
+
+	a = new QAction(QPixmap(":/png/edit-delete.png"), tr("Delete selected"), this);
+	connect(a, SIGNAL(triggered()), this, SLOT(deleteSelected()));
+	a->setToolTip(tr("Delete selected plot."));
+	o->addAction(a);	popup->addAction(a);
+	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
+
+	a = new QAction(QPixmap(":/png/layer-visible-off.png"), tr("Hide selected"), this);
+	connect(a, SIGNAL(triggered()), this, SLOT(hideSelected()));
+	a->setToolTip(tr("Hide selected plot."));
+	o->addAction(a);	popup->addAction(a);
+	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
+
+	const MainWindow *mw=findMain(this);
+	if(mw)	{	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(mw->ahide);	}
+
+	a = new QAction(QPixmap(":/png/format-indent-more.png"), tr("New command"), this);
+	connect(a, SIGNAL(triggered()), newCmdDlg, SLOT(show()));
+	a->setToolTip(tr("Show dialog for new command and put it into the script."));
+	o->addAction(a);	popup->addAction(a);
+	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
+
+	a = new QAction(QPixmap(box_xpm), tr("New inplot"), this);
+	connect(a, SIGNAL(triggered()), subplotDlg, SLOT(show()));
+	a->setToolTip(tr("Show dialog for new inplot and put it into the script."));
+	o->addAction(a);	popup->addAction(a);
+	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
+
+	a = new QAction(QPixmap(":/png/object-order-lower.png"), tr("Move plot up"), this);
+	connect(a, SIGNAL(triggered()), this, SLOT(movePlotUp()));
+	a->setToolTip(tr("Move selected plot up to previous subplot."));
+	o->addAction(a);	popup->addAction(a);
+	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
+
+	a = new QAction(QPixmap(":/png/object-order-raise.png"), tr("Move plot down"), this);
+	connect(a, SIGNAL(triggered()), this, SLOT(movePlotDown()));
+	a->setToolTip(tr("Move selected plot down to next subplot."));
+	o->addAction(a);	popup->addAction(a);
+	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(text_xpm), tr("Add text"), this);
-	connect(a, SIGNAL(triggered()), mgl, SLOT(addText()));
-	connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
-	a->setToolTip(tr("Add text which properties can be changed later by mouse."));
-	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);	oo->addAction(a);
 
 	o->addMenu(oo);	l->addStretch(1);
 
@@ -438,42 +514,42 @@ void PlotPanel::toolLeft(QBoxLayout *l)
 
 	// zooming menu
 	oo = o->addMenu(tr("Zoom/move"));
-	a = new QAction(QPixmap(":/xpm/go-previous.png"), tr("Move left"), this);
+	a = new QAction(QPixmap(":/png/arrow-left.png"), tr("Move left"), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(shiftLeft()));
 	a->setShortcut(Qt::ALT+Qt::Key_Left);
 	a->setToolTip(tr("Move graphics left by 1/3 of its width."));
 	oo->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/go-up.png"), tr("Move up"), this);
+	a = new QAction(QPixmap(":/png/arrow-up.png"), tr("Move up"), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(shiftUp()));
 	a->setShortcut(Qt::ALT+Qt::Key_Up);
 	a->setToolTip(tr("Move graphics up by 1/3 of its height."));
 	oo->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/zoom-in.png"), tr("Zoom in"), this);
+	a = new QAction(QPixmap(":/png/zoom-in.png"), tr("Zoom in"), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(zoomIn()));
 	a->setShortcut(Qt::ALT+Qt::Key_Equal);
 	a->setToolTip(tr("Zoom in graphics."));
 	oo->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/zoom-out.png"), tr("Zoom out"), this);
+	a = new QAction(QPixmap(":/png/zoom-out.png"), tr("Zoom out"), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(zoomOut()));
 	a->setShortcut(Qt::ALT+Qt::Key_Minus);
 	a->setToolTip(tr("Zoom out graphics."));
 	oo->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/go-down.png"), tr("Move down"), this);
+	a = new QAction(QPixmap(":/png/arrow-down.png"), tr("Move down"), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(shiftDown()));
 	a->setShortcut(Qt::ALT+Qt::Key_Down);
 	a->setToolTip(tr("Move graphics up down 1/3 of its height."));
 	oo->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/go-next.png"), tr("Move right"), this);
+	a = new QAction(QPixmap(":/png/arrow-right.png"), tr("Move right"), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(shiftRight()));
 	a->setShortcut(Qt::ALT+Qt::Key_Right);
 	a->setToolTip(tr("Move graphics right by 1/3 of its width."));
@@ -482,38 +558,32 @@ void PlotPanel::toolLeft(QBoxLayout *l)
 
 	// rotate menu
 	oo = o->addMenu(tr("Rotate"));
-	a = new QAction(tr("Rotate up"), this);
+	a = new QAction(QPixmap(":/png/object-rotate-up.png"), tr("Rotate up"), this);
 	a->setShortcut(Qt::SHIFT+Qt::META+Qt::Key_Up);
 	connect(a, SIGNAL(triggered()), tet, SLOT(stepUp()));	oo->addAction(a);
 	a->setToolTip(tr("Increase \\theta angle by 10 degrees."));
-	a = new QAction(tr("Rotate down"), this);
+	a = new QAction(QPixmap(":/png/object-rotate-down.png"), tr("Rotate down"), this);
 	a->setShortcut(Qt::SHIFT+Qt::META+Qt::Key_Down);
 	connect(a, SIGNAL(triggered()), tet, SLOT(stepDown()));	oo->addAction(a);
 	a->setToolTip(tr("Decrease \\theta angle by 10 degrees."));
-	a = new QAction(tr("Rotate left"), this);
+	a = new QAction(QPixmap(":/png/object-rotate-left.png"), tr("Rotate left"), this);
 	a->setShortcut(Qt::SHIFT+Qt::META+Qt::Key_Right);
 	connect(a, SIGNAL(triggered()), phi, SLOT(stepUp()));	oo->addAction(a);
 	a->setToolTip(tr("Increase \\phi angle by 10 degrees."));
-	a = new QAction(tr("Rotate right"), this);
+	a = new QAction(QPixmap(":/png/object-rotate-right.png"), tr("Rotate right"), this);
 	a->setShortcut(Qt::SHIFT+Qt::META+Qt::Key_Left);
 	connect(a, SIGNAL(triggered()), phi, SLOT(stepDown()));	oo->addAction(a);
 	a->setToolTip(tr("Decrease \\phi angle by 10 degrees."));
 
 	// animation menu
 	oo = o->addMenu(tr("Animation"));
-	a = new QAction(QPixmap(":/xpm/media-seek-forward.png"), tr("Next slide"), this);
+	a = new QAction(QPixmap(":/png/media-seek-forward.png"), tr("Next slide"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(nextSlide()));
 	a->setShortcut(Qt::CTRL+Qt::Key_Period);
 	a->setToolTip(tr("Show next slide (Ctrl+.)."));	oo->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/media-seek-backward.png"), tr("Prev slide"), this);
-	connect(a, SIGNAL(triggered()), this, SLOT(prevSlide()));
-	a->setShortcut(Qt::CTRL+Qt::Key_Comma);
-	a->setToolTip(tr("Show previous slide (Ctrl+,)."));	oo->addAction(a);
-	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
-
-	a = new QAction(QPixmap(":/xpm/film-b.png"), tr("Slideshow"), this);
+	a = new QAction(QPixmap(":/png/media-playback-start.png"), tr("Slideshow"), this);
 	a->setCheckable(true);
 	connect(a, SIGNAL(toggled(bool)), this, SLOT(animStart(bool)));
 	connect(this, SIGNAL(animSwitch(bool)),a,SLOT(setChecked(bool)));
@@ -522,8 +592,117 @@ void PlotPanel::toolLeft(QBoxLayout *l)
 	oo->addAction(tr("Setup show"), this, SLOT(animSetup()), Qt::CTRL+Qt::Key_W);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
+	a = new QAction(QPixmap(":/png/media-seek-backward.png"), tr("Prev slide"), this);
+	connect(a, SIGNAL(triggered()), this, SLOT(prevSlide()));
+	a->setShortcut(Qt::CTRL+Qt::Key_Comma);
+	a->setToolTip(tr("Show previous slide (Ctrl+,)."));	oo->addAction(a);
+	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
+
 	l->addStretch(1);
 }
 //-----------------------------------------------------------------------------
 QString PlotPanel::getFit()	{	return QString(mgl_get_fit(mgl->getGraph()));	}
 //-----------------------------------------------------------------------------
+void PlotPanel::deleteSelected()
+{
+	if(curPos>=0)
+	{
+		textMGL->moveCursor(QTextCursor::Start);
+		for(int i=0;i<curPos;i++)	textMGL->moveCursor(QTextCursor::NextBlock);
+		QTextCursor tc= textMGL->textCursor();
+		tc.select(QTextCursor::LineUnderCursor);
+		tc.removeSelectedText();
+		tc.deleteChar();
+		curPos = -1;	execute();
+	}
+	else emit setStatus("No selection.");
+}
+//-----------------------------------------------------------------------------
+void PlotPanel::hideSelected()
+{
+	if(curPos>=0)
+	{
+		textMGL->moveCursor(QTextCursor::Start);
+		for(int i=0;i<curPos;i++)	textMGL->moveCursor(QTextCursor::NextBlock);
+		textMGL->insertPlainText("#h ");
+		curPos = -1;	execute();
+	}
+	else emit setStatus("No selection.");
+}
+//-----------------------------------------------------------------------------
+void PlotPanel::putCmd(const QString &cmd)
+{
+	textMGL->moveCursor(QTextCursor::Start);
+	if(curPos>=0)	for(int i=0;i<curPos;i++)
+		textMGL->moveCursor(QTextCursor::NextBlock);
+	textMGL->insertPlainText(cmd+"\n");
+	curPos = -1;	execute();
+}
+//-----------------------------------------------------------------------------
+void PlotPanel::insCmd(const QString &cmd)
+{
+	textMGL->moveCursor(QTextCursor::EndOfBlock);
+	textMGL->insertPlainText("\n"+cmd);
+	curPos = -1;	execute();
+}
+//-----------------------------------------------------------------------------
+void PlotPanel::movePlotUp()
+{
+	if(curPos>0)
+	{
+		QTextCursor tc = textMGL->textCursor();
+		tc.movePosition(QTextCursor::Start);
+		tc.movePosition(QTextCursor::NextBlock,QTextCursor::MoveAnchor,curPos);
+		tc.select(QTextCursor::BlockUnderCursor);
+		QString s = tc.selectedText();
+		tc.deleteChar();
+		bool ins=true;
+
+		while(tc.movePosition(QTextCursor::PreviousBlock))
+		{
+			QString q = tc.block().text();
+			if(q.startsWith("subplot ") || q.startsWith("inplot ") || q.startsWith("multiplot ") || q.startsWith("gridplot ") || q.startsWith("columnplot ") || q.startsWith("stickplot "))
+			{
+				tc.movePosition(QTextCursor::EndOfBlock);
+				tc.insertText(s);	ins=false;	break;
+			}
+		}
+		if(ins)
+		{
+			tc.movePosition(QTextCursor::Start);	tc.insertText(s+"\n");
+			tc.movePosition(QTextCursor::Start);	tc.deleteChar();
+		}
+		curPos = tc.block().blockNumber();	execute();
+	}
+	else if(curPos<0)	emit setStatus("No selection.");
+}
+//-----------------------------------------------------------------------------
+void PlotPanel::movePlotDown()
+{
+	if(curPos>=0)
+	{
+		QTextCursor tc = textMGL->textCursor();
+		tc.movePosition(QTextCursor::Start);
+		tc.movePosition(QTextCursor::NextBlock,QTextCursor::MoveAnchor,curPos);
+		tc.select(QTextCursor::BlockUnderCursor);
+		QString s = tc.selectedText();
+		if(curPos==0)	s = "\n"+s;
+		tc.deleteChar();
+		bool ins=true;
+
+		while(tc.movePosition(QTextCursor::NextBlock))
+		{
+			QString q = tc.block().text();
+			if(q.startsWith("subplot ") || q.startsWith("inplot ") || q.startsWith("multiplot ") || q.startsWith("gridplot ") || q.startsWith("columnplot ") || q.startsWith("stickplot "))
+			{
+				tc.movePosition(QTextCursor::EndOfBlock);
+				tc.insertText(s);	ins=false;	break;
+			}
+		}
+		if(ins)	{	tc.movePosition(QTextCursor::End);	tc.insertText(s);	}
+		curPos = tc.block().blockNumber();	execute();
+	}
+	else emit setStatus("No selection.");
+}
+//-----------------------------------------------------------------------------
+
