@@ -38,6 +38,7 @@
 #include "setup_dlg.h"
 #include "text_pnl.h"
 #include "plot_pnl.h"
+#include "subplot_dlg.h"
 //-----------------------------------------------------------------------------
 FilesDialog *files_dlg=0;
 QString defFontFamily;
@@ -55,6 +56,7 @@ TextPanel::TextPanel(QWidget *parent) : QWidget(parent)
 	optDialog = new OptionDialog(this);
 	stlDialog = new StyleDialog(this);
 	newCmdDlg = new NewCmdDialog(this);
+	subplotDlg = new SubplotDialog(this);
 	setupDlg = new SetupDialog(this);
 	dataOpenDlg = createDataOpenDlg(this);
 	if(!files_dlg)	files_dlg= new FilesDialog;
@@ -64,7 +66,8 @@ TextPanel::TextPanel(QWidget *parent) : QWidget(parent)
 	vars = words;
 
 	connect(setupDlg, SIGNAL(putText(const QString &)), this, SLOT(animPutText(const QString &)));
-	connect(newCmdDlg, SIGNAL(result(const QString&)), this, SLOT(putLine(const QString&)));
+	connect(newCmdDlg, SIGNAL(result(const QString&, bool)), this, SLOT(putLine(const QString&, bool)));
+	connect(subplotDlg, SIGNAL(result(const QString&)), this, SLOT(putLine(const QString&)));
 	connect(findDialog, SIGNAL(findText(const QString &, bool, bool)), this, SLOT(findText(const QString &, bool, bool)));
 	connect(findDialog, SIGNAL(replText(const QString &, const QString &, bool, bool)), this, SLOT(replText(const QString &, const QString &, bool, bool)));
 
@@ -79,7 +82,8 @@ TextPanel::TextPanel(QWidget *parent) : QWidget(parent)
 	menu = new QMenu(tr("Edit"),this);
 	v = new QVBoxLayout(this);
 	h = new QHBoxLayout();	v->addLayout(h);
-	toolTop(h);				v->addWidget(edit);
+	toolTop(h);
+	v->addWidget(edit);
 }
 //-----------------------------------------------------------------------------
 TextPanel::~TextPanel()	{	delete printer;	}
@@ -112,6 +116,19 @@ void TextPanel::insNVal()
 	edit->textCursor().insertText(QString::number(res.GetVal(0)));
 }
 //-----------------------------------------------------------------------------
+void TextPanel::insPrim()
+{
+	QString str(graph->mgl->primitives);
+	if(str.isEmpty())
+	{
+		QMessageBox::warning(this,tr("UDAV"),tr("There is manual primitives."));
+		return;
+	}
+	edit->moveCursor(QTextCursor::Start);
+	edit->insertPlainText("subplot 1 1 0 '#'\n"+str+"subplot 1 1 0\n#----------\n");
+	graph->mgl->primitives = "";
+}
+//-----------------------------------------------------------------------------
 void TextPanel::insFitF()
 {
 	QString str(graph->getFit());
@@ -132,7 +149,7 @@ void TextPanel::insFile()
 //-----------------------------------------------------------------------------
 void TextPanel::insPath()
 {
-	QString str = QFileDialog::getExistingDirectory(this, tr("UDAV - Insert filename"));
+	QString str = QFileDialog::getExistingDirectory(this, tr("UDAV - Insert path"));
 	if(str.isEmpty())	return;
 	edit->textCursor().insertText("'"+str+"'");
 }
@@ -140,11 +157,11 @@ void TextPanel::insPath()
 void TextPanel::refreshData()
 {
 	vars=words;
-	mglVar *v = parser.FindVar("");
-	while(v)
+	long i,n=parser.GetNumVar();
+	for(i=0;i<n;i++)
 	{
-		if(v->s.length()>2)	vars<<QString::fromStdWString(v->s);
-		v = v->next;
+		const mglDataA *v=parser.GetVar(i);
+		if(v && v->s.length()>2)	vars<<QString::fromStdWString(v->s);
 	}
 	setCompleter(mglCompleter);
 }
@@ -208,9 +225,20 @@ void TextPanel::animPutText(const QString &s)
 //-----------------------------------------------------------------------------
 //void TextPanel::putText(const QString &txt)	{	edit->insertPlainText(txt);	}
 //-----------------------------------------------------------------------------
-void TextPanel::putLine(const QString &txt)
-{	edit->moveCursor(QTextCursor::StartOfLine);
-	edit->insertPlainText(txt+"\n");	}
+void TextPanel::putLine(const QString &txt, bool replace)
+{
+	edit->moveCursor(QTextCursor::StartOfLine);
+	if(replace)
+	{
+		QTextCursor c = edit->textCursor();
+		c.select(QTextCursor::BlockUnderCursor);
+		c.removeSelectedText();
+		edit->setTextCursor(c);
+		if(c.atStart())	edit->insertPlainText(txt);
+		else	edit->insertPlainText("\n"+txt);
+	}
+	else	edit->insertPlainText(txt+"\n");
+}
 //-----------------------------------------------------------------------------
 void TextPanel::addStyle()
 {
@@ -237,7 +265,6 @@ QString TextPanel::selection()
 //-----------------------------------------------------------------------------
 void TextPanel::setCursorPosition(int n)
 {
-	graph->setCurPos(n);
 	if(n<0)	return;
 	edit->moveCursor(QTextCursor::Start);
 	for(int i=0;i<n;i++)	edit->moveCursor(QTextCursor::NextBlock);
@@ -281,10 +308,10 @@ void TextPanel::loadHDF5(const QString &fileName)
 			H5Dread(hd, H5T_C_S1, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf);
 			buf[dims[0]]=0;		// to be sure :)
 			QString str = buf;
-			if(str.contains("#----- End of QMathGL block -----\n"))
+			if(str.contains("subplot 1 1 0\n#----- End of QMathGL block -----\n"))
 			{
-				graph->mgl->primitives = str.section("#----- End of QMathGL block -----\n",0,0);
-				str = str.section("#----- End of QMathGL block -----\n",1);
+				graph->mgl->primitives = str.section("subplot 1 1 0\n#----- End of QMathGL block -----\n",0,0).section("subplot 1 1 0 '#'\n",1);
+				str = str.section("subplot 1 1 0\n#----- End of QMathGL block -----\n",1);
 			}
 			edit->setText(str);
 			graph->animParseText(edit->toPlainText());
@@ -296,7 +323,7 @@ void TextPanel::loadHDF5(const QString &fileName)
 		else if(H5Tget_class(ht)==H5T_FLOAT || H5Tget_class(ht)==H5T_INTEGER)
 		{
 			for(int j=0;name[j];j++)	if(!isalnum(name[j]))	name[j]='_';
-			mglVar *v = parser.AddVar(name);
+			mglData *v = parser.AddVar(name);
 			nx = ny = nz = 1;
 			if(rank>0 && rank<=3)
 			{
@@ -336,7 +363,7 @@ void TextPanel::saveHDF5(const QString &fileName)
 	{	// save script
 		QString txt;
 		if(!graph->mgl->primitives.isEmpty())
-			txt += graph->mgl->primitives + "#----- End of QMathGL block -----";
+			txt += "subplot 1 1 0 '#'\n"+graph->mgl->primitives + "subplot 1 1 0\n#----- End of QMathGL block -----\n";
 		txt += edit->toPlainText();
 		dims[0] = txt.length()+1;
 		char *buf = new char[dims[0]+1];
@@ -348,10 +375,13 @@ void TextPanel::saveHDF5(const QString &fileName)
 		H5Dclose(hd);	H5Sclose(hs);
 		delete []buf;
 	}
-	mglVar *v = parser.FindVar("");
+	long n = parser.GetNumVar();
 	char name[256];
-	while(v)
+	for(long i=0;i<n;i++)
 	{
+		const mglData *v = dynamic_cast<const mglData *>(parser.GetVar(i));
+		mglData tmp;
+		if(!v)	{	tmp.Set(parser.GetVar(i));	v = &tmp;	}
 		wcstombs(name,v->s.c_str(),v->s.length()+1);
 		if(v->nz==1 && v->ny == 1)
 		{	rank = 1;	dims[0] = v->nx;	}
@@ -364,7 +394,6 @@ void TextPanel::saveHDF5(const QString &fileName)
 
 		H5Dwrite(hd, H5T_NATIVE_FLOAT, hs, hs, H5P_DEFAULT, v->a);
 		H5Dclose(hd);	H5Sclose(hs);
-		v = v->next;
 	}
 	H5Fclose(hf);
 	setCurrentFile(fileName);
@@ -461,13 +490,16 @@ void TextPanel::addSetup()	{	setupDlg->exec();	}
 //-----------------------------------------------------------------------------
 #include "xpm/option.xpm"
 #include "xpm/style.xpm"
+#include "xpm/curve.xpm"
+#include "xpm/box.xpm"
 //-----------------------------------------------------------------------------
 void TextPanel::toolTop(QBoxLayout *l)
 {
-	QAction *a;
+	QAction *a, *aa;
 	QMenu *o=menu, *oo;
 	QToolButton *bb;
 	const MainWindow *mw=findMain(this);
+//	l->setSpacing(3);
 
 	// general buttons
 	if(mw)
@@ -476,13 +508,13 @@ void TextPanel::toolTop(QBoxLayout *l)
 		bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(mw->asave);
 	}
 	// edit menu
-	a = new QAction(QPixmap(":/xpm/edit-undo.png"), tr("Undo"), this);
+	a = new QAction(QPixmap(":/png/edit-undo.png"), tr("Undo"), this);
 	connect(a, SIGNAL(triggered()), edit, SLOT(undo()));
 	a->setToolTip(tr("Undo editor change (Ctrl+Z)."));
 	a->setShortcut(Qt::CTRL+Qt::Key_Z);	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/edit-redo.png"), tr("Redo"), this);
+	a = new QAction(QPixmap(":/png/edit-redo.png"), tr("Redo"), this);
 	connect(a, SIGNAL(triggered()), edit, SLOT(redo()));
 	a->setToolTip(tr("Redo editor change (Ctrl+Shift+Z)."));
 	a->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_Z);	o->addAction(a);
@@ -490,28 +522,28 @@ void TextPanel::toolTop(QBoxLayout *l)
 
 	o->addSeparator();
 	o->addAction(tr("Clear all"), edit, SLOT(clear()));
-	a = new QAction(QPixmap(":/xpm/edit-cut.png"), tr("Cut text"), this);
+	a = new QAction(QPixmap(":/png/edit-cut.png"), tr("Cut text"), this);
 	connect(a, SIGNAL(triggered()), edit, SLOT(cut()));
 	a->setToolTip(tr("Cut selected text to clipboard (Ctrl+X)."));
 	a->setShortcut(Qt::CTRL+Qt::Key_X);	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/edit-copy.png"), tr("Copy text"), this);
+	a = new QAction(QPixmap(":/png/edit-copy.png"), tr("Copy text"), this);
 	connect(a, SIGNAL(triggered()), edit, SLOT(copy()));
 	a->setToolTip(tr("Copy selected text or data to clipboard (Ctrl+C)."));
 	a->setShortcut(Qt::CTRL+Qt::Key_C);	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	a = new QAction(QPixmap(":/xpm/edit-paste.png"), tr("Paste text"), this);
+	a = new QAction(QPixmap(":/png/edit-paste.png"), tr("Paste text"), this);
 	connect(a, SIGNAL(triggered()), edit, SLOT(paste()));
 	a->setToolTip(tr("Paste text or data from clipboard (Ctrl+V)."));
 	a->setShortcut(Qt::CTRL+Qt::Key_V);	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
-	o->addAction(QPixmap(":/xpm/edit-select-all.png"), tr("Select all"), edit, SLOT(selectAll()), Qt::CTRL+Qt::Key_A);
+	o->addAction(QPixmap(":/png/edit-select-all.png"), tr("Select all"), edit, SLOT(selectAll()), Qt::CTRL+Qt::Key_A);
 	o->addSeparator();
 
-	a = new QAction(QPixmap(":/xpm/edit-find.png"), tr("Find/Replace"), this);
+	a = new QAction(QPixmap(":/png/edit-find.png"), tr("Find/Replace"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(find()));
 	a->setToolTip(tr("Show dialog for text finding (Ctrl+F)."));
 	a->setShortcut(Qt::CTRL+Qt::Key_F);	o->addAction(a);
@@ -524,11 +556,14 @@ void TextPanel::toolTop(QBoxLayout *l)
 
 	// insert menu
 	oo = o->addMenu(tr("Insert"));
-	a = new QAction(QPixmap(":/xpm/format-indent-more.png"), tr("New command"), this);
+	aa=a = new QAction(QPixmap(":/png/format-indent-more.png"), tr("New command"), this);
 	a->setShortcut(Qt::META+Qt::Key_C);	connect(a, SIGNAL(triggered()), this, SLOT(newCmd()));
-	a->setToolTip(tr("Show dialog for new command and put it into the script."));
+	a->setToolTip(tr("Show dialog for new command or edit arguments of existed one."));
+	oo->addAction(a);
+	a = new QAction(QPixmap(box_xpm), tr("New inplot"), this);
+	a->setShortcut(Qt::META+Qt::Key_C);	connect(a, SIGNAL(triggered()), subplotDlg, SLOT(show()));
+	a->setToolTip(tr("Show dialog for new inplot and put it into the script."));
 	oo->addAction(a);
-	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
 	a = new QAction(tr("Fitted formula"), this);
 	a->setShortcut(Qt::META+Qt::Key_F);	connect(a, SIGNAL(triggered()), this, SLOT(insFitF()));
@@ -546,22 +581,30 @@ void TextPanel::toolTop(QBoxLayout *l)
 	a->setShortcut(Qt::META+Qt::Key_N);	connect(a, SIGNAL(triggered()), this, SLOT(insNVal()));
 	a->setToolTip(tr("Replace expression by its numerical value."));
 	oo->addAction(a);
-	a = new QAction(QPixmap(":/xpm/x-office-spreadsheet.png"), tr("File name"), this);
+	a = new QAction(QPixmap(":/png/text-csv.png"), tr("File name"), this);
 	a->setShortcut(Qt::META+Qt::Key_P);	connect(a, SIGNAL(triggered()), this, SLOT(insFile()));
 	a->setToolTip(tr("Select and insert file name."));
 	oo->addAction(a);
-	a = new QAction(QPixmap(":/xpm/folder.png"), tr("Folder path"), this);
+	a = new QAction(QPixmap(":/png/folder.png"), tr("Folder path"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(insPath()));
 	a->setToolTip(tr("Select and insert folder name."));
 	oo->addAction(a);
+	a = new QAction(QPixmap(curve_xpm), tr("Manual primitives"), this);
+	a->setShortcut(Qt::META+Qt::Key_P);	connect(a, SIGNAL(triggered()), this, SLOT(insPrim()));
+	a->setToolTip(tr("Move mouse-handled primitives to script."));
+	oo->addAction(a);
 
-	a = new QAction(QPixmap(":/xpm/document-properties.png"), tr("Graphics setup"), this);
+	bb = new QToolButton(this);	l->addWidget(bb);
+	bb->setDefaultAction(aa);	bb->setMenu(oo);
+	bb->setPopupMode(QToolButton::MenuButtonPopup);
+
+	a = new QAction(QPixmap(":/png/document-properties.png"), tr("Graphics setup"), this);
 	a->setShortcut(Qt::META+Qt::Key_G);	connect(a, SIGNAL(triggered()), this, SLOT(addSetup()));
 	a->setToolTip(tr("Show dialog for plot setup and put code into the script.\nThis dialog setup axis, labels, lighting and other general things."));
-	oo->addAction(a);
+	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
 	l->addStretch(1);
-	if(mw)	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(mw->acalc);
+	if(mw)	{	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(mw->acalc);	}
 }
 //-----------------------------------------------------------------------------
