@@ -23,8 +23,6 @@
 #else
 #include <unistd.h>
 #endif
-#include <QUrl>
-#include <QFile>
 #include <QMenuBar>
 #include <QMessageBox>
 #include <QApplication>
@@ -32,13 +30,12 @@
 #include <QSplitter>
 #include <QFileDialog>
 #include <QStatusBar>
-#include <QTextStream>
 #include <QDockWidget>
 #include <QCloseEvent>
 #include <QTextCodec>
 #include <QTranslator>
-#include <QVariant>
 #include <QMimeData>
+#include <QUrl>
 //-----------------------------------------------------------------------------
 #if !defined(WIN32) && !defined(__APPLE__)
 #include <X11/Xlib.h>
@@ -59,6 +56,7 @@ int MaxRecentFiles=5;
 bool editPosBottom = false;
 bool mglAutoSave = false;
 bool mglHighlight = true;
+bool mglDotsRefr = true;
 bool mglAutoPure = true;
 bool mglCompleter = true;
 bool loadInNewWnd = false;
@@ -96,11 +94,17 @@ void mgl_ask_qt(const wchar_t *quest, wchar_t *res);
 //-----------------------------------------------------------------------------
 int main(int argc, char **argv)
 {
+	mgl_suppress_warn(true);
 //	QCoreApplication::setAttribute(Qt::AA_X11InitThreads);
 #if !defined(WIN32) && !defined(__APPLE__)
 	// try to fix possible multi-threading errors
 	// must be placed before ANY window creation
 	XInitThreads();
+#endif
+#ifdef WIN32
+	char buf[512];	getcwd(buf,500);	strcat(buf,"\\plugins\\");
+	QCoreApplication::addLibraryPath(buf);
+	QCoreApplication::addLibraryPath("c:\\plugins\\");
 #endif
 	mgl_ask_func = mgl_ask_qt;
 	QApplication a(argc, argv);
@@ -146,26 +150,6 @@ int main(int argc, char **argv)
 //		mgl addon
 //
 //-----------------------------------------------------------------------------
-/*mglCommand udav_base_cmd[] = {
-	{L"fplot",L"Plot curve by formula",L"fplot 'func' ['stl'='' num=100]", mgls_fplot, mglc_fplot},
-	{L"fsurf",L"Plot surface by formula",L"fsurf 'func' ['stl'='' numx=100 numy=100]", mgls_fsurf, mglc_fsurf},
-	{L"fgets",L"Print string from file",L"fgets x y {z} 'fname' [pos=0 'stl'='' size=-1.4]", mgls_fgets, mglc_fgets},
-{L"",0,0,0,0}};
-//-----------------------------------------------------------------------------
-void udavAddCommands(const mglCommand *cmd)	// NOTE it work but I don't how I can use it
-{
-	int i, mp, mc;
-	// determine the number of symbols
-	for(i=0;parser.Cmd[i].name[0];i++){};	mp = i;
-	for(i=0;cmd[i].name[0];i++){};			mc = i;
-	mglCommand *buf = new mglCommand[mp+mc+1];
-	memcpy(buf, parser.Cmd, mp*sizeof(mglCommand));
-	memcpy(buf+mp, cmd, (mc+1)*sizeof(mglCommand));
-	qsort(buf, mp+mc, sizeof(mglCommand), mgl_cmd_cmp);
-	if(parser.Cmd!=mgls_base_cmd)	delete []parser.Cmd;
-	parser.Cmd = buf;
-}*/
-//-----------------------------------------------------------------------------
 void udavLoadDefCommands()	{}	//{	udavAddCommands(udav_base_cmd);	}
 //-----------------------------------------------------------------------------
 //
@@ -190,22 +174,30 @@ MainWindow::MainWindow(QWidget *wp) : QMainWindow(wp)
 	messWnd->setAllowedAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
 	addDockWidget(Qt::BottomDockWidgetArea, messWnd);
 	messWnd->resize(size().width(), 0);	new MessSyntax(mess);
-	connect(mess,SIGNAL(cursorPositionChanged()),this,SLOT(messClicked()));
+//	connect(mess,SIGNAL(cursorPositionChanged()),this,SLOT(messClicked()));
 	connect(mess,SIGNAL(selectionChanged()),this,SLOT(messClicked()));
 
+	hideWnd = new QDockWidget(tr("Hidden plots"),this);
+	hidden = new TextEdit(this);	hideWnd->setWidget(hidden);
+	hideWnd->setAllowedAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
+	addDockWidget(Qt::BottomDockWidgetArea, hideWnd);
+	hideWnd->resize(size().width(), 0);	hidden->setReadOnly(true);
+	connect(hidden,SIGNAL(selectionChanged()),this,SLOT(hiddenClicked()));	// TODO
+//	connect(hidden,SIGNAL(cursorPositionChanged()),this,SLOT(hiddenClicked()));
+
 	calcWnd = new QDockWidget(tr("Calculator"),this);
 
-	aload = a = new QAction(QPixmap(":/xpm/document-open.png"), tr("Open file"), this);
+	aload = a = new QAction(QPixmap(":/png/document-open.png"), tr("Open file"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(choose()));
 	a->setToolTip(tr("Open and execute/show script or data from file (Ctrl+O).\nYou may switch off automatic exection in UDAV properties."));
 	a->setShortcut(Qt::CTRL+Qt::Key_O);
 
-	asave = a = new QAction(QPixmap(":/xpm/document-save.png"), tr("Save script"), this);
+	asave = a = new QAction(QPixmap(":/png/document-save.png"), tr("Save script"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(save()));
 	a->setToolTip(tr("Save script to a file (Ctrl+S)"));
 	a->setShortcut(Qt::CTRL+Qt::Key_S);
 
-	acalc = a = new QAction(QPixmap(":/xpm/accessories-calculator.png"), tr("Calculator"), this);
+	acalc = a = new QAction(QPixmap(":/png/accessories-calculator.png"), tr("Calculator"), this);
 	a->setShortcut(Qt::Key_F4);	a->setCheckable(true);
 	connect(a, SIGNAL(toggled(bool)), calcWnd, SLOT(setVisible(bool)));
 	connect(calcWnd, SIGNAL(visibilityChanged(bool)), a, SLOT(setChecked(bool)));
@@ -218,18 +210,24 @@ MainWindow::MainWindow(QWidget *wp) : QMainWindow(wp)
 	connect(messWnd, SIGNAL(visibilityChanged(bool)), a, SLOT(setChecked(bool)));
 	a->setChecked(false);	messWnd->setVisible(false);
 
+	ahide = a = new QAction(QPixmap(":/png/layer-visible-on.png"), tr("Show hidden plots"), this);
+	a->setShortcut(Qt::Key_F8);	a->setCheckable(true);
+	connect(a, SIGNAL(toggled(bool)), hideWnd, SLOT(setVisible(bool)));
+	connect(hideWnd, SIGNAL(visibilityChanged(bool)), a, SLOT(setChecked(bool)));
+	a->setChecked(false);	hideWnd->setVisible(false);
+
 	graph = new PlotPanel(this);
-	rtab->addTab(graph,QPixmap(":/xpm/x-office-presentation.png"),tr("Canvas"));
+	rtab->addTab(graph,QPixmap(":/png/office-chart-line.png"),tr("Canvas"));
 	//	connect(info,SIGNAL(addPanel(QWidget*)),this,SLOT(addPanel(QWidget*)));
 	info = createMemPanel(this);
-	rtab->addTab(info,QPixmap(":/xpm/system-file-manager.png"),tr("Info"));
+	rtab->addTab(info,QPixmap(":/png/system-file-manager.png"),tr("Info"));
 	hlp = createHlpPanel(this);
-	rtab->addTab(hlp,QPixmap(":/xpm/help-contents.png"),tr("Help"));
+	rtab->addTab(hlp,QPixmap(":/png/help-contents.png"),tr("Help"));
 	edit = new TextPanel(this);	edit->graph = graph;
 	graph->textMGL = edit->edit;
 	connect(graph->mgl,SIGNAL(showWarn(QString)),mess,SLOT(setText(QString)));
 	connect(graph,SIGNAL(clearWarn()),mess,SLOT(clear()));
-	ltab->addTab(edit,QPixmap(":/xpm/text-x-generic.png"),tr("Script"));
+	ltab->addTab(edit,QPixmap(":/png/text-plain.png"),tr("Script"));
 
 	calcWnd->setWidget(createCalcDlg(this, edit->edit));
 	calcWnd->setAllowedAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
@@ -252,10 +250,9 @@ MainWindow::MainWindow(QWidget *wp) : QMainWindow(wp)
 	connect(graph->mgl, SIGNAL(refreshData()), edit, SLOT(refreshData()));
 	connect(graph->mgl,SIGNAL(doubleClick(int)),edit,SLOT(newCmd(int)));
 
+	connect(edit->edit,SIGNAL(textChanged()),this,SLOT(updateHidden()));
 	connect(mess, SIGNAL(textChanged()), this, SLOT(warnChanged()));
-//	connect(mdi, SIGNAL(subWindowActivated(QMdiSubWindow *)), this, SLOT(subActivated(QMdiSubWindow *)));
 	connect(propDlg, SIGNAL(sizeChanged(int,int)), graph->mgl, SLOT(imgSize(int,int)));
-//	connect(propDlg, SIGNAL(propUpdated()), edit->edit, SLOT(update()));	// TODO: update qmglsyntax
 	connect(edit->edit,SIGNAL(textChanged()), this, SLOT(setAsterix()));
 	connect(edit->edit, SIGNAL(cursorPositionChanged()), this, SLOT(editPosChanged()));
 	connect(edit,SIGNAL(setCurrentFile(QString)),this,SLOT(setCurrentFile(QString)));
@@ -275,7 +272,7 @@ void MainWindow::makeMenu()
 	// file menu
 	{
 	o = menuBar()->addMenu(tr("File"));
-	a = new QAction(QPixmap(":/xpm/document-new.png"), tr("New script"), this);
+	a = new QAction(QPixmap(":/png/document-new.png"), tr("New script"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(newDoc()));
 	a->setToolTip(tr("Create new empty script window (Ctrl+N)."));
 	a->setShortcut(Qt::CTRL+Qt::Key_N);	o->addAction(a);
@@ -289,7 +286,7 @@ void MainWindow::makeMenu()
 
 	o->addSeparator();
 	o->addAction(tr("Print script"), edit, SLOT(printText()));
-	a = new QAction(QPixmap(":/xpm/document-print.png"), tr("Print graphics"), this);
+	a = new QAction(QPixmap(":/png/document-print.png"), tr("Print graphics"), this);
 	connect(a, SIGNAL(triggered()), graph->mgl, SLOT(print()));
 	a->setToolTip(tr("Open printer dialog and print graphics (Ctrl+P)"));
 	a->setShortcut(Qt::CTRL+Qt::Key_P);	o->addAction(a);
@@ -305,22 +302,23 @@ void MainWindow::makeMenu()
 	// settings menu
 	{
 	o = menuBar()->addMenu(tr("Settings"));
-	a = new QAction(QPixmap(":/xpm/preferences-system.png"), tr("Properties"), this);
+	a = new QAction(QPixmap(":/png/preferences-system.png"), tr("Properties"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(properties()));
 	a->setToolTip(tr("Show dialog for UDAV properties."));	o->addAction(a);
 	o->addAction(tr("Set arguments"), createArgsDlg(this), SLOT(exec()));
 
 	o->addAction(acalc);
 	o->addAction(ainfo);
+	o->addAction(ahide);
 	}
 
 	menuBar()->addSeparator();
 	o = menuBar()->addMenu(tr("Help"));
-	a = new QAction(QPixmap(":/xpm/help-contents.png"), tr("MGL help"), this);
+	a = new QAction(QPixmap(":/png/help-contents.png"), tr("MGL help"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(showHelp()));
 	a->setToolTip(tr("Show help on MGL commands (F1)."));
 	a->setShortcut(Qt::Key_F1);	o->addAction(a);
-	a = new QAction(QPixmap(":/xpm/help-faq.png"), tr("Hints"), this);
+	a = new QAction(QPixmap(":/png/help-faq.png"), tr("Hints"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(showHint()));
 	a->setToolTip(tr("Show hints of MGL usage."));	o->addAction(a);
 	o->addAction(tr("About"), this, SLOT(about()));
@@ -443,37 +441,10 @@ void MainWindow::setEditPos(bool bottom)
 //-----------------------------------------------------------------------------
 void MainWindow::properties()	{	propDlg->exec();	}
 //-----------------------------------------------------------------------------
-void MainWindow::messClicked()
-{
-	QString m = mess->toPlainText(), q;
-	int p = mess->textCursor().blockNumber();
-	for(;p>=0;p--)
-	{
-		q = m.section('\n',p,p);
-		if(q.contains("in line "))
-		{
-			QString s = q.section(' ',-1);
-			int n = s.toInt()-1;	if(n<0)	return;
-			edit->moveCursor(QTextCursor::Start);
-			for(int i=0;i<n;i++)	edit->moveCursor(QTextCursor::NextBlock);
-			break;
-		}
-	}
-	edit->setFocus();
-}
-//-----------------------------------------------------------------------------
-void MainWindow::warnChanged()
-{
-	if(mess->toPlainText().isEmpty())
-	{	messWnd->hide();	ainfo->setChecked(false);	}
-	else
-	{	messWnd->show();	ainfo->setChecked(true);	}
-}
-//-----------------------------------------------------------------------------
 void MainWindow::about()
 {
-	QString s = tr("UDAV v. 2.")+QString::number(MGL_VER2)+
-				tr("\n(c) Alexey Balakin, 2007-2014\nhttp://mathgl.sf.net/");
+	QString s = tr("<a href='http://mathgl.sourceforge.net/doc_en/UDAV-overview.html'>UDAV</a> v. 2.")+QString::number(MGL_VER2)+
+	tr("<br>(c) Alexey Balakin, 2007-2014<br><br><a href='http://www.gnu.org/copyleft/gpl.html'>License is GPL v.2 or later.</a>");
 	QMessageBox::about(this, tr("UDAV - about"), s);
 }
 //-----------------------------------------------------------------------------
@@ -508,6 +479,7 @@ void MainWindow::writeSettings()
 	settings.setValue("/autoExec",  mglAutoExecute);
 	settings.setValue("/autoSave",  mglAutoSave);
 	settings.setValue("/highlight",  mglHighlight);
+	settings.setValue("/dotsRefresh", mglDotsRefr);
 	settings.setValue("/autoPure",  mglAutoPure);
 	settings.setValue("/editAtTop", editPosBottom);
 	settings.setValue("/fontFamily", defFontFamily);
@@ -550,6 +522,7 @@ void MainWindow::readSettings()
 	editPosBottom = settings.value("/editAtTop", false).toBool();
 	mglCompleter = settings.value("/completer",  true).toBool();
 	loadInNewWnd = settings.value("/loadInNewWnd", false).toBool();
+	mglDotsRefr = settings.value("/dotsRefresh", true).toBool();
 	defFontFamily = settings.value("/fontFamily", "Georgia").toString();
 	defFontSize = settings.value("/fontSize", 10).toInt();
 	edit->setEditorFont();	setEditPos(editPosBottom);
@@ -720,19 +693,19 @@ void updateDataItems()
 //-----------------------------------------------------------------------------
 void MainWindow::addPanel(QWidget *w, QString name)
 {
-	ltab->addTab(w,QPixmap(":/xpm/x-office-spreadsheet.png"),name);
+	ltab->addTab(w,QPixmap(":/png/text-csv.png"),name);
 	ltab->setCurrentWidget(w);
 }
 //-----------------------------------------------------------------------------
-MainWindow *findMain(QWidget *wnd)
+MGL_LOCAL_PURE MainWindow *findMain(QWidget *wnd)
 {
 	MainWindow *mw=0;
 	QObject *w=wnd;
 
-	while(w)
+	while(w && !mw)
 	{
 		mw = dynamic_cast<MainWindow *>(w);
-		if(mw)	break;	else w = w->parent();
+		w = w->parent();
 	}
 	return mw;
 }
@@ -743,3 +716,53 @@ void raisePanel(QWidget *w)
 	if(mw)	mw->rtab->setCurrentWidget(w);
 }
 //-----------------------------------------------------------------------------
+void MainWindow::updateHidden()
+{
+	QTextCursor tc = edit->edit->textCursor();
+	long pos = tc.position(), i=0;
+	hidden->clear();
+	tc.movePosition(QTextCursor::Start);
+	do {
+		i++;
+		if(tc.block().text().startsWith("#h "))
+			hidden->append("Line "+QString::number(i)+QString::fromWCharArray(L" → ")+tc.block().text().mid(3)+"\n");
+	} while(tc.movePosition(QTextCursor::NextBlock));
+	tc.setPosition(pos);
+}
+//-----------------------------------------------------------------------------
+void MainWindow::hiddenClicked()
+{
+	QString q = hidden->textCursor().block().text();
+	if(q.contains("Line "))
+	{
+		int n = q.section(' ',1,1).toInt()-1;
+		edit->edit->moveCursor(QTextCursor::Start);
+		for(int i=0;i<n;i++)	edit->edit->moveCursor(QTextCursor::NextBlock);
+		edit->edit->textCursor().deleteChar();
+		edit->edit->textCursor().deleteChar();
+		edit->edit->textCursor().deleteChar();
+	}
+	graph->execute();
+}
+//-----------------------------------------------------------------------------
+void MainWindow::messClicked()
+{
+	QString q = mess->textCursor().block().text();
+	if(q.contains("in line "))
+	{
+		QString s = q.section(' ',-1);
+		int n = q.section(' ',-1).toInt()-1;	if(n<0)	return;
+		edit->moveCursor(QTextCursor::Start);
+		for(int i=0;i<n;i++)	edit->moveCursor(QTextCursor::NextBlock);
+	}
+	edit->setFocus();
+}
+//-----------------------------------------------------------------------------
+void MainWindow::warnChanged()
+{
+	if(mess->toPlainText().isEmpty())
+	{	messWnd->hide();	ainfo->setChecked(false);	}
+	else
+	{	messWnd->show();	ainfo->setChecked(true);	}
+}
+//-----------------------------------------------------------------------------
