@@ -25,21 +25,35 @@ void mgl_ask_gets(const wchar_t *quest, wchar_t *res);
 //-----------------------------------------------------------------------------
 int main(int argc, char *argv[])
 {
+	mgl_suppress_warn(true);
 	mglGraph gr;
 	mglParse p(true);
 	char ch, buf[2048], iname[256]="", oname[256]="";
 	std::vector<std::wstring> var;
+	std::wstring str;
+	bool none=false;
 
-	register size_t i, n;
 	while(1)
 	{
-		ch = getopt(argc, argv, "1:2:3:4:5:6:7:8:9:ho:L:C:A:");
+		ch = getopt(argc, argv, "1:2:3:4:5:6:7:8:9:hno:L:C:A:s:");
 		if(ch>='1' && ch<='9')	p.AddParam(ch-'0', optarg);
+		else if(ch=='s')
+		{
+			setlocale(LC_CTYPE, "");
+			FILE *fp = fopen(optarg,"r");
+			if(fp)
+			{
+				wchar_t ch;
+				while((ch=fgetwc(fp))!=WEOF)	str.push_back(ch);
+				fclose(fp);	str += L"\n";
+			}
+		}
+		else if(ch=='n')	none = true;
 		else if(ch=='L')	setlocale(LC_CTYPE, optarg);
 		else if(ch=='A')
 		{
 			std::wstring str;
-			for(i=0;optarg[i];i++)	str.push_back(optarg[i]);
+			for(long i=0;optarg[i];i++)	str.push_back(optarg[i]);
 			var.push_back(str);
 		}
 		else if(ch=='C')
@@ -63,8 +77,9 @@ int main(int argc, char *argv[])
 				"\t...          ...\n"
 				"\t-9 str       set str as argument $9 for script\n"
 				"\t-L loc       set locale to loc\n"
+				"\t-s opt       set MGL script for setting up the plot\n"
 				"\t-o name      set output file name\n"
-				"\t-            get script from standard input\n"
+				"\t-n           no manual output (script should save results by itself)\n"
 				"\t-A val       add animation value val\n"
 				"\t-C n1:n2:dn  add animation value in range [n1,n2] with step dn\n"
 				"\t-C n1:n2     add animation value in range [n1,n2] with step 1\n"
@@ -78,18 +93,20 @@ int main(int argc, char *argv[])
 	}
 	if(ch=='h')	return 0;
 	if(*oname==0)	{	strncpy(oname,*iname?iname:"out",250);	strcat(oname,".png");	}
-	
+	else	none = false;
+
 	mgl_ask_func = mgl_ask_gets;
 	// prepare for animation
-	std::wstring str;
 	setlocale(LC_CTYPE, "");
 	FILE *fp = *iname?fopen(iname,"r"):stdin;
+	if(!fp)	{	printf("No file for MGL script\n");	return 0;	}
 	wchar_t cw;
 	while((cw=fgetwc(fp))!=WEOF)	str.push_back(cw);
 //	while(!feof(fp))	str.push_back(fgetwc(fp));
 	if(*iname)	fclose(fp);
 
-	for(i=0;;)	// collect exact values
+	unsigned long n;
+	for(long i=0;;)	// collect exact values
 	{
 		n = str.find(L"##a ",i);
 		if(n==std::string::npos)	break;
@@ -108,7 +125,7 @@ int main(int argc, char *argv[])
 	if(var.size()>1)	// there is animation
 	{
 		if(gif)	gr.StartGIF(oname);
-		for(i=0;i<var.size();i++)
+		for(unsigned long i=0;i<var.size();i++)
 		{
 			gr.NewFrame();
 			printf("frame %ld for $0 = \"%ls\"\n",i,var[i].c_str());
@@ -116,7 +133,7 @@ int main(int argc, char *argv[])
 			p.Execute(&gr,str.c_str());
 			if(gr.Message()[0])	printf("%s\n",gr.Message());
 			gr.EndFrame();
-			snprintf(buf,2048,"%s-%ld",oname,i);
+			snprintf(buf,2048,"%s-%ld",oname,i);	buf[2047]=0;
 			if(!gif)	gr.WriteFrame(buf);
 		}
 		if(gif)	gr.CloseGIF();
@@ -125,10 +142,10 @@ int main(int argc, char *argv[])
 	{
 		p.Execute(&gr,str.c_str());
 		printf("%s\n",gr.Message());
-		gr.WriteFrame(oname);
+		if(!none)	gr.WriteFrame(oname);
 	}
 	if(!mglGlobalMess.empty())	printf("%s",mglGlobalMess.c_str());
-	printf("Write output to %s\n",oname);
+	if(!none || gif)	printf("Write output to %s\n",oname);
 	return 0;
 }
 //-----------------------------------------------------------------------------
