@@ -26,21 +26,16 @@
 //-----------------------------------------------------------------------------
 #ifdef USE_GETTEXT
 #include <libintl.h>
-#else
-// Workaround for gcc 4.2
-#ifndef _LIBINTL_H
-#define gettext(x)	(x)
 #endif
+#ifndef _LIBINTL_H	// Workaround for gcc 4.2
+#define mgl_gettext(x)	(x)
+#else
+#define mgl_gettext(x)	gettext(x)
 #endif
 //-----------------------------------------------------------------------------
 #include "mgl2/canvas_wnd.h"
 #include "mgl2/fltk.h"
 //-----------------------------------------------------------------------------
-#include "xpm/alpha_on.xpm"
-#include "xpm/light_on.xpm"
-#include "xpm/zoom_on.xpm"
-#include "xpm/show_on.xpm"
-#include "xpm/rotate_on.xpm"
 #include "xpm/show_sl.xpm"
 #include "xpm/next_sl.xpm"
 #include "xpm/prev_sl.xpm"
@@ -57,12 +52,13 @@
 #include "xpm/rotate.xpm"
 #include "xpm/ok.xpm"
 #include "xpm/wire.xpm"
+#include "xpm/stop.xpm"
 //-----------------------------------------------------------------------------
-Fl_Pixmap xpm_a1(alpha_xpm), xpm_a2(alpha_on_xpm);
-Fl_Pixmap xpm_l1(light_on_xpm), xpm_l2(light_xpm);
-Fl_Pixmap xpm_z1(zoom_in_xpm), xpm_z2(zoom_on_xpm);
-Fl_Pixmap xpm_s1(show_sl_xpm), xpm_s2(show_on_xpm);
-Fl_Pixmap xpm_r1(rotate_xpm), xpm_r2(rotate_on_xpm);
+Fl_Pixmap xpm_a1(alpha_xpm);
+Fl_Pixmap xpm_l1(light_xpm);
+Fl_Pixmap xpm_z1(zoom_in_xpm);
+Fl_Pixmap xpm_s1(show_sl_xpm);
+Fl_Pixmap xpm_r1(rotate_xpm);
 Fl_Pixmap xpm_wire(wire_xpm);
 //-----------------------------------------------------------------------------
 /// Class allows the window creation for displaying plot bitmap with the help of FLTK library
@@ -75,7 +71,7 @@ using mglCanvasWnd::Window;
 	Fl_MGLView *mgl;	///< Pointer to MGL widget with buttons
 
 	mglCanvasFL();
-	~mglCanvasFL();
+	virtual ~mglCanvasFL();
 
 	/// Create a window for plotting. Now implemeted only for GLUT.
 	void Window(int argc, char **argv, int (*draw)(mglBase *gr, void *p), const char *title,
@@ -123,12 +119,17 @@ Fl_MathGL::Fl_MathGL(int xx, int yy, int ww, int hh, const char *lbl) : Fl_Widge
 //-----------------------------------------------------------------------------
 Fl_MathGL::~Fl_MathGL()	{	if(mgl_use_graph(gr,-1)<1)	mgl_delete_graph(gr);	}
 //-----------------------------------------------------------------------------
+void Fl_MathGL::stop(bool stop)	{	gr->AskStop(stop);	}
+//-----------------------------------------------------------------------------
+void mgl_fltk_event_func(void *)	{	Fl::awake();	}
+//-----------------------------------------------------------------------------
 void Fl_MathGL::set_graph(HMGL GR)
 {
 	mglCanvas *gg = dynamic_cast<mglCanvas *>(GR);
 	if(!gg)	return;
 	if(mgl_use_graph(gr,-1)<1)	mgl_delete_graph(gr);
 	gr=gg;	mgl_use_graph(gg,1);
+	gr->SetEventFunc(mgl_fltk_event_func, NULL);
 }
 //-----------------------------------------------------------------------------
 void Fl_MathGL::draw()
@@ -161,7 +162,7 @@ void Fl_MathGL::update()
 		mgl_set_alpha(gr,flag&1);	mgl_set_light(gr,flag&2);
 		if(tet_val)	tet = tet_val->value();
 		if(phi_val)	phi = phi_val->value();
-		mgl_zoom(gr,x1,y1,x2,y2);	mgl_view(gr,phi,0,tet);
+		mgl_zoom(gr,x1,y1,x2,y2);	mgl_view(gr,-phi,-tet,0);
 		setlocale(LC_NUMERIC, "C");
 		// use frames for quickly redrawing while adding/changing primitives
 		if(mgl_is_frames(gr))	mgl_new_frame(gr);
@@ -174,7 +175,7 @@ void Fl_MathGL::update()
 	}
 	if(mgl_get_width(gr)!=w() || mgl_get_height(gr)!=h())
 		size(mgl_get_width(gr), mgl_get_height(gr));
-	redraw();	Fl::flush();
+	gr->AskStop(false);	redraw();	Fl::flush();
 }
 //-----------------------------------------------------------------------------
 void Fl_MathGL::resize(int xx, int yy, int ww, int hh)
@@ -196,7 +197,7 @@ int Fl_MathGL::handle(int code)
 			mglPoint p = gr->CalcXYZ(Fl::event_x()-x(), Fl::event_y()-y());
 			if(g)	g->LastMousePos = p;
 			char s[128];
-			snprintf(s,128,"x=%g, y=%g, z=%g",p.x,p.y,p.z);
+			snprintf(s,128,"x=%g, y=%g, z=%g",p.x,p.y,p.z);	s[127]=0;
 			draw();	fl_color(FL_BLACK);		fl_draw(s,40,70);
 		}
 	}
@@ -313,7 +314,7 @@ void Fl_MGLView::toggle(int &val, Fl_Button *b, const char *txt)
 	val = 1-val;	b->value(val);
 	if(menu && txt && *txt)
 	{
-		Fl_Menu_Item *m = (Fl_Menu_Item *)menu->find_item(gettext(txt));
+		Fl_Menu_Item *m = (Fl_Menu_Item *)menu->find_item(mgl_gettext(txt));
 		if(m && val)	m->set();
 		if(m && !val)	m->clear();
 	}
@@ -325,7 +326,7 @@ void Fl_MGLView::setoff(int &val, Fl_Button *b, const char *txt)
 	val = 0;	b->value(val);
 	if(menu && txt && *txt)
 	{
-		Fl_Menu_Item *m = (Fl_Menu_Item *)menu->find_item(gettext(txt));
+		Fl_Menu_Item *m = (Fl_Menu_Item *)menu->find_item(mgl_gettext(txt));
 		if(m && val)	m->set();
 		if(m && !val)	m->clear();
 	}
@@ -335,11 +336,11 @@ void Fl_MGLView::setoff(int &val, Fl_Button *b, const char *txt)
 void MGL_NO_EXPORT mgl_grid_cb(Fl_Widget*, void* v)
 {	if(v)	((Fl_MGLView*)v)->toggle_grid();	}
 //-------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_alpha_cb(Fl_Widget*, void* v)	// alpha?xpm_a2:xpm_a1
+void MGL_NO_EXPORT mgl_alpha_cb(Fl_Widget*, void* v)
 {	if(v)	((Fl_MGLView*)v)->toggle_alpha();	}
 void mglCanvasFL::ToggleAlpha()	{	Fl::lock();	mgl->toggle_alpha();	Fl::unlock();	}
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_light_cb(Fl_Widget*, void* v)	// light?xpm_l2:xpm_l1
+void MGL_NO_EXPORT mgl_light_cb(Fl_Widget*, void* v)
 {	if(v)	((Fl_MGLView*)v)->toggle_light();	}
 void mglCanvasFL::ToggleLight()	{	Fl::lock();	mgl->toggle_light();	Fl::unlock();	}
 //-----------------------------------------------------------------------------
@@ -379,77 +380,77 @@ void mglCanvasFL::Update()		{	Fl::lock();	mgl->update();	Fl::unlock();	}
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_export_png_cb(Fl_Widget*, void* v)
 {
-	char *fname = fl_file_chooser(gettext("Save File As?"), "*.png", 0);
+	char *fname = fl_file_chooser(mgl_gettext("Save File As?"), "*.png", 0);
 	if(!fname || !fname[0])	return;
 	mgl_write_png(((Fl_MGLView*)v)->FMGL->get_graph(),fname,0);
 }
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_export_bps_cb(Fl_Widget*, void* v)
 {
-	char *fname = fl_file_chooser(gettext("Save File As?"), "*.eps", 0);
+	char *fname = fl_file_chooser(mgl_gettext("Save File As?"), "*.eps", 0);
 	if(!fname || !fname[0])	return;
 	mgl_write_bps(((Fl_MGLView*)v)->FMGL->get_graph(),fname,0);
 }
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_export_pngn_cb(Fl_Widget*, void* v)
 {
-	char *fname = fl_file_chooser(gettext("Save File As?"), "*.png", 0);
+	char *fname = fl_file_chooser(mgl_gettext("Save File As?"), "*.png", 0);
 	if(!fname || !fname[0])	return;
 	mgl_write_png_solid(((Fl_MGLView*)v)->FMGL->get_graph(),fname,0);
 }
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_export_jpeg_cb(Fl_Widget*, void* v)
 {
-	char *fname = fl_file_chooser(gettext("Save File As?"), "*.jpg", 0);
+	char *fname = fl_file_chooser(mgl_gettext("Save File As?"), "*.jpg", 0);
 	if(!fname || !fname[0])	return;
 	mgl_write_jpg(((Fl_MGLView*)v)->FMGL->get_graph(),fname,0);
 }
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_export_svg_cb(Fl_Widget*, void* v)
 {
-	char *fname = fl_file_chooser(gettext("Save File As?"), "*.svg", 0);
+	char *fname = fl_file_chooser(mgl_gettext("Save File As?"), "*.svg", 0);
 	if(!fname || !fname[0])	return;
 	mgl_write_svg(((Fl_MGLView*)v)->FMGL->get_graph(),fname,0);
 }
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_export_eps_cb(Fl_Widget*, void* v)
 {
-	char *fname = fl_file_chooser(gettext("Save File As?"), "*.eps", 0);
+	char *fname = fl_file_chooser(mgl_gettext("Save File As?"), "*.eps", 0);
 	if(!fname || !fname[0])	return;
 	mgl_write_eps(((Fl_MGLView*)v)->FMGL->get_graph(),fname,0);
 }
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_export_prc_cb(Fl_Widget*, void* v)
 {
-	char *fname = fl_file_chooser(gettext("Save File As?"), "*.prc", 0);
+	char *fname = fl_file_chooser(mgl_gettext("Save File As?"), "*.prc", 0);
 	if(!fname || !fname[0])	return;
 	mgl_write_prc(((Fl_MGLView*)v)->FMGL->get_graph(),fname,0,1);
 }
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_export_tex_cb(Fl_Widget*, void* v)
 {
-	char *fname = fl_file_chooser(gettext("Save File As?"), "*.tex", 0);
+	char *fname = fl_file_chooser(mgl_gettext("Save File As?"), "*.tex", 0);
 	if(!fname || !fname[0])	return;
 	mgl_write_tex(((Fl_MGLView*)v)->FMGL->get_graph(),fname,0);
 }
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_export_obj_cb(Fl_Widget*, void* v)
 {
-	char *fname = fl_file_chooser(gettext("Save File As?"), "*.obj", 0);
+	char *fname = fl_file_chooser(mgl_gettext("Save File As?"), "*.obj", 0);
 	if(!fname || !fname[0])	return;
 	mgl_write_obj(((Fl_MGLView*)v)->FMGL->get_graph(),fname,0,1);
 }
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_export_off_cb(Fl_Widget*, void* v)
 {
-	char *fname = fl_file_chooser(gettext("Save File As?"), "*.off", 0);
+	char *fname = fl_file_chooser(mgl_gettext("Save File As?"), "*.off", 0);
 	if(!fname || !fname[0])	return;
 	mgl_write_off(((Fl_MGLView*)v)->FMGL->get_graph(),fname,0,0);
 }
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_export_stl_cb(Fl_Widget*, void* v)
 {
-	char *fname = fl_file_chooser(gettext("Save File As?"), "*.stl", 0);
+	char *fname = fl_file_chooser(mgl_gettext("Save File As?"), "*.stl", 0);
 	if(!fname || !fname[0])	return;
 	mgl_write_stl(((Fl_MGLView*)v)->FMGL->get_graph(),fname,0);
 }
@@ -541,108 +542,94 @@ void MGL_NO_EXPORT mgl_sshow_cb(Fl_Widget *, void *v)
 	if(e->is_sshow())	Fl::add_timeout(e->delay(e->par), mgl_time_cb, v);
 }
 void mglCanvasFL::Animation()	{	Fl::lock();	mgl_sshow_cb(0,mgl);	Fl::unlock();	}
-void MGL_NO_EXPORT mgl_no_cb(Fl_Widget *, void *)	{}
+void MGL_LOCAL_CONST mgl_no_cb(Fl_Widget *, void *)	{}
+//-----------------------------------------------------------------------------
+void MGL_NO_EXPORT mgl_stop_cb(Fl_Widget*, void* v)
+{	Fl_MGLView *e = (Fl_MGLView*)v;	if(e)	e->FMGL->stop();	}
 //-----------------------------------------------------------------------------
 Fl_Menu_Item pop_graph[20] = {
-	{ gettext("Export"), 0, mgl_no_cb, 0, FL_SUBMENU,0,0,0,0},
-		{ gettext("... as PNG"),	0, mgl_export_png_cb,0,0,0,0,0,0 },
-		{ gettext("... as PNG (solid)"),	0, mgl_export_pngn_cb,0,0,0,0,0,0 },
-		{ gettext("... as JPEG"),	0, mgl_export_jpeg_cb,0,0,0,0,0,0 },
-		{ gettext("... as SVG"),	0, mgl_export_svg_cb,0,0,0,0,0,0 },
-		{ gettext("... as vector EPS"),	0, mgl_export_eps_cb,0,0,0,0,0,0 },
-		{ gettext("... as bitmap EPS"),	0, mgl_export_bps_cb, 0, FL_MENU_DIVIDER,0,0,0,0 },
-		{ gettext("... as TeX"),	0, mgl_export_tex_cb,0,0,0,0,0,0 },
-		{ gettext("... as OBJ"),	0, mgl_export_obj_cb,0,0,0,0,0,0 },
-		{ gettext("... as PRC"),	0, mgl_export_prc_cb,0,0,0,0,0,0 },
-		{ gettext("... as OFF"),	0, mgl_export_off_cb,0,0,0,0,0,0 },
-		{ gettext("... as STL"),	0, mgl_export_stl_cb,0,0,0,0,0,0 },
+	{ mgl_gettext("Export"), 0, mgl_no_cb, 0, FL_SUBMENU,0,0,0,0},
+		{ mgl_gettext("... as PNG"),	0, mgl_export_png_cb,0,0,0,0,0,0 },
+		{ mgl_gettext("... as PNG (solid)"),	0, mgl_export_pngn_cb,0,0,0,0,0,0 },
+		{ mgl_gettext("... as JPEG"),	0, mgl_export_jpeg_cb,0,0,0,0,0,0 },
+		{ mgl_gettext("... as SVG"),	0, mgl_export_svg_cb,0,0,0,0,0,0 },
+		{ mgl_gettext("... as vector EPS"),	0, mgl_export_eps_cb,0,0,0,0,0,0 },
+		{ mgl_gettext("... as bitmap EPS"),	0, mgl_export_bps_cb, 0, FL_MENU_DIVIDER,0,0,0,0 },
+		{ mgl_gettext("... as TeX"),	0, mgl_export_tex_cb,0,0,0,0,0,0 },
+		{ mgl_gettext("... as OBJ"),	0, mgl_export_obj_cb,0,0,0,0,0,0 },
+		{ mgl_gettext("... as PRC"),	0, mgl_export_prc_cb,0,0,0,0,0,0 },
+		{ mgl_gettext("... as OFF"),	0, mgl_export_off_cb,0,0,0,0,0,0 },
+		{ mgl_gettext("... as STL"),	0, mgl_export_stl_cb,0,0,0,0,0,0 },
 		{ 0,0,0,0,0,0,0,0,0 },
-	{ gettext("Copy graphics"),	0, 0, 0, FL_MENU_INACTIVE|FL_MENU_DIVIDER,0,0,0,0},
-	{ gettext("Normal view"),	0, mgl_norm_cb,0,0,0,0,0,0 },
-	{ gettext("Redraw plot"),	0, mgl_draw_cb,0,0,0,0,0,0 },
-	{ gettext("Adjust size"),	0, mgl_adjust_cb,0,0,0,0,0,0 },
-	{ gettext("Reload data"),	0, mgl_oncemore_cb,0,0,0,0,0,0 },
+	{ mgl_gettext("Copy graphics"),	0, 0, 0, FL_MENU_INACTIVE|FL_MENU_DIVIDER,0,0,0,0},
+	{ mgl_gettext("Normal view"),	0, mgl_norm_cb,0,0,0,0,0,0 },
+	{ mgl_gettext("Redraw plot"),	0, mgl_draw_cb,0,0,0,0,0,0 },
+	{ mgl_gettext("Adjust size"),	0, mgl_adjust_cb,0,0,0,0,0,0 },
+	{ mgl_gettext("Reload data"),	0, mgl_oncemore_cb,0,0,0,0,0,0 },
 	{ 0,0,0,0,0,0,0,0,0 }
 };
 //-----------------------------------------------------------------------------
 Fl_MGLView::Fl_MGLView(int xx, int yy, int ww, int hh, const char *lbl) : Fl_Window(xx,yy,ww,hh,lbl)
 {
+	Fl_Button *o;
 	grid = alpha = light = sshow = 0;	menu = 0;
 	next = prev = reload = NULL;	delay = NULL;
 
-	Fl_Button *o;
-	Fl_Group *g = new Fl_Group(0,30,435,30);
-
+	Fl_Group *g = new Fl_Group(0,0,435,30);
 	alpha_bt = new Fl_Button(0, 1, 25, 25);	alpha_bt->type(FL_TOGGLE_BUTTON);
 	alpha_bt->image(xpm_a1);	alpha_bt->callback(mgl_alpha_cb,this);
-	alpha_bt->tooltip(gettext("Switch on/off transparency in the picture"));
-//	alpha_bt->box(FL_PLASTIC_UP_BOX);		alpha_bt->down_box(FL_PLASTIC_DOWN_BOX);
+	alpha_bt->tooltip(mgl_gettext("Switch on/off transparency in the picture"));
 	light_bt = new Fl_Button(25, 1, 25, 25);	light_bt->type(FL_TOGGLE_BUTTON);
 	light_bt->image(xpm_l1);	light_bt->callback(mgl_light_cb,this);
-	light_bt->tooltip(gettext("Switch on/off lightning in the picture"));
-//	light_bt->box(FL_PLASTIC_UP_BOX);		light_bt->down_box(FL_PLASTIC_DOWN_BOX);
+	light_bt->tooltip(mgl_gettext("Switch on/off lightning in the picture"));
 	grid_bt = new Fl_Button(50, 1, 25, 25);	grid_bt->type(FL_TOGGLE_BUTTON);
 	grid_bt->image(xpm_wire);	grid_bt->callback(mgl_grid_cb,this);
-	grid_bt->tooltip(gettext("Switch on/off grid drawing"));
-	//	grid_bt->box(FL_PLASTIC_UP_BOX);		grid_bt->down_box(FL_PLASTIC_DOWN_BOX);
+	grid_bt->tooltip(mgl_gettext("Switch on/off grid drawing"));
 
 	rotate_bt = new Fl_Button(80, 1, 25, 25);rotate_bt->type(FL_TOGGLE_BUTTON);
 	rotate_bt->image(xpm_r1);	rotate_bt->callback(mgl_rotate_cb,this);
-	rotate_bt->tooltip(gettext("Rotate picture by holding left mouse button"));
-//	rotate_bt->box(FL_PLASTIC_UP_BOX);		rotate_bt->down_box(FL_PLASTIC_DOWN_BOX);
+	rotate_bt->tooltip(mgl_gettext("Rotate picture by holding left mouse button"));
 	zoom_bt = new Fl_Button(105, 1, 25, 25);	zoom_bt->type(FL_TOGGLE_BUTTON);
 	zoom_bt->image(xpm_z1);	zoom_bt->callback(mgl_zoom_cb,this);
-	zoom_bt->tooltip(gettext("Zoom in selected region of the picture"));
-//	zoom_bt->box(FL_PLASTIC_UP_BOX);			zoom_bt->down_box(FL_PLASTIC_DOWN_BOX);
-	o = new Fl_Button(130, 1, 25, 25);		o->tooltip(gettext("Return picture to normal zoom"));
+	zoom_bt->tooltip(mgl_gettext("Zoom in selected region of the picture"));
+	o = new Fl_Button(130, 1, 25, 25);		o->tooltip(mgl_gettext("Return picture to normal zoom"));
 	o->image(new Fl_Pixmap(zoom_out_xpm));	o->callback(mgl_norm_cb,this);
-//	o->box(FL_PLASTIC_UP_BOX);	o->down_box(FL_PLASTIC_DOWN_BOX);
-
-	o = new Fl_Button(160, 1, 25, 25);	o->tooltip(gettext("Refresh the picture"));
+	o = new Fl_Button(160, 1, 25, 25);	o->tooltip(mgl_gettext("Stop drawing"));
+	o->image(new Fl_Pixmap(stop_xpm));	o->callback(mgl_stop_cb,this);
+	o = new Fl_Button(160, 1, 25, 25);	o->tooltip(mgl_gettext("Refresh the picture"));
 	o->image(new Fl_Pixmap(ok_xpm));	o->callback(mgl_draw_cb,this);
-//	o->box(FL_PLASTIC_UP_BOX);	o->down_box(FL_PLASTIC_DOWN_BOX);
 
 	Fl_Counter *tet, *phi;
 	tet = new Fl_Counter(195, 1, 90, 25, 0);	tet->callback(mgl_draw_cb,this);
 	phi = new Fl_Counter(290, 1, 90, 25, 0);	phi->callback(mgl_draw_cb,this);
 	tet->lstep(10);	tet->step(1);	tet->range(-180,180);
-	tet->tooltip(gettext("Theta angle (tilt z-axis)"));
+	tet->tooltip(mgl_gettext("Theta angle (tilt z-axis)"));
 	phi->lstep(10);	phi->step(1);	phi->range(-180,180);
-	phi->tooltip(gettext("Phi angle (rotate in x*y plane)"));
-//	tet->box(FL_PLASTIC_UP_BOX);	phi->box(FL_PLASTIC_UP_BOX);
+	phi->tooltip(mgl_gettext("Phi angle (rotate in x*y plane)"));
 	g->end();	g->resizable(0);
 
-	g = new Fl_Group(0,30,30,260);
-	o = new Fl_Button(1, 30, 25, 25);		o->tooltip(gettext("Shift the picture up"));
+	g = new Fl_Group(0,0,30,260);
+	o = new Fl_Button(1, 30, 25, 25);		o->tooltip(mgl_gettext("Shift the picture up"));
 	o->image(new Fl_Pixmap(up_1_xpm));		o->callback(mgl_su_cb,this);
 //	o->box(FL_PLASTIC_UP_BOX);	o->down_box(FL_PLASTIC_DOWN_BOX);
-	o = new Fl_Button(1, 55, 25, 25);		o->tooltip(gettext("Shift the picture left"));
+	o = new Fl_Button(1, 55, 25, 25);		o->tooltip(mgl_gettext("Shift the picture left"));
 	o->image(new Fl_Pixmap(left_1_xpm));	o->callback(mgl_sl_cb,this);
-//	o->box(FL_PLASTIC_UP_BOX);	o->down_box(FL_PLASTIC_DOWN_BOX);
-	o = new Fl_Button(1, 80, 25, 25);		o->tooltip(gettext("Zoom in the picture"));
+	o = new Fl_Button(1, 80, 25, 25);		o->tooltip(mgl_gettext("Zoom in the picture"));
 	o->image(new Fl_Pixmap(zoom_1_xpm));	o->callback(mgl_sz_cb,this);
-//	o->box(FL_PLASTIC_UP_BOX);	o->down_box(FL_PLASTIC_DOWN_BOX);
-	o = new Fl_Button(1, 105, 25, 25);		o->tooltip(gettext("Zoom out the picture"));
+	o = new Fl_Button(1, 105, 25, 25);		o->tooltip(mgl_gettext("Zoom out the picture"));
 	o->image(new Fl_Pixmap(norm_1_xpm));	o->callback(mgl_so_cb,this);
-//	o->box(FL_PLASTIC_UP_BOX);	o->down_box(FL_PLASTIC_DOWN_BOX);
-	o = new Fl_Button(1, 130, 25, 25);		o->tooltip(gettext("Shift the picture right"));
+	o = new Fl_Button(1, 130, 25, 25);		o->tooltip(mgl_gettext("Shift the picture right"));
 	o->image(new Fl_Pixmap(right_1_xpm));	o->callback(mgl_sr_cb,this);
-//	o->box(FL_PLASTIC_UP_BOX);	o->down_box(FL_PLASTIC_DOWN_BOX);
-	o = new Fl_Button(1, 155, 25, 25);		o->tooltip(gettext("Shift the picture down"));
+	o = new Fl_Button(1, 155, 25, 25);		o->tooltip(mgl_gettext("Shift the picture down"));
 	o->image(new Fl_Pixmap(down_1_xpm));	o->callback(mgl_sd_cb,this);
-//	o->box(FL_PLASTIC_UP_BOX);	o->down_box(FL_PLASTIC_DOWN_BOX);
 
-	o = new Fl_Button(1, 185, 25, 25);		o->tooltip(gettext("Show previous frame in slideshow"));
+	o = new Fl_Button(1, 185, 25, 25);		o->tooltip(mgl_gettext("Show previous frame in slideshow"));
 	o->image(new Fl_Pixmap(prev_sl_xpm));	o->callback(mgl_sprev_cb,this);
-//	o->box(FL_PLASTIC_UP_BOX);	o->down_box(FL_PLASTIC_DOWN_BOX);
 	anim_bt = new Fl_Button(1, 210, 25, 25);	anim_bt->type(FL_TOGGLE_BUTTON);
 	anim_bt->image(xpm_s1);	anim_bt->callback(mgl_sshow_cb,this);
-	anim_bt->tooltip(gettext("Run/Stop slideshow (graphics animation)"));
-//	anim_bt->box(FL_PLASTIC_UP_BOX);		anim_bt->down_box(FL_PLASTIC_DOWN_BOX);
-	o = new Fl_Button(1, 235, 25, 25);		o->tooltip(gettext("Show next frame in slideshow"));
+	anim_bt->tooltip(mgl_gettext("Run/Stop slideshow (graphics animation)"));
+	o = new Fl_Button(1, 235, 25, 25);		o->tooltip(mgl_gettext("Show next frame in slideshow"));
 	o->image(new Fl_Pixmap(next_sl_xpm));	o->callback(mgl_snext_cb,this);
-//	o->box(FL_PLASTIC_UP_BOX);	o->down_box(FL_PLASTIC_DOWN_BOX);
-
 	g->end();	g->resizable(0);
 
 	scroll = new Fl_Scroll(30, 30, 800, 600);
@@ -651,8 +638,7 @@ Fl_MGLView::Fl_MGLView(int xx, int yy, int ww, int hh, const char *lbl) : Fl_Win
 	FMGL->tet_val = tet;
 	FMGL->phi_val = phi;
 	FMGL->set_popup(pop_graph,FMGL,this);
-	scroll->end();
-	resizable(scroll);	end();
+	scroll->end();	resizable(scroll);	end();
 }
 Fl_MGLView::~Fl_MGLView()	{}
 //-----------------------------------------------------------------------------
@@ -674,7 +660,7 @@ void mglCanvasFL::GotoFrame(int d)
 void MGL_NO_EXPORT mgl_fl_next(void *v)	{	((mglCanvasWnd*)v)->NextFrame();	}	///< Callback function for next frame
 void MGL_NO_EXPORT mgl_fl_prev(void *v)	{	((mglCanvasWnd*)v)->PrevFrame();	}	///< Callback function for prev frame
 void MGL_NO_EXPORT mgl_fl_reload(void *v)	{	((mglCanvasWnd*)v)->ReLoad();	}		///< Callback function for reloading
-mreal MGL_NO_EXPORT mgl_fl_delay(void *v)	{	return ((mglCanvasWnd*)v)->GetDelay();	}	///< Callback function for delay
+mreal MGL_LOCAL_PURE mgl_fl_delay(void *v)	{	return ((mglCanvasWnd*)v)->GetDelay();	}	///< Callback function for delay
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_makemenu_fltk(Fl_Menu_ *m, Fl_MGLView *w)
 {
@@ -686,7 +672,7 @@ void MGL_EXPORT mgl_makemenu_fltk(Fl_Menu_ *m, Fl_MGLView *w)
 	m->add("Graphics/Redraw", "f5", mgl_draw_cb, w);
 	m->add("Graphics/Adjust size", "f6", mgl_adjust_cb, w);
 	m->add("Graphics/Reload data", "f9", mgl_oncemore_cb, w);
-	//TODO	m->add("Graphics/Stop", "f7", mgl_stop_cb, w);
+	m->add("Graphics/Stop", "f7", mgl_stop_cb, w);
 	//TODO	m->add("Graphics/Copy graphics","+^c", mgl_copyimg_cb, w);
 
 	m->add("Graphics/Export/as PNG", "#p", mgl_export_png_cb, w);
@@ -731,10 +717,9 @@ void mglCanvasFL::Window(int argc, char **argv, int (*draw)(mglBase *gr, void *p
 		Adjust();
 	}
 
-	char *tmp[1];	tmp[0]=new char[1];	tmp[0][0]=0;
-	Wnd->show(argv ? argc:0, argv ? argv:tmp);
-	delete []tmp[0];
-}
+	static char ctmp[1];	ctmp[0]=0;
+	static char *tmp[1];	tmp[0]=ctmp;
+	Wnd->show(argv ? argc:0, argv ? argv:tmp);}
 //-----------------------------------------------------------------------------
 HMGL MGL_EXPORT mgl_create_graph_fltk(int (*draw)(HMGL gr, void *p), const char *title, void *par, void (*load)(void *p))
 {
