@@ -36,6 +36,7 @@
 #include <QPrintDialog>
 #include <QFileDialog>
 #include <QInputDialog>
+#include <QThread>
 #include <limits.h>
 
 #include "mgl2/canvas_wnd.h"
@@ -57,6 +58,7 @@ using mglCanvasWnd::Window;
 	QMainWindow *Wnd;	///< Pointer to window
 
 	mglCanvasQT();
+    virtual ~mglCanvasQT() {	if(Wnd)	delete Wnd;	}
 
 	/// Create a window for plotting. Now implemeted only for GLUT.
 	void Window(int argc, char **argv, int (*draw)(mglBase *gr, void *p),const char *title,
@@ -87,9 +89,38 @@ void MGL_EXPORT mgl_ask_qt(const wchar_t *quest, wchar_t *res)
 //		class QMathGL
 //
 //-----------------------------------------------------------------------------
+/// Internal class to be used for multi-threading plotting
+/*class mglTask : public QObject
+{
+	Q_OBJECT
+public:
+	mglCanvas *gr;		///< Built-in mglCanvasQT-er instance (used by default)
+	void *draw_par;		///< Parameters for drawing function mglCanvasWnd::DrawFunc.
+	/// Drawing function for window procedure. It should return the number of frames.
+	int (*draw_func)(mglBase *gr, void *par);
+	mglDraw *draw;		///< Class for drawing -- need to call directly due to inheritance mechanism
+public slots:
+	void doWork();
+signals:
+	void plotDone();
+};
+//-----------------------------------------------------------------------------
+void mglTask::doWork()
+{
+	setlocale(LC_NUMERIC, "C");
+	if(mgl_is_frames(gr))	mgl_new_frame(gr);
+	if(draw_func)	draw_func(gr, draw_par);
+	else if(draw)	{	mglGraph g(gr);	draw->Draw(&g);	}
+	if(mgl_is_frames(gr))	mgl_end_frame(gr);
+	setlocale(LC_NUMERIC, "");
+	gr->Finish();
+	emit plotDone();
+}*/
+//-----------------------------------------------------------------------------
 QMathGL::QMathGL(QWidget *parent, Qt::WindowFlags f) : QWidget(parent, f)
 {
 	autoResize = false;	draw_par = 0;	draw_func = 0;
+	dotsRefr = true;
 	gr = new mglCanvas;	appName = "MathGL";
 	popup = 0;	grBuf = 0;	draw = 0;
 	phi = tet = per = 0;
@@ -97,19 +128,30 @@ QMathGL::QMathGL(QWidget *parent, Qt::WindowFlags f) : QWidget(parent, f)
 	alpha = light = zoom = rotate = grid = viewYZ = custZoom = custDraw = false;
 	resize(600, 400);	mgl_set_flag(gr, true, MGL_CLF_ON_UPD);
 	timer = new QTimer(this);
+	timerRefr = new QTimer(this);	timerRefr->setInterval(100);
+	timerRefr->setSingleShot(true);
 	enableWheel = enableMouse = true;
-//	resize(graph->GetWidth(), graph->GetHeight());
-//	mglConvertFromGraph(pic, graph, &grBuf);
 	connect(timer, SIGNAL(timeout()), this, SLOT(nextSlide()));
+	connect(timerRefr, SIGNAL(timeout()), this, SLOT(refreshHQ()));
+
+/*	thread = new QThread();
+	task = new mglTask();	task->moveToThread(thread);
+	connect(thread, SIGNAL(started()), task, SLOT(doWork()));
+	connect(task, SIGNAL(plotDone()), thread, SLOT(quit()));
+	connect(thread, SIGNAL(finished()), this, SLOT(afterPlot()));*/
 }
 //-----------------------------------------------------------------------------
 QMathGL::~QMathGL()
 {
+	timer->stop();	timerRefr->stop();
 	if(mgl_use_graph(gr,-1)<1)	mgl_delete_graph(gr);
 	if(grBuf)	delete []grBuf;
 	if(draw)	delete draw;
 }
 //-----------------------------------------------------------------------------
+void QMathGL::setDotsPreview(bool d)
+{	dotsRefr = d;	}
+//-----------------------------------------------------------------------------
 void QMathGL::setDraw(int (*func)(mglBase *gr, void *par), void *par)
 {
 	if(draw)	delete draw;	draw = 0;
@@ -125,6 +167,7 @@ void QMathGL::setDraw(mglDraw *dr)
 }
 //-----------------------------------------------------------------------------
 double QMathGL::getRatio()	{	return double(mgl_get_width(gr))/mgl_get_height(gr);	}
+void mgl_qt_event_func(void *)	{	QApplication::processEvents();	}
 //-----------------------------------------------------------------------------
 void QMathGL::setGraph(HMGL GR)	///< Set grapher object
 {
@@ -132,6 +175,7 @@ void QMathGL::setGraph(HMGL GR)	///< Set grapher object
 	if(!gg)	return;
 	if(mgl_use_graph(gr,-1)<1)	mgl_delete_graph(gr);
 	gr=gg;	mgl_use_graph(gg,1);
+	gr->SetEventFunc(mgl_qt_event_func, NULL);
 }
 //-----------------------------------------------------------------------------
 void QMathGL::paintEvent(QPaintEvent *)
@@ -257,46 +301,47 @@ void QMathGL::restore()
 	else refresh();
 }
 //-----------------------------------------------------------------------------
-void QMathGL::stop()	{	gr->Stop=true;	}	//{	thr->terminate();	}
+void QMathGL::stop()	{	gr->AskStop(true);	}
+//-----------------------------------------------------------------------------
 void QMathGL::update()
 {
-/*	if(!thr->isRunning())
-	{
-		thr->gr = gr;	thr->text = text;
-		thr->line = line;	thr->start();
-	}
-	if(rotate)	thr->wait();
-	else	while(thr->isRunning())	qApp->processEvents();
-	if(warnMGL)	warnMGL->setText(thr->warn);*/
-
 	if(draw_func || draw)
 	{
 		mgl_reset_frames(gr);	// remove previous frames
 		if(mgl_get_flag(gr,MGL_CLF_ON_UPD))	mgl_set_def_param(gr);
 		mgl_set_alpha(gr,alpha);	mgl_set_light(gr,light);
-		if(!isHidden())	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-		setlocale(LC_NUMERIC, "C");
 		// use frames for quickly redrawing while adding/changing primitives
 		if(custDraw)	emit customDraw(x1,y1,x2,y2,true);
+
+		if(!isHidden())	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+/*		task->gr = gr;	task->draw = draw;
+		task->draw_func = draw_func;
+		task->draw_par = draw_par;
+		thread->start();*/
+		setlocale(LC_NUMERIC, "C");
 		if(mgl_is_frames(gr))	mgl_new_frame(gr);
 		if(draw_func)	draw_func(gr, draw_par);
 		else if(draw)	{	mglGraph g(gr);	draw->Draw(&g);	}
 		if(mgl_is_frames(gr))	mgl_end_frame(gr);
 		setlocale(LC_NUMERIC, "");
-		if(!isHidden())	QApplication::restoreOverrideCursor();
-		emit refreshData();
-		emit showWarn(mgl_get_mess(gr));
-		mousePos="";
+		gr->AskStop(false);	afterPlot();
 	}
+}
+//-----------------------------------------------------------------------------
+void QMathGL::afterPlot()
+{
+	emit refreshData();
+	emit showWarn(mgl_get_mess(gr));
+	mousePos="";
+	if(!isHidden())	QApplication::restoreOverrideCursor();
 	refresh();
 }
 //-----------------------------------------------------------------------------
-void QMathGL::draw_thr()
+void QMathGL::drawPrim()
 {
-	mgl_clf(gr);
+	if(!gr)	return;
 	mglCanvasWnd *g = dynamic_cast<mglCanvasWnd *>(gr);
-	int c = g?g->GetCurFig():mgl_get_num_frame(gr)-1;
-	mgl_get_frame(gr, c);
+	mgl_get_frame(gr, g?g->GetCurFig():mgl_get_num_frame(gr)-1);
 	mglParse pr;
 	long i, n=primitives.count('\n');
 	mglGraph gg(gr);
@@ -313,27 +358,24 @@ void QMathGL::draw_thr()
 	gg.SetRanges(ox1,ox2);	gg.Pop();	setlocale(LC_NUMERIC, "");
 }
 //-----------------------------------------------------------------------------
-void *mgl_qt_thr(void *p)	{	QMathGL *q = (QMathGL *)p;	q->draw_thr();	return 0;	}
 void QMathGL::refresh()
 {
+	if(dotsRefr)
+	{
+		timerRefr->start();
+		int q = gr->GetQuality();
+		prevQuality = q!=MGL_DRAW_DOTS?q:prevQuality;
+		gr->SetQuality(MGL_DRAW_DOTS);
+	}
 	if(mgl_is_frames(gr) && mgl_get_num_frame(gr)>0)
 	{
-		if(draw_func || draw)
-		{
-/*#if MGL_HAVE_PTHREAD	// TODO add qthread here later
-			pthread_t tmp;
-			pthread_create(&tmp, 0, mgl_qt_thr, this);
-			pthread_join(tmp, 0);
-#else*/
-			draw_thr();
-/*#endif*/
-		}
+		drawPrim();
 		if(custZoom)	emit customZoom(x1,y1,x2,y2,tet,phi,per);
 		else
 		{	mgl_zoom(gr,x1,y1,x2,y2);
-			mgl_perspective(gr,per);
-			if(viewYZ)	mgl_view(gr,0,phi,tet);
-			else 		mgl_view(gr,phi,0,tet);
+			mgl_ask_perspective(gr,per);
+			if(viewYZ)	mgl_view(gr,0,-tet,-phi);
+			else 		mgl_view(gr,-phi,-tet,0);
 		}
 	}
 	mglConvertFromGraph(pic, gr, &grBuf);
@@ -341,6 +383,25 @@ void QMathGL::refresh()
 	repaint();
 }
 //-----------------------------------------------------------------------------
+void QMathGL::refreshHQ()
+{
+	gr->SetQuality(prevQuality);
+	if(mgl_is_frames(gr) && mgl_get_num_frame(gr)>0)
+	{
+		drawPrim();
+		if(custZoom)	emit customZoom(x1,y1,x2,y2,tet,phi,per);
+		else
+		{	mgl_zoom(gr,x1,y1,x2,y2);
+			mgl_ask_perspective(gr,per);
+			if(viewYZ)	mgl_view(gr,0,-tet,-phi);
+			else 		mgl_view(gr,-phi,-tet,0);
+		}
+	}
+	mglConvertFromGraph(pic, gr, &grBuf, &img);
+	if(pic.size()!=size())	setSize(pic.width(), pic.height());
+	repaint();
+}
+//-----------------------------------------------------------------------------
 void QMathGL::mousePressEvent(QMouseEvent *ev)
 {
 	if(!zoom && !rotate && ev->button()&Qt::LeftButton)
@@ -489,6 +550,13 @@ void QMathGL::mouseMoveEvent(QMouseEvent *ev)
 					else	dr = hypot(dx,dy);
 					res = tst.section(' ',0,4)+" "+QString::number(tst.section(' ',5,5).toFloat()+dr)+" "+tst.section(' ',6);
 				}
+				else if(cmd=="arc")
+				{
+					double x_=tst.section(' ',3,3).toFloat()-xx, y_=tst.section(' ',4,4).toFloat()-yy, a_=tst.section(' ',5,5).toFloat();
+					double c=cos(M_PI*a_/180), s=sin(M_PI*a_/180);
+					double a = atan2(x_,y_) - atan2(x_*c-y_*s+dx,x_*s+y_*c+dy);
+					res = tst.section(' ',0,4)+" "+QString::number(a*180/M_PI)+" "+tst.section(' ',6);
+				}
 				else if(p.n==2)
 				{
 					xx=tst.section(' ',5,5).toFloat();	yy=tst.section(' ',6,6).toFloat();
@@ -510,7 +578,7 @@ void QMathGL::mouseMoveEvent(QMouseEvent *ev)
 			QString tst = primitives.section('\n',id,id), cmd=tst.section(' ',0,0), res;
 			float dx = 2*(xe-x0)/float(w), dy = 2*(y0-ye)/float(h);
 			float x1=tst.section(' ',1,1).toFloat(), y1=tst.section(' ',2,2).toFloat(),x2,y2;
-			if(cmd=="ball" || cmd=="text")
+			if(cmd=="ball")
 				res = cmd+" "+QString::number(x1+dx)+" "+QString::number(y1+dy)+" "+tst.section(' ',3);
 			else if(cmd=="curve")
 			{
@@ -595,28 +663,48 @@ void QMathGL::exportGIF(QString fname)
 {
 	if(fname.isEmpty())	fname = mgl_get_plotid(gr);
 	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename."),QMessageBox::Ok,0,0);
-	else	mgl_write_gif(gr,setExtension(fname,"png").toStdString().c_str(), appName.toStdString().c_str());
+	else
+#if MGL_HAVE_GIF
+		mgl_write_gif(gr,setExtension(fname,"png").toStdString().c_str(), appName.toStdString().c_str());
+#else
+		img.save(setExtension(fname,"gif"));
+#endif
 }
 //-----------------------------------------------------------------------------
 void QMathGL::exportPNG(QString fname)
 {
 	if(fname.isEmpty())	fname = mgl_get_plotid(gr);
 	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename."),QMessageBox::Ok,0,0);
-	else	mgl_write_png(gr,setExtension(fname,"png").toStdString().c_str(), appName.toStdString().c_str());
+	else
+#if MGL_HAVE_PNG
+		mgl_write_png(gr,setExtension(fname,"png").toStdString().c_str(), appName.toStdString().c_str());
+#else
+		img.save(setExtension(fname,"png"));
+#endif
 }
 //-----------------------------------------------------------------------------
 void QMathGL::exportPNGs(QString fname)
 {
 	if(fname.isEmpty())	fname = mgl_get_plotid(gr);
 	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename."),QMessageBox::Ok,0,0);
-	else	mgl_write_png_solid(gr,setExtension(fname,"png").toStdString().c_str(), appName.toStdString().c_str());
+	else
+#if MGL_HAVE_PNG
+		mgl_write_png_solid(gr,setExtension(fname,"png").toStdString().c_str(), appName.toStdString().c_str());
+#else
+		img.save(setExtension(fname,"png"));
+#endif
 }
 //-----------------------------------------------------------------------------
 void QMathGL::exportJPG(QString fname)
 {
 	if(fname.isEmpty())	fname = mgl_get_plotid(gr);
 	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename."),QMessageBox::Ok,0,0);
-	else	mgl_write_jpg(gr,setExtension(fname,"jpg").toStdString().c_str(), appName.toStdString().c_str());
+	else
+#if MGL_HAVE_JPEG
+		mgl_write_jpg(gr,setExtension(fname,"jpg").toStdString().c_str(), appName.toStdString().c_str());
+#else
+		img.save(setExtension(fname,"jpg"));
+#endif
 }
 //-----------------------------------------------------------------------------
 void QMathGL::exportBPS(QString fname)
@@ -763,7 +851,7 @@ void QMathGL::exportMGLD(QString fname)
 	}
 }
 //-----------------------------------------------------------------------------
-void mglConvertFromGraph(QPixmap &pic, mglCanvas *gr, uchar **buf)
+void mglConvertFromGraph(QPixmap &pic, mglCanvas *gr, uchar **buf, QImage *out)
 {
 	const uchar *bb = mgl_get_rgb(gr);
 	register long i,w=mgl_get_width(gr), h=mgl_get_height(gr);
@@ -777,6 +865,7 @@ void mglConvertFromGraph(QPixmap &pic, mglCanvas *gr, uchar **buf)
 		(*buf)[4*i+3] = 255;
 	}
 	QImage img(*buf, w, h, QImage::Format_RGB32);
+	if(out)	*out = img;
 	pic = QPixmap::fromImage(img);
 }
 //-----------------------------------------------------------------------------
@@ -849,7 +938,7 @@ void QMathGL::adjust()
 {
 	mgl_set_size(gr, parentWidget()->width()-3, parentWidget()->height()-3);
 	setSize(parentWidget()->width()-3, parentWidget()->height()-3);
-	update();
+	update();	// TODO replace to refresh ?!?
 }
 //-----------------------------------------------------------------------------
 void QMathGL::addMark()
@@ -870,12 +959,24 @@ void QMathGL::addRhomb()
 void QMathGL::addEllipse()
 {	primitives += "ellipse -0.2 0 0.2 0 0.1 'r'\n";	refresh();	}
 //-----------------------------------------------------------------------------
+void QMathGL::addArc()
+{	primitives += "arc 0 0 0.2 0 60 'r2'\n";	refresh();	}
+//-----------------------------------------------------------------------------
+void QMathGL::addPolygon(int n)
+{
+	if(n<3)
+		n = QInputDialog::getText(QApplication::activeWindow(), "MathGL", tr("Enter number of vertexes")).toInt();
+	if(n>=3)
+	{	primitives += "polygon 0 0 0 0.2 "+QString::number(n)+" 'r'\n";	refresh();	}
+}
+//{	primitives += "arc -0.2 0 0.2 0 0.1 'r'\n";	refresh();	}
+//-----------------------------------------------------------------------------
 void QMathGL::addText(QString txt)
 {
 	if(txt.isEmpty())
 		txt = QInputDialog::getText(QApplication::activeWindow(), "MathGL", tr("Enter text"));
 	if(!txt.isEmpty())
-	{	primitives += "text 0 0 '"+txt+"' ''\n";	refresh();	}
+	{	primitives += "text 0 0 0.1 0 '"+txt+"' ''\n";	refresh();	}
 }
 //-----------------------------------------------------------------------------
 //
@@ -928,15 +1029,10 @@ void mglCanvasQT::Window(int argc, char **argv, int (*draw)(mglBase *gr, void *p
 		// must be placed before ANY window creation
 		XInitThreads();
 #endif
-		QApplication *a;
-		if(!argv)
-		{
-			static char tmp[2][1];
-			tmp[0][0]=tmp[1][0]=0;
-			static int aa=1;
-			a = new QApplication(aa, (char **)tmp);
-		}
-		else	a = new QApplication(argc, argv);
+//		static char buf=0, *tmp=&buf;
+//		if(!argv)	{	argc = 1;	argv = &tmp;	}
+		if(!argv)	argc = 0;
+		QApplication *a = new QApplication(argc, argv);
 		a->connect(a, SIGNAL(lastWindowClosed()), a, SLOT(quit()));
 	}
 
@@ -957,33 +1053,36 @@ void mglCanvasQT::Window(int argc, char **argv, int (*draw)(mglBase *gr, void *p
 	else	Wnd->showMaximized();
 }
 //-----------------------------------------------------------------------------
-#include "xpm/fileprint.xpm"
-#include "xpm/copy.xpm"
-#include "xpm/left_1.xpm"
-#include "xpm/right_1.xpm"
-#include "xpm/down_1.xpm"
-#include "xpm/norm_1.xpm"
-#include "xpm/zoom_1.xpm"
-#include "xpm/up_1.xpm"
 #include "xpm/alpha.xpm"
-#include "xpm/light.xpm"
-#include "xpm/zoom_in.xpm"
-#include "xpm/zoom_out.xpm"
-#include "xpm/rotate.xpm"
-#include "xpm/ok.xpm"
-#include "xpm/show_sl.xpm"
-#include "xpm/next_sl.xpm"
-#include "xpm/prev_sl.xpm"
-#include "xpm/text.xpm"
-#include "xpm/line.xpm"
+#include "xpm/arc.xpm"
+#include "xpm/copy.xpm"
 #include "xpm/curve.xpm"
-#include "xpm/mark_o.xpm"
-#include "xpm/mark_s.xpm"
+#include "xpm/down_1.xpm"
+#include "xpm/fileprint.xpm"
+#include "xpm/left_1.xpm"
+#include "xpm/light.xpm"
+#include "xpm/line.xpm"
 #include "xpm/mark_a.xpm"
 #include "xpm/mark_d.xpm"
+#include "xpm/mark_o.xpm"
+#include "xpm/mark_s.xpm"
+#include "xpm/right_1.xpm"
+#include "xpm/next_sl.xpm"
+#include "xpm/norm_1.xpm"
+#include "xpm/ok.xpm"
+#include "xpm/prev_sl.xpm"
+#include "xpm/rotate.xpm"
+#include "xpm/show_sl.xpm"
+#include "xpm/text.xpm"
+#include "xpm/polygon.xpm"
+#include "xpm/zoom_1.xpm"
+#include "xpm/zoom_in.xpm"
+#include "xpm/zoom_out.xpm"
+#include "xpm/up_1.xpm"
+#include "xpm/stop.xpm"
 //-----------------------------------------------------------------------------
 #define TR	QObject::tr
-QMenu *mglMakeMenu(QMainWindow *Wnd, QMathGL *QMGL, QSpinBox *&tet, QSpinBox *&phi)
+MGL_EXPORT QMenu *mglMakeMenu(QMainWindow *Wnd, QMathGL *QMGL, QSpinBox *&tet, QSpinBox *&phi)
 {
 	QAction *a;
 	QMenu *o, *oo;
@@ -1055,11 +1154,16 @@ QMenu *mglMakeMenu(QMainWindow *Wnd, QMathGL *QMGL, QSpinBox *&tet, QSpinBox *&p
 		a->setShortcut(Qt::ALT+Qt::Key_Space);
 		o->addAction(a);	bb->addAction(a);	popup->addAction(a);
 		bb->addSeparator();
+		o->addAction(a);	bb->addAction(a);	popup->addAction(a);
 		a = new QAction(QPixmap(ok_xpm), TR("Re&draw"), Wnd);
 		Wnd->connect(a, SIGNAL(triggered()), QMGL, SLOT(update()));
 		a->setToolTip(TR("Execute script and redraw graphics (F5)."));
 		a->setShortcut(Qt::Key_F5);
 		o->addAction(a);	bb->addAction(a);	popup->addAction(a);
+		a = new QAction(QPixmap(stop_xpm), TR("Stop"), Wnd);
+		Wnd->connect(a, SIGNAL(triggered()), QMGL, SLOT(stop()));
+		a->setToolTip(TR("Ask to stop plot drawing (F7)."));
+		a->setShortcut(Qt::Key_F7);
 		a = new QAction(TR("&Adjust size"), Wnd);
 		Wnd->connect(a, SIGNAL(triggered()), QMGL, SLOT(adjust()));
 		a->setToolTip(TR("Change canvas size to fill whole region (F6)."));
@@ -1077,6 +1181,11 @@ QMenu *mglMakeMenu(QMainWindow *Wnd, QMathGL *QMGL, QSpinBox *&tet, QSpinBox *&p
 		Wnd->connect(QMGL, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
 		a->setToolTip(TR("Add line which properties can be changed later by mouse."));
 		bb->addAction(a);	oo->addAction(a);
+		a = new QAction(QPixmap(arc_xpm), TR("Add arc"), Wnd);
+		Wnd->connect(a, SIGNAL(triggered()), QMGL, SLOT(addArc()));
+		Wnd->connect(QMGL, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
+		a->setToolTip(TR("Add arc which properties can be changed later by mouse."));
+		bb->addAction(a);	oo->addAction(a);
 		a = new QAction(QPixmap(curve_xpm), TR("Add curve"), Wnd);
 		Wnd->connect(a, SIGNAL(triggered()), QMGL, SLOT(addCurve()));
 		Wnd->connect(QMGL, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
@@ -1097,6 +1206,11 @@ QMenu *mglMakeMenu(QMainWindow *Wnd, QMathGL *QMGL, QSpinBox *&tet, QSpinBox *&p
 		Wnd->connect(QMGL, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
 		a->setToolTip(TR("Add ellipse which properties can be changed later by mouse."));
 		bb->addAction(a);	oo->addAction(a);
+		a = new QAction(QPixmap(polygon_xpm), TR("Add polygon"), Wnd);
+		Wnd->connect(a, SIGNAL(triggered()), QMGL, SLOT(addPolygon()));
+		Wnd->connect(QMGL, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
+		a->setToolTip(TR("Add polygon which properties can be changed later by mouse."));
+		bb->addAction(a);	oo->addAction(a);
 		a = new QAction(QPixmap(mark_a_xpm), TR("Add mark"), Wnd);
 		Wnd->connect(a, SIGNAL(triggered()), QMGL, SLOT(addMark()));
 		Wnd->connect(QMGL, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
